@@ -9,7 +9,7 @@
     This module shouldn't be used directly.
 
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
 import os
 import sys
@@ -19,6 +19,9 @@ import weakref
 from celery.local import Proxy
 from celery.utils.threads import LocalStack
 
+__all__ = ['set_default_app', 'get_current_app', 'get_current_task',
+           'get_current_worker_task', 'current_app', 'current_task']
+
 #: Global default app used when no current app.
 default_app = None
 
@@ -53,10 +56,11 @@ def _get_current_app():
     return _tls.current_app or default_app
 
 C_STRICT_APP = os.environ.get('C_STRICT_APP')
-if os.environ.get('C_STRICT_APP'):
+if os.environ.get('C_STRICT_APP'):  # pragma: no cover
     def get_current_app():
+        raise Exception('USES CURRENT APP')
         import traceback
-        sys.stderr.write('USES CURRENT_APP\n')
+        print('-- USES CURRENT_APP', file=sys.stderr)  # noqa+
         traceback.print_stack(file=sys.stderr)
         return _get_current_app()
 else:
