@@ -7,22 +7,29 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import os
 
+from collections import Callable
+
 from celery.local import Proxy
 from celery import _state
-from celery._state import (  # noqa
+from celery._state import (
     set_default_app,
     get_current_app as current_app,
     get_current_task as current_task,
     _get_active_apps,
+    _task_stack,
 )
 from celery.utils import gen_task_name
 
 from .builtins import shared_task as _shared_task
-from .base import Celery, AppPickler  # noqa
+from .base import Celery, AppPickler
+
+__all__ = ['Celery', 'AppPickler', 'default_app', 'app_or_default',
+           'bugreport', 'enable_trace', 'disable_trace', 'shared_task',
+           'set_default_app', 'current_app', 'current_task',
+           'push_current_task', 'pop_current_task']
 
 #: Proxy always returning the app set as default.
 default_app = Proxy(lambda: _state.default_app)
@@ -40,8 +47,18 @@ app_or_default = None
 default_loader = os.environ.get('CELERY_LOADER') or 'default'  # XXX
 
 
-def bugreport():
-    return current_app().bugreport()
+#: Function used to push a task to the thread local stack
+#: keeping track of the currently executing task.
+#: You must remember to pop the task after.
+push_current_task = _task_stack.push
+
+#: Function used to pop a task from the thread local stack
+#: keeping track of the currently executing task.
+pop_current_task = _task_stack.pop
+
+
+def bugreport(app=None):
+    return (app or current_app()).bugreport()
 
 
 def _app_or_default(app=None):
@@ -84,8 +101,8 @@ App = Celery  # XXX Compat
 
 
 def shared_task(*args, **kwargs):
-    """Task decorator that creates shared tasks,
-    and returns a proxy that always returns the task from the current apps
+    """Create shared tasks (decorator).
+    Will return a proxy that always takes the task from the current apps
     task registry.
 
     This can be used by library authors to create tasks that will work
@@ -121,7 +138,7 @@ def shared_task(*args, **kwargs):
                     with app._finalize_mutex:
                         app._task_from_fun(fun, **options)
 
-            # Returns a proxy that always gets the task from the current
+            # Return a proxy that always gets the task from the current
             # apps task registry.
             def task_by_cons():
                 app = current_app()
@@ -131,6 +148,6 @@ def shared_task(*args, **kwargs):
             return Proxy(task_by_cons)
         return __inner
 
-    if len(args) == 1 and callable(args[0]):
+    if len(args) == 1 and isinstance(args[0], Callable):
         return create_shared_task(**kwargs)(args[0])
     return create_shared_task(*args, **kwargs)
