@@ -7,36 +7,59 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import os
 import threading
 import warnings
 
-from collections import deque
+from collections import Callable, defaultdict, deque
 from contextlib import contextmanager
 from copy import deepcopy
-from functools import wraps
+from operator import attrgetter
 
 from billiard.util import register_after_fork
 from kombu.clocks import LamportClock
-from kombu.utils import cached_property
+from kombu.common import oid_from
+from kombu.utils import cached_property, uuid
 
 from celery import platforms
+from celery._state import (
+    _task_stack, _tls, get_current_app, _register_app, get_current_worker_task,
+)
 from celery.exceptions import AlwaysEagerIgnored, ImproperlyConfigured
+from celery.five import items, values
 from celery.loaders import get_loader_cls
 from celery.local import PromiseProxy, maybe_evaluate
-from celery._state import _task_stack, _tls, get_current_app, _register_app
 from celery.utils.functional import first, maybe_list
 from celery.utils.imports import instantiate, symbol_by_name
+from celery.utils.log import ensure_process_aware_logger
+from celery.utils.objects import mro_lookup
 
 from .annotations import prepare as prepare_annotations
 from .builtins import shared_task, load_shared_tasks
 from .defaults import DEFAULTS, find_deprecated_settings
 from .registry import TaskRegistry
-from .utils import AppPickler, Settings, bugreport, _unpickle_app
+from .utils import (
+    AppPickler, Settings, bugreport, _unpickle_app, _unpickle_app_v2, appstr,
+)
+
+__all__ = ['Celery']
 
 _EXECV = os.environ.get('FORKED_BY_MULTIPROCESSING')
+BUILTIN_FIXUPS = frozenset([
+    'celery.fixups.django:fixup',
+])
+
+ERR_ENVVAR_NOT_SET = """\
+The environment variable {0!r} is not set,
+and as such the configuration could not be loaded.
+Please set this variable and make it point to
+a configuration module."""
+
+
+def app_has_custom(app, attr):
+    return mro_lookup(app.__class__, attr, stop=(Celery, object),
+                      monkey_patched=[__name__])
 
 
 def _unpickle_appattr(reverse_name, args):
@@ -46,6 +69,7 @@ def _unpickle_appattr(reverse_name, args):
 
 
 class Celery(object):
+    #: This is deprecated, use :meth:`reduce_keys` instead
     Pickler = AppPickler
 
     SYSTEM = platforms.SYSTEM
@@ -57,6 +81,7 @@ class Celery(object):
     loader_cls = 'celery.loaders.app:AppLoader'
     log_cls = 'celery.app.log:Logging'
     control_cls = 'celery.app.control:Control'
+    task_cls = 'celery.app.task:Task'
     registry_cls = TaskRegistry
     _pool = None
 
@@ -64,8 +89,7 @@ class Celery(object):
                  amqp=None, events=None, log=None, control=None,
                  set_as_current=True, accept_magic_kwargs=False,
                  tasks=None, broker=None, include=None, changes=None,
-                 config_source=None,
-                 **kwargs):
+                 config_source=None, fixups=None, task_cls=None, **kwargs):
         self.clock = LamportClock()
         self.main = main
         self.amqp_cls = amqp or self.amqp_cls
@@ -74,10 +98,13 @@ class Celery(object):
         self.loader_cls = loader or self.loader_cls
         self.log_cls = log or self.log_cls
         self.control_cls = control or self.control_cls
+        self.task_cls = task_cls or self.task_cls
         self.set_as_current = set_as_current
         self.registry_cls = symbol_by_name(self.registry_cls)
         self.accept_magic_kwargs = accept_magic_kwargs
+        self.user_options = defaultdict(set)
         self._config_source = config_source
+        self.steps = defaultdict(set)
 
         self.configured = False
         self._pending_defaults = deque()
@@ -89,6 +116,11 @@ class Celery(object):
         if not isinstance(self._tasks, TaskRegistry):
             self._tasks = TaskRegistry(self._tasks or {})
 
+        # If the class defins a custom __reduce_args__ we need to use
+        # the old way of pickling apps, which is pickling a list of
+        # args instead of the new way that pickles a dict of keywords.
+        self._using_v1_reduce = app_has_custom(self, '__reduce_args__')
+
         # these options are moved to the config to
         # simplify pickling of the app object.
         self._preconf = changes or {}
@@ -97,6 +129,11 @@ class Celery(object):
         if include:
             self._preconf['CELERY_IMPORTS'] = include
 
+        # Apply fixups.
+        self.fixups = set(fixups or ())
+        for fixup in self.fixups | BUILTIN_FIXUPS:
+            symbol_by_name(fixup)(self)
+
         if self.set_as_current:
             self.set_current()
 
@@ -133,7 +170,7 @@ class Celery(object):
 
     def worker_main(self, argv=None):
         return instantiate(
-            'celery.bin.celeryd:WorkerCommand',
+            'celery.bin.worker:worker',
             app=self).execute_from_commandline(argv)
 
     def task(self, *args, **opts):
@@ -145,8 +182,7 @@ class Celery(object):
             # the task instance from the current app.
             # Really need a better solution for this :(
             from . import shared_task as proxies_to_curapp
-            opts['_force_evaluate'] = True  # XXX Py2.5
-            return proxies_to_curapp(*args, **opts)
+            return proxies_to_curapp(*args, _force_evaluate=True, **opts)
 
         def inner_create_task_cls(shared=True, filter=None, **opts):
             _filt = filter  # stupid 2to3
@@ -165,7 +201,7 @@ class Celery(object):
                 if self.finalized or opts.get('_force_evaluate'):
                     ret = self._task_from_fun(fun, **opts)
                 else:
-                    # return a proxy object that is only evaluated on first use
+                    # return a proxy object that evaluates on first use
                     ret = PromiseProxy(self._task_from_fun, (fun, ), opts)
                     self._pending.append(ret)
                 if _filt:
@@ -174,7 +210,7 @@ class Celery(object):
 
             return _create_task_cls
 
-        if len(args) == 1 and callable(args[0]):
+        if len(args) == 1 and isinstance(args[0], Callable):
             return inner_create_task_cls(**opts)(*args)
         if args:
             raise TypeError(
@@ -183,15 +219,16 @@ class Celery(object):
 
     def _task_from_fun(self, fun, **options):
         base = options.pop('base', None) or self.Task
+        bind = options.pop('bind', False)
 
         T = type(fun.__name__, (base, ), dict({
             'app': self,
             'accept_magic_kwargs': False,
-            'run': staticmethod(fun),
+            'run': fun if bind else staticmethod(fun),
+            '_decorated': True,
             '__doc__': fun.__doc__,
             '__module__': fun.__module__}, **options))()
         task = self._tasks[T.name]  # return global instance.
-        task.bind(self)
         return task
 
     def finalize(self):
@@ -204,11 +241,11 @@ class Celery(object):
                 while pending:
                     maybe_evaluate(pending.popleft())
 
-                for task in self._tasks.itervalues():
+                for task in values(self._tasks):
                     task.bind(self)
 
     def add_defaults(self, fun):
-        if not callable(fun):
+        if not isinstance(fun, Callable):
             d, fun = fun, lambda: d
         if self.configured:
             return self.conf.add_defaults(fun())
@@ -224,56 +261,83 @@ class Celery(object):
         if not module_name:
             if silent:
                 return False
-            raise ImproperlyConfigured(self.error_envvar_not_set % module_name)
+            raise ImproperlyConfigured(ERR_ENVVAR_NOT_SET.format(module_name))
         return self.config_from_object(module_name, silent=silent)
 
     def config_from_cmdline(self, argv, namespace='celery'):
         self.conf.update(self.loader.cmdline_config_parser(argv, namespace))
 
+    def setup_security(self, allowed_serializers=None, key=None, cert=None,
+                       store=None, digest='sha1', serializer='json'):
+        from celery.security import setup_security
+        return setup_security(allowed_serializers, key, cert,
+                              store, digest, serializer, app=self)
+
+    def autodiscover_tasks(self, packages, related_name='tasks'):
+        if self.conf.CELERY_FORCE_BILLIARD_LOGGING:
+            # we'll use billiard's processName instead of
+            # multiprocessing's one in all the loggers
+            # created after this call
+            ensure_process_aware_logger()
+
+        self.loader.autodiscover_tasks(packages, related_name)
+
     def send_task(self, name, args=None, kwargs=None, countdown=None,
                   eta=None, task_id=None, producer=None, connection=None,
-                  result_cls=None, expires=None, queues=None, publisher=None,
-                  link=None, link_error=None,
-                  **options):
+                  router=None, result_cls=None, expires=None,
+                  publisher=None, link=None, link_error=None,
+                  add_to_parent=True, reply_to=None, **options):
+        task_id = task_id or uuid()
         producer = producer or publisher  # XXX compat
-        if self.conf.CELERY_ALWAYS_EAGER:  # pragma: no cover
+        router = router or self.amqp.router
+        conf = self.conf
+        if conf.CELERY_ALWAYS_EAGER:  # pragma: no cover
             warnings.warn(AlwaysEagerIgnored(
                 'CELERY_ALWAYS_EAGER has no effect on send_task'))
-
-        result_cls = result_cls or self.AsyncResult
-        router = self.amqp.Router(queues)
-        options.setdefault('compression',
-                           self.conf.CELERY_MESSAGE_COMPRESSION)
         options = router.route(options, name, args, kwargs)
-        with self.producer_or_acquire(producer) as producer:
-            return result_cls(producer.publish_task(
-                name, args, kwargs,
-                task_id=task_id,
-                countdown=countdown, eta=eta,
-                callbacks=maybe_list(link),
-                errbacks=maybe_list(link_error),
-                expires=expires, **options
-            ))
+        if connection:
+            producer = self.amqp.TaskProducer(connection)
+        with self.producer_or_acquire(producer) as P:
+            self.backend.on_task_call(P, task_id)
+            task_id = P.publish_task(
+                name, args, kwargs, countdown=countdown, eta=eta,
+                task_id=task_id, expires=expires,
+                callbacks=maybe_list(link), errbacks=maybe_list(link_error),
+                reply_to=reply_to or self.oid, **options
+            )
+        result = (result_cls or self.AsyncResult)(task_id)
+        if add_to_parent:
+            parent = get_current_worker_task()
+            if parent:
+                parent.add_trail(result)
+        return result
 
-    def connection(self, hostname=None, userid=None,
-                   password=None, virtual_host=None, port=None, ssl=None,
-                   insist=None, connect_timeout=None, transport=None,
-                   transport_options=None, heartbeat=None, **kwargs):
+    def connection(self, hostname=None, userid=None, password=None,
+                   virtual_host=None, port=None, ssl=None,
+                   connect_timeout=None, transport=None,
+                   transport_options=None, heartbeat=None,
+                   login_method=None, failover_strategy=None, **kwargs):
         conf = self.conf
         return self.amqp.Connection(
-            hostname or conf.BROKER_HOST,
+            hostname or conf.BROKER_URL,
             userid or conf.BROKER_USER,
             password or conf.BROKER_PASSWORD,
             virtual_host or conf.BROKER_VHOST,
             port or conf.BROKER_PORT,
             transport=transport or conf.BROKER_TRANSPORT,
-            insist=self.either('BROKER_INSIST', insist),
             ssl=self.either('BROKER_USE_SSL', ssl),
-            connect_timeout=self.either(
-                'BROKER_CONNECTION_TIMEOUT', connect_timeout),
             heartbeat=heartbeat,
-            transport_options=dict(conf.BROKER_TRANSPORT_OPTIONS,
-                                   **transport_options or {}))
+            login_method=login_method or conf.BROKER_LOGIN_METHOD,
+            failover_strategy=(
+                failover_strategy or conf.BROKER_FAILOVER_STRATEGY
+            ),
+            transport_options=dict(
+                conf.BROKER_TRANSPORT_OPTIONS, **transport_options or {}
+            ),
+            connect_timeout=self.either(
+                'BROKER_CONNECTION_TIMEOUT', connect_timeout
+            ),
+        )
     broker_connection = connection
 
     @contextmanager
@@ -299,26 +363,6 @@ class Celery(object):
                 yield producer
     default_producer = producer_or_acquire  # XXX compat
 
-    def with_default_connection(self, fun):
-        """With any function accepting a `connection`
-        keyword argument, establishes a default connection if one is
-        not already passed to it.
-
-        Any automatically established connection will be closed after
-        the function returns.
-
-        **Deprecated**
-
-        Use ``with app.connection_or_acquire(connection)`` instead.
-
-        """
-        @wraps(fun)
-        def _inner(*args, **kwargs):
-            connection = kwargs.pop('connection', None)
-            with self.connection_or_acquire(connection) as c:
-                return fun(*args, **dict(kwargs, connection=c))
-        return _inner
-
     def prepare_config(self, c):
         """Prepare configuration before it is merged with the defaults."""
         return find_deprecated_settings(c)
@@ -342,7 +386,7 @@ class Celery(object):
             )
 
     def select_queues(self, queues=None):
-        return self.amqp.queues.select_subset(queues)
+        return self.amqp.queues.select(queues)
 
     def either(self, default_key, *values):
         """Fallback to the value of a configuration key if none of the
@@ -359,7 +403,12 @@ class Celery(object):
             self.loader)
         return backend(app=self, url=url)
 
+    def on_configure(self):
+        """Callback calld when the app loads configuration"""
+        pass
+
     def _get_config(self):
+        self.on_configure()
         self.configured = True
         s = Settings({}, [self.prepare_config(self.loader.conf),
                           deepcopy(DEFAULTS)])
@@ -367,9 +416,9 @@ class Celery(object):
         # load lazy config dict initializers.
         pending = self._pending_defaults
         while pending:
-            s.add_defaults(pending.popleft()())
+            s.add_defaults(maybe_evaluate(pending.popleft()()))
         if self._preconf:
-            for key, value in self._preconf.iteritems():
+            for key, value in items(self._preconf):
                 setattr(s, key, value)
         return s
 
@@ -385,11 +434,15 @@ class Celery(object):
                 amqp._producer_pool.force_close_all()
                 amqp._producer_pool = None
 
+    def signature(self, *args, **kwargs):
+        kwargs['app'] = self
+        return self.canvas.signature(*args, **kwargs)
+
     def create_task_cls(self):
         """Creates a base task class using default configuration
         taken from this app."""
         return self.subclass_with_self(
-            'celery.app.task:Task', name='Task', attribute='_app',
+            self.task_cls, name='Task', attribute='_app',
             keep_reduce=True, abstract=True,
         )
 
@@ -422,13 +475,17 @@ class Celery(object):
         return type(name or Class.__name__, (Class, ), attrs)
 
     def _rgetattr(self, path):
-        return reduce(getattr, [self] + path.split('.'))
+        return attrgetter(path)(self)
 
     def __repr__(self):
-        return '<%s %s:0x%x>' % (self.__class__.__name__,
-                                 self.main or '__main__', id(self), )
+        return '<{0} {1}>'.format(type(self).__name__, appstr(self))
 
     def __reduce__(self):
+        if self._using_v1_reduce:
+            return self.__reduce_v1__()
+        return (_unpickle_app_v2, (self.__class__, self.__reduce_keys__()))
+
+    def __reduce_v1__(self):
         # Reduce only pickles the configuration changes,
         # so the default configuration doesn't have to be passed
         # between processes.
@@ -437,11 +494,30 @@ class Celery(object):
             (self.__class__, self.Pickler) + self.__reduce_args__(),
         )
 
+    def __reduce_keys__(self):
+        """Return keyword arguments used to reconstruct the object
+        when unpickling."""
+        return {
+            'main': self.main,
+            'changes': self.conf.changes,
+            'loader': self.loader_cls,
+            'backend': self.backend_cls,
+            'amqp': self.amqp_cls,
+            'events': self.events_cls,
+            'log': self.log_cls,
+            'control': self.control_cls,
+            'accept_magic_kwargs': self.accept_magic_kwargs,
+            'fixups': self.fixups,
+            'config_source': self._config_source,
+            'task_cls': self.task_cls,
+        }
+
     def __reduce_args__(self):
-        return (self.main, self.conf.changes, self.loader_cls,
-                self.backend_cls, self.amqp_cls, self.events_cls,
-                self.log_cls, self.control_cls, self.accept_magic_kwargs,
-                self._config_source)
+        """Deprecated method, please use :meth:`__reduce_keys__` instead."""
+        return (self.main, self.conf.changes,
+                self.loader_cls, self.backend_cls, self.amqp_cls,
+                self.events_cls, self.log_cls, self.control_cls,
+                self.accept_magic_kwargs, self._config_source)
 
     @cached_property
     def Worker(self):
@@ -455,10 +531,6 @@ class Celery(object):
     def Beat(self, **kwargs):
         return self.subclass_with_self('celery.apps.beat:Beat')
 
-    @cached_property
-    def TaskSet(self):
-        return self.subclass_with_self('celery.task.sets:TaskSet')
-
     @cached_property
     def Task(self):
         return self.create_task_cls()
@@ -471,12 +543,22 @@ class Celery(object):
     def AsyncResult(self):
         return self.subclass_with_self('celery.result:AsyncResult')
 
+    @cached_property
+    def ResultSet(self):
+        return self.subclass_with_self('celery.result:ResultSet')
+
     @cached_property
     def GroupResult(self):
         return self.subclass_with_self('celery.result:GroupResult')
 
+    @cached_property
+    def TaskSet(self):  # XXX compat
+        """Deprecated! Please use :class:`celery.group` instead."""
+        return self.subclass_with_self('celery.task.sets:TaskSet')
+
     @cached_property
     def TaskSetResult(self):  # XXX compat
+        """Deprecated! Please use :attr:`GroupResult` instead."""
         return self.subclass_with_self('celery.result:TaskSetResult')
 
     @property
@@ -491,6 +573,10 @@ class Celery(object):
     def current_task(self):
         return _task_stack.top
 
+    @cached_property
+    def oid(self):
+        return oid_from(self)
+
     @cached_property
     def amqp(self):
         return instantiate(self.amqp_cls, app=self)
@@ -519,8 +605,23 @@ class Celery(object):
     def log(self):
         return instantiate(self.log_cls, app=self)
 
+    @cached_property
+    def canvas(self):
+        from celery import canvas
+        return canvas
+
     @cached_property
     def tasks(self):
         self.finalize()
         return self._tasks
+
+    @cached_property
+    def timezone(self):
+        from celery.utils.timeutils import timezone
+        conf = self.conf
+        tz = conf.CELERY_TIMEZONE
+        if not tz:
+            return (timezone.get_timezone('UTC') if conf.CELERY_ENABLE_UTC
+                    else timezone.local)
+        return timezone.get_timezone(self.conf.CELERY_TIMEZONE)
 App = Celery  # compat
