@@ -8,17 +8,32 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
+
+import warnings
 
 from kombu.pidbox import Mailbox
 from kombu.utils import cached_property
 
-from . import app_or_default
+from celery.exceptions import DuplicateNodenameWarning
+
+__all__ = ['Inspect', 'Control', 'flatten_reply']
+
+W_DUPNODE = """\
+Received multiple replies from node name {0!r}.
+Please make sure you give each node a unique nodename using the `-n` option.\
+"""
 
 
 def flatten_reply(reply):
     nodes = {}
+    seen = set()
     for item in reply:
+        dup = next((nodename in seen for nodename in item), None)
+        if dup:
+            warnings.warn(DuplicateNodenameWarning(
+                W_DUPNODE.format(dup),
+            ))
+        seen.update(item)
         nodes.update(item)
     return nodes
 
@@ -58,6 +73,9 @@ class Inspect(object):
     def report(self):
         return self._request('report')
 
+    def clock(self):
+        return self._request('clock')
+
     def active(self, safe=False):
         return self._request('dump_active', safe=safe)
 
@@ -83,15 +101,30 @@ class Inspect(object):
     def active_queues(self):
         return self._request('active_queues')
 
+    def query_task(self, ids):
+        return self._request('query_task', ids=ids)
+
     def conf(self):
         return self._request('dump_conf')
 
+    def hello(self, from_node, revoked=None):
+        return self._request('hello', from_node=from_node, revoked=revoked)
+
+    def memsample(self):
+        return self._request('memsample')
+
+    def memdump(self, samples=10):
+        return self._request('memdump', samples=samples)
+
+    def objgraph(self, type='Request', n=200, max_depth=10):
+        return self._request('objgraph', num=n, max_depth=max_depth, type=type)
+
 
 class Control(object):
     Mailbox = Mailbox
 
     def __init__(self, app=None):
-        self.app = app_or_default(app)
+        self.app = app
         self.mailbox = self.Mailbox('celery', type='fanout',
                                     accept=self.app.conf.CELERY_ACCEPT_CONTENT)
 
@@ -112,6 +145,11 @@ class Control(object):
             return self.app.amqp.TaskConsumer(conn).purge()
     discard_all = purge
 
+    def election(self, id, topic, action=None, connection=None):
+        self.broadcast('election', connection=connection, arguments={
+            'id': id, 'topic': topic, 'action': action,
+        })
+
     def revoke(self, task_id, destination=None, terminate=False,
                signal='SIGTERM', **kwargs):
         """Tell all (or specific) workers to revoke a task by id.
@@ -136,7 +174,7 @@ class Control(object):
     def ping(self, destination=None, timeout=1, **kwargs):
         """Ping all (or specific) workers.
 
-        Returns answer from alive workers.
+        Will return the list of answers.
 
         See :meth:`broadcast` for supported keyword arguments.
 
@@ -234,7 +272,7 @@ class Control(object):
         Supports the same arguments as :meth:`broadcast`.
 
         """
-        return self.broadcast('pool_grow', {}, destination, **kwargs)
+        return self.broadcast('pool_grow', {'n': n}, destination, **kwargs)
 
     def pool_shrink(self, n=1, destination=None, **kwargs):
         """Tell all (or specific) workers to shrink the pool by ``n``.
@@ -242,7 +280,7 @@ class Control(object):
         Supports the same arguments as :meth:`broadcast`.
 
         """
-        return self.broadcast('pool_shrink', {}, destination, **kwargs)
+        return self.broadcast('pool_shrink', {'n': n}, destination, **kwargs)
 
     def broadcast(self, command, arguments=None, destination=None,
                   connection=None, reply=False, timeout=1, limit=None,
