@@ -12,22 +12,27 @@ import os
 import platform as _platform
 import re
 
-from celery import platforms
+from collections import Mapping
+
 from celery.datastructures import ConfigurationView
+from celery.five import items, string_t
+from celery.platforms import pyimplementation
 from celery.utils.text import pretty
 from celery.utils.imports import qualname
 
 from .defaults import find
 
+__all__ = ['appstr', 'Settings', 'filter_hidden_settings', 'bugreport']
+
 #: Format used to generate bugreport information.
 BUGREPORT_INFO = """
-software -> celery:%(celery_v)s kombu:%(kombu_v)s py:%(py_v)s
-            billiard:%(billiard_v)s %(driver_v)s
-platform -> system:%(system)s arch:%(arch)s imp:%(py_i)s
-loader   -> %(loader)s
-settings -> transport:%(transport)s results:%(results)s
+software -> celery:{celery_v} kombu:{kombu_v} py:{py_v}
+            billiard:{billiard_v} {driver_v}
+platform -> system:{system} arch:{arch} imp:{py_i}
+loader   -> {loader}
+settings -> transport:{transport} results:{results}
 
-%(human_settings)s
+{human_settings}
 """
 
 HIDDEN_SETTINGS = re.compile(
@@ -36,6 +41,11 @@ HIDDEN_SETTINGS = re.compile(
 )
 
 
+def appstr(app):
+    """String used in __repr__ etc, to id app instances."""
+    return '{0}:0x{1:x}'.format(app.main or '__main__', id(app))
+
+
 class Settings(ConfigurationView):
     """Celery settings object."""
 
@@ -54,7 +64,7 @@ class Settings(ConfigurationView):
         return self.BROKER_TRANSPORT
 
     @property
-    def BROKER_HOST(self):
+    def BROKER_URL(self):
         return (os.environ.get('CELERY_BROKER_URL') or
                 self.first('BROKER_URL', 'BROKER_HOST'))
 
@@ -64,16 +74,19 @@ class Settings(ConfigurationView):
         return self.first('CELERY_TIMEZONE', 'TIME_ZONE')
 
     def without_defaults(self):
-        """Returns the current configuration, but without defaults."""
+        """Return the current configuration, but without defaults."""
         # the last stash is the default settings, so just skip that
         return Settings({}, self._order[:-1])
 
+    def value_set_for(self, key):
+        return key in self.without_defaults()
+
     def find_option(self, name, namespace='celery'):
         """Search for option by name.
 
-        Will return ``(namespace, option_name, Option)`` tuple, e.g.::
+        Will return ``(namespace, key, type)`` tuple, e.g.::
 
-            >>> celery.conf.find_option('disable_rate_limits')
+            >>> app.conf.find_option('disable_rate_limits')
             ('CELERY', 'DISABLE_RATE_LIMITS',
              <Option: type->bool default->False>))
 
@@ -88,7 +101,7 @@ class Settings(ConfigurationView):
         return self.get_by_parts(*self.find_option(name, namespace)[:-1])
 
     def get_by_parts(self, *parts):
-        """Returns the current value for setting specified as a path.
+        """Return the current value for setting specified as a path.
 
         Example::
 
@@ -98,18 +111,24 @@ class Settings(ConfigurationView):
         """
         return self['_'.join(part for part in parts if part)]
 
-    def humanize(self):
-        """Returns a human readable string showing changes to the
+    def table(self, with_defaults=False, censored=True):
+        filt = filter_hidden_settings if censored else lambda v: v
+        return filt(dict(
+            (k, v) for k, v in items(
+                self if with_defaults else self.without_defaults())
+            if k.isupper() and not k.startswith('_')
+        ))
+
+    def humanize(self, with_defaults=False, censored=True):
+        """Return a human readable string showing changes to the
         configuration."""
         return '\n'.join(
-            '%s: %s' % (key, pretty(value, width=50))
-            for key, value in filter_hidden_settings(dict(
-                (k, v) for k, v in self.without_defaults().iteritems()
-                if k.isupper() and not k.startswith('_'))).iteritems())
+            '{0}: {1}'.format(key, pretty(value, width=50))
+            for key, value in items(self.table(with_defaults, censored)))
 
 
 class AppPickler(object):
-    """Default application pickler/unpickler."""
+    """Old application pickler/unpickler (< 3.1)."""
 
     def __call__(self, cls, *args):
         kwargs = self.build_kwargs(*args)
@@ -137,42 +156,56 @@ class AppPickler(object):
 
 
 def _unpickle_app(cls, pickler, *args):
+    """Rebuild app for versions 2.5+"""
     return pickler()(cls, *args)
 
 
+def _unpickle_app_v2(cls, kwargs):
+    """Rebuild app for versions 3.1+"""
+    kwargs['set_as_current'] = False
+    return cls(**kwargs)
+
+
 def filter_hidden_settings(conf):
 
-    def maybe_censor(key, value):
-        return '********' if HIDDEN_SETTINGS.search(key) else value
+    def maybe_censor(key, value, mask='*' * 8):
+        if isinstance(value, Mapping):
+            return filter_hidden_settings(value)
+        if isinstance(value, string_t) and HIDDEN_SETTINGS.search(key):
+            return mask
+        if 'BROKER_URL' in key.upper():
+            from kombu import Connection
+            return Connection(value).as_uri(mask=mask)
+        return value
 
-    return dict((k, maybe_censor(k, v)) for k, v in conf.iteritems())
+    return dict((k, maybe_censor(k, v)) for k, v in items(conf))
 
 
 def bugreport(app):
-    """Returns a string containing information useful in bug reports."""
+    """Return a string containing information useful in bug reports."""
     import billiard
     import celery
     import kombu
 
     try:
         conn = app.connection()
-        driver_v = '%s:%s' % (conn.transport.driver_name,
-                              conn.transport.driver_version())
+        driver_v = '{0}:{1}'.format(conn.transport.driver_name,
+                                    conn.transport.driver_version())
         transport = conn.transport_cls
     except Exception:
         transport = driver_v = ''
 
-    return BUGREPORT_INFO % {
-        'system': _platform.system(),
-        'arch': ', '.join(p for p in _platform.architecture() if p),
-        'py_i': platforms.pyimplementation(),
-        'celery_v': celery.VERSION_BANNER,
-        'kombu_v': kombu.__version__,
-        'billiard_v': billiard.__version__,
-        'py_v': _platform.python_version(),
-        'driver_v': driver_v,
-        'transport': transport,
-        'results': app.conf.CELERY_RESULT_BACKEND or 'disabled',
-        'human_settings': app.conf.humanize(),
-        'loader': qualname(app.loader.__class__),
-    }
+    return BUGREPORT_INFO.format(
+        system=_platform.system(),
+        arch=', '.join(x for x in _platform.architecture() if x),
+        py_i=pyimplementation(),
+        celery_v=celery.VERSION_BANNER,
+        kombu_v=kombu.__version__,
+        billiard_v=billiard.__version__,
+        py_v=_platform.python_version(),
+        driver_v=driver_v,
+        transport=transport,
+        results=app.conf.CELERY_RESULT_BACKEND or 'disabled',
+        human_settings=app.conf.humanize(),
+        loader=qualname(app.loader.__class__),
+    )
