@@ -7,8 +7,6 @@
 
     - :class:`BaseBackend` defines the interface.
 
-    - :class:`BaseDictBackend` assumes the fields are stored in a dict.
-
     - :class:`KeyValueStoreBackend` is a common base class
       using K/V semantics like _get and _put.
 
@@ -21,32 +19,38 @@ import sys
 from datetime import timedelta
 
 from billiard.einfo import ExceptionInfo
-from kombu import serialization
+from kombu.serialization import (
+    dumps, loads, prepare_accept_content,
+    registry as serializer_registry,
+)
 from kombu.utils.encoding import bytes_to_str, ensure_bytes, from_utf8
 
 from celery import states
 from celery.app import current_task
-from celery.datastructures import LRUCache
-from celery.exceptions import ChordError, TaskRevokedError, TimeoutError
-from celery.result import from_serializable, GroupResult
+from celery.exceptions import ChordError, TimeoutError, TaskRevokedError
+from celery.five import items
+from celery.result import result_from_tuple, GroupResult
 from celery.utils import timeutils
+from celery.utils.functional import LRUCache
 from celery.utils.serialization import (
     get_pickled_exception,
     get_pickleable_exception,
     create_exception_cls,
 )
 
+__all__ = ['BaseBackend', 'KeyValueStoreBackend', 'DisabledBackend']
+
 EXCEPTION_ABLE_CODECS = frozenset(['pickle', 'yaml'])
-is_py3k = sys.version_info >= (3, 0)
+PY3 = sys.version_info >= (3, 0)
 
 
 def unpickle_backend(cls, args, kwargs):
-    """Returns an unpickled backend."""
-    return cls(*args, **kwargs)
+    """Return an unpickled backend."""
+    from celery import current_app
+    return cls(*args, app=current_app._get_current_object(), **kwargs)
 
 
 class BaseBackend(object):
-    """Base backend class."""
     READY_STATES = states.READY_STATES
     UNREADY_STATES = states.UNREADY_STATES
     EXCEPTION_STATES = states.EXCEPTION_STATES
@@ -66,57 +70,37 @@ class BaseBackend(object):
     #: in this case.
     supports_autoexpire = False
 
-    def __init__(self, *args, **kwargs):
-        from celery.app import app_or_default
-        self.app = app_or_default(kwargs.get('app'))
-        self.serializer = kwargs.get('serializer',
-                                     self.app.conf.CELERY_RESULT_SERIALIZER)
+    #: Set to true if the backend is peristent by default.
+    persistent = True
+
+    def __init__(self, app, serializer=None,
+                 max_cached_results=None, accept=None, **kwargs):
+        self.app = app
+        conf = self.app.conf
+        self.serializer = serializer or conf.CELERY_RESULT_SERIALIZER
         (self.content_type,
          self.content_encoding,
-         self.encoder) = serialization.registry._encoders[self.serializer]
-
-    def encode(self, data):
-        _, _, payload = serialization.encode(data, serializer=self.serializer)
-        return payload
-
-    def decode(self, payload):
-        payload = is_py3k and payload or str(payload)
-        return serialization.decode(payload,
-                                    content_type=self.content_type,
-                                    content_encoding=self.content_encoding)
-
-    def prepare_expires(self, value, type=None):
-        if value is None:
-            value = self.app.conf.CELERY_TASK_RESULT_EXPIRES
-        if isinstance(value, timedelta):
-            value = timeutils.timedelta_seconds(value)
-        if value is not None and type:
-            return type(value)
-        return value
-
-    def encode_result(self, result, status):
-        if status in self.EXCEPTION_STATES and isinstance(result, Exception):
-            return self.prepare_exception(result)
-        else:
-            return self.prepare_value(result)
-
-    def store_result(self, task_id, result, status, traceback=None):
-        """Store the result and status of a task."""
-        raise NotImplementedError(
-            'store_result is not supported by this backend.')
+         self.encoder) = serializer_registry._encoders[self.serializer]
+        self._cache = LRUCache(
+            limit=max_cached_results or conf.CELERY_MAX_CACHED_RESULTS,
+        )
+        self.accept = prepare_accept_content(
+            conf.CELERY_ACCEPT_CONTENT if accept is None else accept,
+        )
 
     def mark_as_started(self, task_id, **meta):
         """Mark a task as started"""
         return self.store_result(task_id, meta, status=states.STARTED)
 
-    def mark_as_done(self, task_id, result):
+    def mark_as_done(self, task_id, result, request=None):
         """Mark task as successfully executed."""
-        return self.store_result(task_id, result, status=states.SUCCESS)
+        return self.store_result(task_id, result,
+                                 status=states.SUCCESS, request=request)
 
-    def mark_as_failure(self, task_id, exc, traceback=None):
+    def mark_as_failure(self, task_id, exc, traceback=None, request=None):
         """Mark task as executed with failure. Stores the execption."""
         return self.store_result(task_id, exc, status=states.FAILURE,
-                                 traceback=traceback)
+                                 traceback=traceback, request=request)
 
     def fail_from_current_stack(self, task_id, exc=None):
         type_, real_exc, tb = sys.exc_info()
@@ -128,15 +112,16 @@ class BaseBackend(object):
         finally:
             del(tb)
 
-    def mark_as_retry(self, task_id, exc, traceback=None):
+    def mark_as_retry(self, task_id, exc, traceback=None, request=None):
         """Mark task as being retries. Stores the current
         exception (if any)."""
         return self.store_result(task_id, exc, status=states.RETRY,
-                                 traceback=traceback)
+                                 traceback=traceback, request=request)
 
-    def mark_as_revoked(self, task_id, reason=''):
+    def mark_as_revoked(self, task_id, reason='', request=None):
         return self.store_result(task_id, TaskRevokedError(reason),
-                                 status=states.REVOKED, traceback=None)
+                                 status=states.REVOKED, traceback=None,
+                                 request=request)
 
     def prepare_exception(self, exc):
         """Prepare exception for serialization."""
@@ -148,18 +133,25 @@ class BaseBackend(object):
         """Convert serialized exception to Python exception."""
         if self.serializer in EXCEPTION_ABLE_CODECS:
             return get_pickled_exception(exc)
-        return create_exception_cls(from_utf8(exc['exc_type']),
-                                    sys.modules[__name__])(exc['exc_message'])
+        return create_exception_cls(
+            from_utf8(exc['exc_type']), __name__)(exc['exc_message'])
 
     def prepare_value(self, result):
         """Prepare value for storage."""
         if isinstance(result, GroupResult):
-            return result.serializable()
+            return result.as_tuple()
         return result
 
-    def forget(self, task_id):
-        raise NotImplementedError('%s does not implement forget.' % (
-            self.__class__))
+    def encode(self, data):
+        _, _, payload = dumps(data, serializer=self.serializer)
+        return payload
+
+    def decode(self, payload):
+        payload = PY3 and payload or str(payload)
+        return loads(payload,
+                     content_type=self.content_type,
+                     content_encoding=self.content_encoding,
+                     accept=self.accept)
 
     def wait_for(self, task_id, timeout=None, propagate=True, interval=0.5):
         """Wait for task and return its result.
@@ -190,96 +182,36 @@ class BaseBackend(object):
             if timeout and time_elapsed >= timeout:
                 raise TimeoutError('The operation timed out.')
 
-    def cleanup(self):
-        """Backend cleanup. Is run by
-        :class:`celery.task.DeleteExpiredTaskMetaTask`."""
-        pass
+    def prepare_expires(self, value, type=None):
+        if value is None:
+            value = self.app.conf.CELERY_TASK_RESULT_EXPIRES
+        if isinstance(value, timedelta):
+            value = timeutils.timedelta_seconds(value)
+        if value is not None and type:
+            return type(value)
+        return value
 
-    def process_cleanup(self):
-        """Cleanup actions to do at the end of a task worker process."""
-        pass
+    def prepare_persistent(self, enabled=None):
+        if enabled is not None:
+            return enabled
+        p = self.app.conf.CELERY_RESULT_PERSISTENT
+        return self.persistent if p is None else p
 
-    def get_status(self, task_id):
-        """Get the status of a task."""
-        raise NotImplementedError(
-            'get_status is not supported by this backend.')
-
-    def get_result(self, task_id):
-        """Get the result of a task."""
-        raise NotImplementedError(
-            'get_result is not supported by this backend.')
-
-    def get_children(self, task_id):
-        raise NotImplementedError(
-            'get_children is not supported by this backend.')
-
-    def get_traceback(self, task_id):
-        """Get the traceback for a failed task."""
-        raise NotImplementedError(
-            'get_traceback is not supported by this backend.')
-
-    def save_group(self, group_id, result):
-        """Store the result and status of a task."""
-
-        raise NotImplementedError(
-            'save_group is not supported by %s.' % (type(self).__name__, ))
-
-    def restore_group(self, group_id, cache=True):
-        """Get the result of a group."""
-        raise NotImplementedError(
-            'restore_group is not supported by this backend.')
-
-    def delete_group(self, group_id):
-        raise NotImplementedError(
-            'delete_group is not supported by this backend.')
-
-    def reload_task_result(self, task_id):
-        """Reload task result, even if it has been previously fetched."""
-        raise NotImplementedError(
-            'reload_task_result is not supported by this backend.')
-
-    def reload_group_result(self, task_id):
-        """Reload group result, even if it has been previously fetched."""
-        raise NotImplementedError(
-            'reload_group_result is not supported by this backend.')
-
-    def on_chord_part_return(self, task, propagate=True):
-        pass
-
-    def fallback_chord_unlock(self, group_id, body, result=None,
-                              countdown=1, **kwargs):
-        kwargs['result'] = [r.id for r in result]
-        self.app.tasks['celery.chord_unlock'].apply_async(
-            (group_id, body, ), kwargs, countdown=countdown,
-        )
-    on_chord_apply = fallback_chord_unlock
-
-    def current_task_children(self):
-        current = current_task()
-        if current:
-            return [r.serializable() for r in current.request.children]
-
-    def __reduce__(self, args=(), kwargs={}):
-        return (unpickle_backend, (self.__class__, args, kwargs))
-
-    def is_cached(self, task_id):
-        return False
-
-
-class BaseDictBackend(BaseBackend):
-
-    def __init__(self, *args, **kwargs):
-        super(BaseDictBackend, self).__init__(*args, **kwargs)
-        self._cache = LRUCache(limit=kwargs.get('max_cached_results') or
-                               self.app.conf.CELERY_MAX_CACHED_RESULTS)
+    def encode_result(self, result, status):
+        if status in self.EXCEPTION_STATES and isinstance(result, Exception):
+            return self.prepare_exception(result)
+        else:
+            return self.prepare_value(result)
 
     def is_cached(self, task_id):
         return task_id in self._cache
 
-    def store_result(self, task_id, result, status, traceback=None, **kwargs):
-        """Store task result and status."""
+    def store_result(self, task_id, result, status,
+                     traceback=None, request=None, **kwargs):
+        """Update task state and result."""
         result = self.encode_result(result, status)
-        self._store_result(task_id, result, status, traceback, **kwargs)
+        self._store_result(task_id, result, status, traceback,
+                           request=request, **kwargs)
         return result
 
     def forget(self, task_id):
@@ -287,8 +219,7 @@ class BaseDictBackend(BaseBackend):
         self._forget(task_id)
 
     def _forget(self, task_id):
-        raise NotImplementedError('%s does not implement forget.' % (
-            self.__class__))
+        raise NotImplementedError('backend does not implement forget.')
 
     def get_status(self, task_id):
         """Get the status of a task."""
@@ -326,11 +257,12 @@ class BaseDictBackend(BaseBackend):
         return meta
 
     def reload_task_result(self, task_id):
+        """Reload task result, even if it has been previously fetched."""
         self._cache[task_id] = self.get_task_meta(task_id, cache=False)
 
     def reload_group_result(self, group_id):
-        self._cache[group_id] = self.get_group_meta(group_id,
-                                                    cache=False)
+        """Reload group result, even if it has been previously fetched."""
+        self._cache[group_id] = self.get_group_meta(group_id, cache=False)
 
     def get_group_meta(self, group_id, cache=True):
         if cache:
@@ -358,8 +290,40 @@ class BaseDictBackend(BaseBackend):
         self._cache.pop(group_id, None)
         return self._delete_group(group_id)
 
+    def cleanup(self):
+        """Backend cleanup. Is run by
+        :class:`celery.task.DeleteExpiredTaskMetaTask`."""
+        pass
 
-class KeyValueStoreBackend(BaseDictBackend):
+    def process_cleanup(self):
+        """Cleanup actions to do at the end of a task worker process."""
+        pass
+
+    def on_task_call(self, producer, task_id):
+        return {}
+
+    def on_chord_part_return(self, task, propagate=False):
+        pass
+
+    def fallback_chord_unlock(self, group_id, body, result=None,
+                              countdown=1, **kwargs):
+        kwargs['result'] = [r.as_tuple() for r in result]
+        self.app.tasks['celery.chord_unlock'].apply_async(
+            (group_id, body, ), kwargs, countdown=countdown,
+        )
+    on_chord_apply = fallback_chord_unlock
+
+    def current_task_children(self, request=None):
+        request = request or getattr(current_task(), 'request', None)
+        if request:
+            return [r.as_tuple() for r in getattr(request, 'children', [])]
+
+    def __reduce__(self, args=(), kwargs={}):
+        return (unpickle_backend, (self.__class__, args, kwargs))
+BaseDictBackend = BaseBackend  # XXX compat
+
+
+class KeyValueStoreBackend(BaseBackend):
     task_keyprefix = ensure_bytes('celery-task-meta-')
     group_keyprefix = ensure_bytes('celery-taskset-meta-')
     chord_keyprefix = ensure_bytes('chord-unlock-')
@@ -407,7 +371,7 @@ class KeyValueStoreBackend(BaseDictBackend):
         if hasattr(values, 'items'):
             # client returns dict so mapping preserved.
             return dict((self._strip_prefix(k), self.decode(v))
-                        for k, v in values.iteritems()
+                        for k, v in items(values)
                         if v is not None)
         else:
             # client returns list so need to recreate mapping.
@@ -415,46 +379,50 @@ class KeyValueStoreBackend(BaseDictBackend):
                         for i, value in enumerate(values)
                         if value is not None)
 
-    def get_many(self, task_ids, timeout=None, interval=0.5):
-        ids = set(task_ids)
+    def get_many(self, task_ids, timeout=None, interval=0.5,
+                 READY_STATES=states.READY_STATES):
+        interval = 0.5 if interval is None else interval
+        ids = task_ids if isinstance(task_ids, set) else set(task_ids)
         cached_ids = set()
+        cache = self._cache
         for task_id in ids:
             try:
-                cached = self._cache[task_id]
+                cached = cache[task_id]
             except KeyError:
                 pass
             else:
-                if cached['status'] in states.READY_STATES:
+                if cached['status'] in READY_STATES:
                     yield bytes_to_str(task_id), cached
                     cached_ids.add(task_id)
 
-        ids ^= cached_ids
+        ids.difference_update(cached_ids)
         iterations = 0
         while ids:
             keys = list(ids)
             r = self._mget_to_results(self.mget([self.get_key_for_task(k)
                                                  for k in keys]), keys)
-            self._cache.update(r)
-            ids ^= set(bytes_to_str(v) for v in r)
-            for key, value in r.iteritems():
+            cache.update(r)
+            ids.difference_update(set(bytes_to_str(v) for v in r))
+            for key, value in items(r):
                 yield bytes_to_str(key), value
             if timeout and iterations * interval >= timeout:
-                raise TimeoutError('Operation timed out (%s)' % (timeout, ))
+                raise TimeoutError('Operation timed out ({0})'.format(timeout))
             time.sleep(interval)  # don't busy loop.
             iterations += 1
 
     def _forget(self, task_id):
         self.delete(self.get_key_for_task(task_id))
 
-    def _store_result(self, task_id, result, status, traceback=None):
+    def _store_result(self, task_id, result, status,
+                      traceback=None, request=None, **kwargs):
         meta = {'status': status, 'result': result, 'traceback': traceback,
-                'children': self.current_task_children()}
+                'children': self.current_task_children(request)}
         self.set(self.get_key_for_task(task_id), self.encode(meta))
         return result
 
     def _save_group(self, group_id, result):
         self.set(self.get_key_for_group(group_id),
-                 self.encode({'result': result.serializable()}))
+                 self.encode({'result': result.as_tuple()}))
         return result
 
     def _delete_group(self, group_id):
@@ -476,8 +444,7 @@ class KeyValueStoreBackend(BaseDictBackend):
         if meta:
             meta = self.decode(meta)
             result = meta['result']
-            if isinstance(result, (list, tuple)):
-                return {'result': from_serializable(result, self.app)}
+            meta['result'] = result_from_tuple(result, self.app)
             return meta
 
     def on_chord_apply(self, group_id, body, result=None, **kwargs):
@@ -489,7 +456,7 @@ class KeyValueStoreBackend(BaseDictBackend):
     def on_chord_part_return(self, task, propagate=None):
         if not self.implements_incr:
             return
-        from celery import subtask
+        from celery import maybe_signature
         from celery.result import GroupResult
         app = self.app
         if propagate is None:
@@ -498,29 +465,51 @@ class KeyValueStoreBackend(BaseDictBackend):
         if not gid:
             return
         key = self.get_key_for_chord(gid)
-        deps = GroupResult.restore(gid, backend=task.backend)
+        try:
+            deps = GroupResult.restore(gid, backend=task.backend)
+        except Exception as exc:
+            callback = maybe_signature(task.request.chord, app=self.app)
+            return app._tasks[callback.task].backend.fail_from_current_stack(
+                callback.id,
+                exc=ChordError('Cannot restore group: {0!r}'.format(exc)),
+            )
+        if deps is None:
+            try:
+                raise ValueError(gid)
+            except ValueError as exc:
+                callback = maybe_signature(task.request.chord, app=self.app)
+                task = app._tasks[callback.task]
+                return task.backend.fail_from_current_stack(
+                    callback.id,
+                    exc=ChordError('GroupResult {0} no longer exists'.format(
+                        gid,
+                    ))
+                )
         val = self.incr(key)
         if val >= len(deps):
+            callback = maybe_signature(task.request.chord, app=self.app)
             j = deps.join_native if deps.supports_native_join else deps.join
-            callback = subtask(task.request.chord)
             try:
                 ret = j(propagate=propagate)
-            except Exception, exc:
+            except Exception as exc:
                 try:
-                    culprit = deps._failed_join_report().next()
-                    reason = 'Dependency %s raised %r' % (culprit.id, exc)
+                    culprit = next(deps._failed_join_report())
+                    reason = 'Dependency {0.id} raised {1!r}'.format(
+                        culprit, exc,
+                    )
                 except StopIteration:
                     reason = repr(exc)
+
                 app._tasks[callback.task].backend.fail_from_current_stack(
                     callback.id, exc=ChordError(reason),
                 )
             else:
                 try:
                     callback.delay(ret)
-                except Exception, exc:
+                except Exception as exc:
                     app._tasks[callback.task].backend.fail_from_current_stack(
                         callback.id,
-                        exc=ChordError('Callback error: %r' % (exc, )),
+                        exc=ChordError('Callback error: {0!r}'.format(exc)),
                     )
             finally:
                 deps.delete()
