@@ -10,11 +10,13 @@ from __future__ import absolute_import
 
 from kombu.utils import cached_property
 
-from celery.datastructures import LRUCache
 from celery.exceptions import ImproperlyConfigured
+from celery.utils.functional import LRUCache
 
 from .base import KeyValueStoreBackend
 
+__all__ = ['CacheBackend']
+
 _imp = [None]
 
 REQUIRES_BACKEND = """\
@@ -22,8 +24,8 @@ The memcached backend requires either pylibmc or python-memcached.\
 """
 
 UNKNOWN_BACKEND = """\
-The cache backend %r is unknown,
-Please use one of the following backends instead: %s\
+The cache backend {0!r} is unknown,
+Please use one of the following backends instead: {1}\
 """
 
 
@@ -85,13 +87,14 @@ class CacheBackend(KeyValueStoreBackend):
     supports_native_join = True
     implements_incr = True
 
-    def __init__(self, expires=None, backend=None, options={}, **kwargs):
-        super(CacheBackend, self).__init__(self, **kwargs)
+    def __init__(self, app, expires=None, backend=None,
+                 options={}, url=None, **kwargs):
+        super(CacheBackend, self).__init__(app, **kwargs)
 
         self.options = dict(self.app.conf.CELERY_CACHE_BACKEND_OPTIONS,
                             **options)
 
-        self.backend = backend or self.app.conf.CELERY_CACHE_BACKEND
+        self.backend = url or backend or self.app.conf.CELERY_CACHE_BACKEND
         if self.backend:
             self.backend, _, servers = self.backend.partition('://')
             self.servers = servers.rstrip('/').split(';')
@@ -99,7 +102,7 @@ class CacheBackend(KeyValueStoreBackend):
         try:
             self.Client = backends[self.backend]()
         except KeyError:
-            raise ImproperlyConfigured(UNKNOWN_BACKEND % (
+            raise ImproperlyConfigured(UNKNOWN_BACKEND.format(
                 self.backend, ', '.join(backends)))
 
     def get(self, key):
@@ -127,7 +130,7 @@ class CacheBackend(KeyValueStoreBackend):
 
     def __reduce__(self, args=(), kwargs={}):
         servers = ';'.join(self.servers)
-        backend = '%s://%s/' % (self.backend, servers)
+        backend = '{0}://{1}/'.format(self.backend, servers)
         kwargs.update(
             dict(backend=backend,
                  expires=self.expires,
