@@ -22,6 +22,12 @@ except ImportError:         # pragma: no cover
     redis = None            # noqa
     ConnectionError = None  # noqa
 
+__all__ = ['RedisBackend']
+
+REDIS_MISSING = """\
+You need to install the redis library in order to use \
+the Redis result store backend."""
+
 
 class RedisBackend(KeyValueStoreBackend):
     """Redis task result store."""
@@ -53,15 +59,13 @@ class RedisBackend(KeyValueStoreBackend):
         super(RedisBackend, self).__init__(**kwargs)
         conf = self.app.conf
         if self.redis is None:
-            raise ImproperlyConfigured(
-                'You need to install the redis library in order to use '
-                'the Redis result store backend.')
+            raise ImproperlyConfigured(REDIS_MISSING)
 
         # For compatibility with the old REDIS_* configuration keys.
         def _get(key):
-            for prefix in 'CELERY_REDIS_%s', 'REDIS_%s':
+            for prefix in 'CELERY_REDIS_{0}', 'REDIS_{0}':
                 try:
-                    return conf[prefix % key]
+                    return conf[prefix.format(key)]
                 except KeyError:
                     pass
         if host and '://' in host:
