@@ -7,7 +7,6 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import errno
 import os
@@ -16,21 +15,24 @@ import shelve
 import sys
 import traceback
 
+from threading import Event, Thread
+
 from billiard import Process, ensure_multiprocessing
 from kombu.utils import cached_property, reprcall
-from kombu.utils.functional import maybe_promise
+from kombu.utils.functional import maybe_evaluate
 
 from . import __version__
 from . import platforms
 from . import signals
-from . import current_app
-from .app import app_or_default
+from .five import items, reraise, values, monotonic
 from .schedules import maybe_schedule, crontab
 from .utils.imports import instantiate
-from .utils.threads import Event, Thread
 from .utils.timeutils import humanize_seconds
 from .utils.log import get_logger
 
+__all__ = ['SchedulingError', 'ScheduleEntry', 'Scheduler',
+           'PersistentScheduler', 'Service', 'EmbeddedService']
+
 logger = get_logger(__name__)
 debug, info, error, warning = (logger.debug, logger.info,
                                logger.error, logger.warning)
@@ -79,21 +81,22 @@ class ScheduleEntry(object):
 
     def __init__(self, name=None, task=None, last_run_at=None,
                  total_run_count=None, schedule=None, args=(), kwargs={},
-                 options={}, relative=False):
+                 options={}, relative=False, app=None):
+        self.app = app
         self.name = name
         self.task = task
         self.args = args
         self.kwargs = kwargs
         self.options = options
-        self.schedule = maybe_schedule(schedule, relative)
+        self.schedule = maybe_schedule(schedule, relative, app=self.app)
         self.last_run_at = last_run_at or self._default_now()
         self.total_run_count = total_run_count or 0
 
     def _default_now(self):
-        return self.schedule.now() if self.schedule else current_app.now()
+        return self.schedule.now() if self.schedule else self.app.now()
 
     def _next_instance(self, last_run_at=None):
-        """Returns a new instance of the same class, but with
+        """Return a new instance of the same class, but with
         its date and count fields updated."""
         return self.__class__(**dict(
             self,
@@ -102,6 +105,12 @@ class ScheduleEntry(object):
         ))
     __next__ = next = _next_instance  # for 2to3
 
+    def __reduce__(self):
+        return self.__class__, (
+            self.name, self.task, self.last_run_at, self.total_run_count,
+            self.schedule, self.args, self.kwargs, self.options,
+        )
+
     def update(self, other):
         """Update values from another entry.
 
@@ -118,24 +127,28 @@ class ScheduleEntry(object):
         return self.schedule.is_due(self.last_run_at)
 
     def __iter__(self):
-        return vars(self).iteritems()
+        return iter(items(vars(self)))
 
     def __repr__(self):
-        return '<Entry: %s %s {%s}' % (
-            self.name,
-            reprcall(self.task, self.args or (), self.kwargs or {}),
-            self.schedule,
+        return '<Entry: {0.name} {call} {0.schedule}'.format(
+            self,
+            call=reprcall(self.task, self.args or (), self.kwargs or {}),
         )
 
 
 class Scheduler(object):
     """Scheduler for periodic tasks.
 
+    The :program:`celery beat` program may instantiate this class
+    multiple times for introspection purposes, but then with the
+    ``lazy`` argument set.  It is important for subclasses to
+    be idempotent when this argument is set.
+
     :keyword schedule: see :attr:`schedule`.
     :keyword max_interval: see :attr:`max_interval`.
+    :keyword lazy: Do not set up the schedule.
 
     """
-
     Entry = ScheduleEntry
 
     #: The schedule dict/shelve.
@@ -151,10 +164,10 @@ class Scheduler(object):
 
     logger = logger  # compat
 
-    def __init__(self, schedule=None, max_interval=None,
-                 app=None, Publisher=None, lazy=False, **kwargs):
-        app = self.app = app_or_default(app)
-        self.data = maybe_promise({} if schedule is None else schedule)
+    def __init__(self, app, schedule=None, max_interval=None,
+                 Publisher=None, lazy=False, **kwargs):
+        self.app = app
+        self.data = maybe_evaluate({} if schedule is None else schedule)
         self.max_interval = (max_interval
                              or app.conf.CELERYBEAT_MAX_LOOP_INTERVAL
                              or self.max_interval)
@@ -180,7 +193,7 @@ class Scheduler(object):
             info('Scheduler: Sending due task %s (%s)', entry.name, entry.task)
             try:
                 result = self.apply_async(entry, publisher=publisher)
-            except Exception, exc:
+            except Exception as exc:
                 error('Message Error: %s\n%s',
                       exc, traceback.format_stack(), exc_info=True)
             else:
@@ -195,7 +208,7 @@ class Scheduler(object):
         """
         remaining_times = []
         try:
-            for entry in self.schedule.itervalues():
+            for entry in values(self.schedule):
                 next_time_to_run = self.maybe_due(entry, self.publisher)
                 if next_time_to_run:
                     remaining_times.append(next_time_to_run)
@@ -206,10 +219,10 @@ class Scheduler(object):
 
     def should_sync(self):
         return (not self._last_sync or
-                (time.time() - self._last_sync) > self.sync_every)
+                (monotonic() - self._last_sync) > self.sync_every)
 
     def reserve(self, entry):
-        new_entry = self.schedule[entry.name] = entry.next()
+        new_entry = self.schedule[entry.name] = next(entry)
         return new_entry
 
     def apply_async(self, entry, publisher=None, **kwargs):
@@ -228,10 +241,10 @@ class Scheduler(object):
                 result = self.send_task(entry.task, entry.args, entry.kwargs,
                                         publisher=publisher,
                                         **entry.options)
-        except Exception, exc:
-            raise SchedulingError, SchedulingError(
-                "Couldn't apply scheduled task %s: %s" % (
-                    entry.name, exc)), sys.exc_info()[2]
+        except Exception as exc:
+            reraise(SchedulingError, SchedulingError(
+                "Couldn't apply scheduled task {0.name}: {exc}".format(
+                    entry, exc)), sys.exc_info()[2])
         finally:
             if self.should_sync():
                 self._do_sync()
@@ -245,10 +258,10 @@ class Scheduler(object):
 
     def _do_sync(self):
         try:
-            debug('Celerybeat: Synchronizing schedule...')
+            debug('beat: Synchronizing schedule...')
             self.sync()
         finally:
-            self._last_sync = time.time()
+            self._last_sync = monotonic()
 
     def sync(self):
         pass
@@ -257,19 +270,20 @@ class Scheduler(object):
         self.sync()
 
     def add(self, **kwargs):
-        entry = self.Entry(**kwargs)
+        entry = self.Entry(app=self.app, **kwargs)
         self.schedule[entry.name] = entry
         return entry
 
     def _maybe_entry(self, name, entry):
         if isinstance(entry, self.Entry):
+            entry.app = self.app
             return entry
-        return self.Entry(**dict(entry, name=name))
+        return self.Entry(**dict(entry, name=name, app=self.app))
 
     def update_from_dict(self, dict_):
         self.schedule.update(dict(
             (name, self._maybe_entry(name, entry))
-            for name, entry in dict_.items()))
+            for name, entry in items(dict_)))
 
     def merge_inplace(self, b):
         schedule = self.schedule
@@ -281,7 +295,7 @@ class Scheduler(object):
 
         # Update and add new items in the schedule
         for key in B:
-            entry = self.Entry(**dict(b[key], name=key))
+            entry = self.Entry(**dict(b[key], name=key, app=self.app))
             if schedule.get(key):
                 schedule[key].update(entry)
             else:
@@ -291,7 +305,7 @@ class Scheduler(object):
         # callback called for each retry while the connection
         # can't be established.
         def _error_handler(exc, interval):
-            error('Celerybeat: Connection error: %s. '
+            error('beat: Connection error: %s. '
                   'Trying again in %s seconds...', exc, interval)
 
         return self.connection.ensure_connection(
@@ -338,7 +352,7 @@ class PersistentScheduler(Scheduler):
             self._store = self.persistence.open(self.schedule_filename,
                                                 writeback=True)
             entries = self._store.setdefault('entries', {})
-        except Exception, exc:
+        except Exception as exc:
             error('Removing corrupted schedule file %r: %r',
                   self.schedule_filename, exc, exc_info=True)
             self._remove_db()
@@ -373,7 +387,7 @@ class PersistentScheduler(Scheduler):
         self._store.update(__version__=__version__, tz=tz, utc_enabled=utc)
         self.sync()
         debug('Current schedule:\n' + '\n'.join(
-            repr(entry) for entry in entries.itervalues()))
+            repr(entry) for entry in values(entries)))
 
     def get_schedule(self):
         return self._store['entries']
@@ -392,15 +406,15 @@ class PersistentScheduler(Scheduler):
 
     @property
     def info(self):
-        return '    . db -> %s' % (self.schedule_filename, )
+        return '    . db -> {self.schedule_filename}'.format(self=self)
 
 
 class Service(object):
     scheduler_cls = PersistentScheduler
 
-    def __init__(self, max_interval=None, schedule_filename=None,
-                 scheduler_cls=None, app=None):
-        app = self.app = app_or_default(app)
+    def __init__(self, app, max_interval=None, schedule_filename=None,
+                 scheduler_cls=None):
+        self.app = app
         self.max_interval = (max_interval
                              or app.conf.CELERYBEAT_MAX_LOOP_INTERVAL)
         self.scheduler_cls = scheduler_cls or self.scheduler_cls
@@ -415,19 +429,19 @@ class Service(object):
                                 self.scheduler_cls, self.app)
 
     def start(self, embedded_process=False):
-        info('Celerybeat: Starting...')
-        debug('Celerybeat: Ticking with max interval->%s',
+        info('beat: Starting...')
+        debug('beat: Ticking with max interval->%s',
               humanize_seconds(self.scheduler.max_interval))
 
         signals.beat_init.send(sender=self)
         if embedded_process:
             signals.beat_embedded_init.send(sender=self)
-            platforms.set_process_title('celerybeat')
+            platforms.set_process_title('celery beat')
 
         try:
             while not self._is_shutdown.is_set():
                 interval = self.scheduler.tick()
-                debug('Celerybeat: Waking up %s.',
+                debug('beat: Waking up %s.',
                       humanize_seconds(interval, prefix='in '))
                 time.sleep(interval)
         except (KeyboardInterrupt, SystemExit):
@@ -440,7 +454,7 @@ class Service(object):
         self._is_stopped.set()
 
     def stop(self, wait=False):
-        info('Celerybeat: Shutting down...')
+        info('beat: Shutting down...')
         self._is_shutdown.set()
         wait and self._is_stopped.wait()  # block until shutdown done.
 
@@ -488,6 +502,9 @@ else:
 
         def run(self):
             platforms.signals.reset('SIGTERM')
+            platforms.close_open_fds([
+                sys.__stdin__, sys.__stdout__, sys.__stderr__,
+            ])
             self.service.start(embedded_process=True)
 
         def stop(self):
@@ -499,7 +516,7 @@ def EmbeddedService(*args, **kwargs):
     """Return embedded clock service.
 
     :keyword thread: Run threaded instead of as a separate process.
-        Default is :const:`False`.
+        Uses :mod:`multiprocessing` by default, if available.
 
     """
     if kwargs.pop('thread', False) or _Process is None:
