@@ -56,28 +56,35 @@ in any command that also has a `--detach` option.
 
 .. cmdoption:: --umask
 
-    Effective umask (in octal) of the process after detaching.  Inherits
-    the umask of the parent process by default.
+    Effective umask of the process after detaching. Default is 0.
 
 .. cmdoption:: --workdir
 
     Optional directory to change to after detaching.
 
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
 import os
 import re
+import socket
 import sys
 import warnings
+import json
 
 from collections import defaultdict
+from heapq import heappush
+from inspect import getargspec
 from optparse import OptionParser, IndentedHelpFormatter, make_option as Option
+from pprint import pformat
 from types import ModuleType
 
-import celery
+from celery import VERSION_BANNER, Celery, maybe_patch_concurrency
+from celery import signals
 from celery.exceptions import CDeprecationWarning, CPendingDeprecationWarning
-from celery.platforms import EX_FAILURE, EX_USAGE, maybe_patch_concurrency
+from celery.five import items, string, string_t, values
+from celery.platforms import EX_FAILURE, EX_OK, EX_USAGE
+from celery.utils import term
 from celery.utils import text
 from celery.utils.imports import symbol_by_name, import_from_cwd
 
@@ -86,20 +93,70 @@ for warning in (CDeprecationWarning, CPendingDeprecationWarning):
     warnings.simplefilter('once', warning, 0)
 
 ARGV_DISABLED = """
-Unrecognized command line arguments: %s
+Unrecognized command-line arguments: {0}
 
 Try --help?
 """
 
 find_long_opt = re.compile(r'.+?(--.+?)(?:\s|,|$)')
 find_rst_ref = re.compile(r':\w+:`(.+?)`')
+find_sformat = re.compile(r'%(\w)')
+
+__all__ = ['Error', 'UsageError', 'Extensions', 'HelpFormatter',
+           'Command', 'Option', 'daemon_options']
+
+
+class Error(Exception):
+    status = EX_FAILURE
+
+    def __init__(self, reason, status=None):
+        self.reason = reason
+        self.status = status if status is not None else self.status
+        super(Error, self).__init__(reason, status)
+
+    def __str__(self):
+        return self.reason
+    __unicode__ = __str__
+
+
+class UsageError(Error):
+    status = EX_USAGE
+
+
+class Extensions(object):
+
+    def __init__(self, namespace, register):
+        self.names = []
+        self.namespace = namespace
+        self.register = register
+
+    def add(self, cls, name):
+        heappush(self.names, name)
+        self.register(cls, name=name)
+
+    def load(self):
+        try:
+            from pkg_resources import iter_entry_points
+        except ImportError:  # pragma: no cover
+            return
+
+        for ep in iter_entry_points(self.namespace):
+            sym = ':'.join([ep.module_name, ep.attrs[0]])
+            try:
+                cls = symbol_by_name(sym)
+            except (ImportError, SyntaxError) as exc:
+                warnings.warn(
+                    'Cannot load extension {0!r}: {1!r}'.format(sym, exc))
+            else:
+                self.add(cls, ep.name)
+        return self.names
 
 
 class HelpFormatter(IndentedHelpFormatter):
 
     def format_epilog(self, epilog):
         if epilog:
-            return '\n%s\n\n' % epilog
+            return '\n{0}\n\n'.format(epilog)
         return ''
 
     def format_description(self, description):
@@ -108,19 +165,21 @@ class HelpFormatter(IndentedHelpFormatter):
 
 
 class Command(object):
-    """Base class for command line applications.
+    """Base class for command-line applications.
 
     :keyword app: The current app.
     :keyword get_app: Callable returning the current app if no app provided.
 
     """
+    Error = Error
+    UsageError = UsageError
     Parser = OptionParser
 
     #: Arg list used in help.
     args = ''
 
     #: Application version.
-    version = celery.VERSION_BANNER
+    version = VERSION_BANNER
 
     #: If false the parser will raise an exception if positional
     #: args are provided.
@@ -143,6 +202,8 @@ class Command(object):
         Option('--loader', default=None),
         Option('--config', default=None),
         Option('--workdir', default=None, dest='working_directory'),
+        Option('--no-color', '-C', action='store_true', default=None),
+        Option('--quiet', '-q', action='store_true'),
     )
 
     #: Enable if the application should support config from the cmdline.
@@ -160,18 +221,70 @@ class Command(object):
     #: Set to true if this command doesn't have subcommands
     leaf = True
 
-    def __init__(self, app=None, get_app=None):
+    # used by :meth:`say_remote_command_reply`.
+    show_body = True
+    # used by :meth:`say_chat`.
+    show_reply = True
+
+    prog_name = 'celery'
+
+    def __init__(self, app=None, get_app=None, no_color=False,
+                 stdout=None, stderr=None, quiet=False, on_error=None,
+                 on_usage_error=None):
         self.app = app
         self.get_app = get_app or self._get_default_app
+        self.stdout = stdout or sys.stdout
+        self.stderr = stderr or sys.stderr
+        self.no_color = no_color
+        self.colored = term.colored(enabled=not self.no_color)
+        self.quiet = quiet
+        if not self.description:
+            self.description = self.__doc__
+        if on_error:
+            self.on_error = on_error
+        if on_usage_error:
+            self.on_usage_error = on_usage_error
 
     def run(self, *args, **options):
         """This is the body of the command called by :meth:`handle_argv`."""
         raise NotImplementedError('subclass responsibility')
 
-    def execute_from_commandline(self, argv=None):
-        """Execute application from command line.
+    def on_error(self, exc):
+        self.error(self.colored.red('Error: {0}'.format(exc)))
 
-        :keyword argv: The list of command line arguments.
+    def on_usage_error(self, exc):
+        self.handle_error(exc)
+
+    def on_concurrency_setup(self):
+        pass
+
+    def __call__(self, *args, **kwargs):
+        self.verify_args(args)
+        try:
+            ret = self.run(*args, **kwargs)
+            return ret if ret is not None else EX_OK
+        except self.UsageError as exc:
+            self.on_usage_error(exc)
+            return exc.status
+        except self.Error as exc:
+            self.on_error(exc)
+            return exc.status
+
+    def verify_args(self, given, _index=0):
+        S = getargspec(self.run)
+        _index = 1 if S.args and S.args[0] == 'self' else _index
+        required = S.args[_index:-len(S.defaults) if S.defaults else None]
+        missing = required[len(given):]
+        if missing:
+            raise self.UsageError('Missing required {0}: {1}'.format(
+                text.pluralize(len(missing), 'argument'),
+                ', '.join(missing)
+            ))
+
+    def execute_from_commandline(self, argv=None):
+        """Execute application from command-line.
+
+        :keyword argv: The list of command-line arguments.
                        Defaults to ``sys.argv``.
 
         """
@@ -184,11 +297,12 @@ class Command(object):
         # Dump version and exit if '--version' arg set.
         self.early_version(argv)
         argv = self.setup_app_from_commandline(argv)
-        prog_name = os.path.basename(argv[0])
-        return self.handle_argv(prog_name, argv[1:])
+        self.prog_name = os.path.basename(argv[0])
+        return self.handle_argv(self.prog_name, argv[1:])
 
-    def run_from_argv(self, prog_name, argv=None):
-        return self.handle_argv(prog_name, sys.argv if argv is None else argv)
+    def run_from_argv(self, prog_name, argv=None, command=None):
+        return self.handle_argv(prog_name,
+                                sys.argv if argv is None else argv, command)
 
     def maybe_patch_concurrency(self, argv=None):
         argv = argv or sys.argv
@@ -197,24 +311,20 @@ class Command(object):
             maybe_patch_concurrency(argv, *pool_option)
             short_opts, long_opts = pool_option
 
-    def on_concurrency_setup(self):
-        pass
-
     def usage(self, command):
-        """Returns the command line usage string for this app."""
-        return '%%prog [options] %s' % (self.args, )
+        return '%prog {0} [options] {self.args}'.format(command, self=self)
 
     def get_options(self):
-        """Get supported command line options."""
+        """Get supported command-line options."""
         return self.option_list
 
     def expanduser(self, value):
-        if isinstance(value, basestring):
+        if isinstance(value, string_t):
             return os.path.expanduser(value)
         return value
 
-    def handle_argv(self, prog_name, argv):
-        """Parses command line arguments from ``argv`` and dispatches
+    def handle_argv(self, prog_name, argv, command=None):
+        """Parse command-line arguments from ``argv`` and dispatch
         to :meth:`run`.
 
         :param prog_name: The program name (``argv[0]``).
@@ -224,13 +334,14 @@ class Command(object):
         and ``argv`` contains positional arguments.
 
         """
-        options, args = self.prepare_args(*self.parse_options(prog_name, argv))
-        return self.run(*args, **options)
+        options, args = self.prepare_args(
+            *self.parse_options(prog_name, argv, command))
+        return self(*args, **options)
 
     def prepare_args(self, options, args):
         if options:
             options = dict((k, self.expanduser(v))
-                           for k, v in vars(options).iteritems()
+                           for k, v in items(vars(options))
                            if not k.startswith('_'))
         args = [self.expanduser(arg) for arg in args]
         self.check_args(args)
@@ -238,25 +349,36 @@ class Command(object):
 
     def check_args(self, args):
         if not self.supports_args and args:
-            self.die(ARGV_DISABLED % (', '.join(args, )), EX_USAGE)
+            self.die(ARGV_DISABLED.format(', '.join(args)), EX_USAGE)
+
+    def error(self, s):
+        self.out(s, fh=self.stderr)
+
+    def out(self, s, fh=None):
+        print(s, file=fh or self.stdout)
 
     def die(self, msg, status=EX_FAILURE):
-        sys.stderr.write(msg + '\n')
+        self.error(msg)
         sys.exit(status)
 
     def early_version(self, argv):
         if '--version' in argv:
-            sys.stdout.write('%s\n' % self.version)
+            print(self.version, file=self.stdout)
             sys.exit(0)
 
-    def parse_options(self, prog_name, arguments):
+    def parse_options(self, prog_name, arguments, command=None):
         """Parse the available options."""
         # Don't want to load configuration to just print the version,
         # so we handle --version manually here.
-        parser = self.create_parser(prog_name)
-        return parser.parse_args(arguments)
+        self.parser = self.create_parser(prog_name, command)
+        return self.parser.parse_args(arguments)
 
     def create_parser(self, prog_name, command=None):
+        option_list = (
+            self.preload_options +
+            self.get_options() +
+            tuple(self.app.user_options['preload'])
+        )
         return self.prepare_parser(self.Parser(
             prog=prog_name,
             usage=self.usage(command),
@@ -264,19 +386,25 @@ class Command(object):
             epilog=self.epilog,
             formatter=HelpFormatter(),
             description=self.description,
-            option_list=(self.preload_options + self.get_options())))
+            option_list=option_list,
+        ))
 
     def prepare_parser(self, parser):
         docs = [self.parse_doc(doc) for doc in (self.doc, __doc__) if doc]
         for doc in docs:
-            for long_opt, help in doc.iteritems():
+            for long_opt, help in items(doc):
                 option = parser.get_option(long_opt)
                 if option is not None:
-                    option.help = ' '.join(help) % {'default': option.default}
+                    option.help = ' '.join(help).format(default=option.default)
         return parser
 
     def setup_app_from_commandline(self, argv):
         preload_options = self.parse_preload_options(argv)
+        quiet = preload_options.get('quiet')
+        if quiet is not None:
+            self.quiet = quiet
+        self.colored.enabled = \
+            not preload_options.get('no_color', self.no_color)
         workdir = preload_options.get('working_directory')
         if workdir:
             os.chdir(workdir)
@@ -304,7 +432,16 @@ class Command(object):
             if self.enable_config_from_cmdline:
                 argv = self.process_cmdline_config(argv)
         else:
-            self.app = celery.Celery()
+            self.app = Celery()
+
+        user_preload = tuple(self.app.user_options['preload'] or ())
+        if user_preload:
+            user_options = self.preparse_options(argv, user_preload)
+            for user_option in user_preload:
+                user_options.setdefault(user_option.dest, user_option.default)
+            signals.user_preload_options.send(
+                sender=self, app=self.app, options=user_options,
+            )
         return argv
 
     def find_app(self, app):
@@ -314,9 +451,23 @@ class Command(object):
             # last part was not an attribute, but a module
             sym = import_from_cwd(app)
         if isinstance(sym, ModuleType):
-            if getattr(sym, '__path__', None):
-                return self.find_app('%s.celery:' % (app.replace(':', ''), ))
-            return sym.celery
+            try:
+                return sym.app
+            except AttributeError:
+                try:
+                    return sym.celery
+                except AttributeError:
+                    if getattr(sym, '__path__', None):
+                        try:
+                            return self.find_app(
+                                '{0}.celery:'.format(app.replace(':', '')),
+                            )
+                        except ImportError:
+                            pass
+                    for suspect in values(vars(sym)):
+                        if isinstance(suspect, Celery):
+                            return suspect
+                    raise
         return sym
 
     def symbol_by_name(self, name):
@@ -333,25 +484,36 @@ class Command(object):
         return argv
 
     def parse_preload_options(self, args):
+        return self.preparse_options(args, self.preload_options)
+
+    def preparse_options(self, args, options):
         acc = {}
         opts = {}
-        for opt in self.preload_options:
+        for opt in options:
             for t in (opt._long_opts, opt._short_opts):
-                opts.update(dict(zip(t, [opt.dest] * len(t))))
+                opts.update(dict(zip(t, [opt] * len(t))))
         index = 0
         length = len(args)
         while index < length:
             arg = args[index]
-            if arg.startswith('--') and '=' in arg:
-                key, value = arg.split('=', 1)
-                dest = opts.get(key)
-                if dest:
-                    acc[dest] = value
+            if arg.startswith('--'):
+                if '=' in arg:
+                    key, value = arg.split('=', 1)
+                    opt = opts.get(key)
+                    if opt:
+                        acc[opt.dest] = value
+                else:
+                    opt = opts.get(arg)
+                    if opt and opt.action == 'store_true':
+                        acc[opt.dest] = True
             elif arg.startswith('-'):
-                dest = opts.get(arg)
-                if dest:
-                    acc[dest] = args[index + 1]
-                    index += 1
+                opt = opts.get(arg)
+                if opt:
+                    if opt.takes_value():
+                        acc[opt.dest] = args[index + 1]
+                        index += 1
+                    elif opt.action == 'store_true':
+                        acc[opt.dest] = True
             index += 1
         return acc
 
@@ -369,7 +531,7 @@ class Command(object):
         return options
 
     def with_pool_option(self, argv):
-        """Returns tuple of ``(short_opts, long_opts)`` if the command
+        """Return tuple of ``(short_opts, long_opts)`` if the command
         supports a pool argument, and used to monkey patch eventlet/gevent
         environments as early as possible.
 
@@ -378,10 +540,66 @@ class Command(object):
         """
         pass
 
+    def simple_format(self, s, match=find_sformat, expand=r'\1', **keys):
+        if s:
+            host = socket.gethostname()
+            name, _, domain = host.partition('.')
+            keys = dict({'%': '%', 'h': host, 'n': name, 'd': domain}, **keys)
+            return match.sub(lambda m: keys[m.expand(expand)], s)
+        return s
+
     def _get_default_app(self, *args, **kwargs):
         from celery._state import get_current_app
         return get_current_app()  # omit proxy
 
+    def pretty_list(self, n):
+        c = self.colored
+        if not n:
+            return '- empty -'
+        return '\n'.join(
+            str(c.reset(c.white('*'), ' {0}'.format(item))) for item in n
+        )
+
+    def pretty_dict_ok_error(self, n):
+        c = self.colored
+        try:
+            return (c.green('OK'),
+                    text.indent(self.pretty(n['ok'])[1], 4))
+        except KeyError:
+            pass
+        return (c.red('ERROR'),
+                text.indent(self.pretty(n['error'])[1], 4))
+
+    def say_remote_command_reply(self, replies):
+        c = self.colored
+        node = next(iter(replies))  # <-- take first.
+        reply = replies[node]
+        status, preply = self.pretty(reply)
+        self.say_chat('->', c.cyan(node, ': ') + status,
+                      text.indent(preply, 4) if self.show_reply else '')
+
+    def pretty(self, n):
+        OK = str(self.colored.green('OK'))
+        if isinstance(n, list):
+            return OK, self.pretty_list(n)
+        if isinstance(n, dict):
+            if 'ok' in n or 'error' in n:
+                return self.pretty_dict_ok_error(n)
+            else:
+                return OK, json.dumps(n, sort_keys=True, indent=4)
+        if isinstance(n, string_t):
+            return OK, string(n)
+        return OK, pformat(n)
+
+    def say_chat(self, direction, title, body=''):
+        c = self.colored
+        if direction == '<-' and self.quiet:
+            return
+        dirstr = not self.quiet and c.bold(c.white(direction), ' ') or ''
+        self.out(c.reset(dirstr, title))
+        if body and self.show_body:
+            self.out(body)
+
 
 def daemon_options(default_pidfile=None, default_logfile=None):
     return (
@@ -389,5 +607,5 @@ def daemon_options(default_pidfile=None, default_logfile=None):
         Option('--pidfile', default=default_pidfile),
         Option('--uid', default=None),
         Option('--gid', default=None),
-        Option('--umask', default=None),
+        Option('--umask', default=0, type='int'),
     )
