@@ -6,218 +6,80 @@ The :program:`celery` umbrella command.
 .. program:: celery
 
 """
-from __future__ import absolute_import
-from __future__ import with_statement
+from __future__ import absolute_import, unicode_literals
 
 import anyjson
-import heapq
 import os
 import sys
-import warnings
 
+from functools import partial
 from importlib import import_module
-from pprint import pformat
 
+from celery.five import string_t, values
 from celery.platforms import EX_OK, EX_FAILURE, EX_UNAVAILABLE, EX_USAGE
 from celery.utils import term
 from celery.utils import text
-from celery.utils.functional import memoize
-from celery.utils.imports import symbol_by_name
 from celery.utils.timeutils import maybe_iso8601
 
-from celery.bin.base import Command as BaseCommand, Option
+# Cannot use relative imports here due to a Windows issue (#1111).
+from celery.bin.base import Command, Option, Extensions
+
+# Import commands from other modules
+from celery.bin.amqp import amqp
+from celery.bin.beat import beat
+from celery.bin.events import events
+from celery.bin.graph import graph
+from celery.bin.worker import worker
+
+__all__ = ['CeleryCommand', 'main']
 
 HELP = """
 ---- -- - - ---- Commands- -------------- --- ------------
 
-%(commands)s
+{commands}
 ---- -- - - --------- -- - -------------- --- ------------
 
-Type '%(prog_name)s <command> --help' for help using a specific command.
+Type '{prog_name} <command> --help' for help using a specific command.
 """
 
-commands = {}
+MIGRATE_PROGRESS_FMT = """\
+Migrating task {state.count}/{state.strtotal}: \
+{body[task]}[{body[id]}]\
+"""
+
+DEBUG = os.environ.get('C_DEBUG', False)
 
 command_classes = [
     ('Main', ['worker', 'events', 'beat', 'shell', 'multi', 'amqp'], 'green'),
     ('Remote Control', ['status', 'inspect', 'control'], 'blue'),
     ('Utils', ['purge', 'list', 'migrate', 'call', 'result', 'report'], None),
 ]
-
-
-@memoize()
-def _get_extension_classes():
-    extensions = []
-    command_classes.append(('Extensions', extensions, 'magenta'))
-    return extensions
-
-
-def ensure_broadcast_supported(app):
-    if app.connection().transport.driver_type == 'sql':
-        raise Error('SQL broker transports does not support broadcast')
-
-
-class Error(Exception):
-
-    def __init__(self, reason, status=EX_FAILURE):
-        self.reason = reason
-        self.status = status
-        super(Error, self).__init__(reason, status)
-
-    def __str__(self):
-        return self.reason
-
-
-def command(fun, name=None, sortpri=0):
-    commands[name or fun.__name__] = fun
-    fun.sortpri = sortpri
-    return fun
-
-
-def load_extension_commands(namespace='celery.commands'):
-    try:
-        from pkg_resources import iter_entry_points
-    except ImportError:
-        return
-
-    for ep in iter_entry_points(namespace):
-        sym = ':'.join([ep.module_name, ep.attrs[0]])
-        try:
-            cls = symbol_by_name(sym)
-        except (ImportError, SyntaxError), exc:
-            warnings.warn('Cannot load extension %r: %r' % (sym, exc))
-        else:
-            heapq.heappush(_get_extension_classes(), ep.name)
-            command(cls, name=ep.name)
-
-
-class Command(BaseCommand):
-    help = ''
-    args = ''
-    prog_name = 'celery'
-    show_body = True
-    show_reply = True
-
-    option_list = (
-        Option('--quiet', '-q', action='store_true'),
-        Option('--no-color', '-C', action='store_true', default=None),
+if DEBUG:  # pragma: no cover
+    command_classes.append(
+        ('Debug', ['graph'], 'red'),
     )
 
-    def __init__(self, app=None, no_color=False, stdout=sys.stdout,
-                 stderr=sys.stderr, show_reply=True):
-        super(Command, self).__init__(app=app)
-        self.colored = term.colored(enabled=not no_color)
-        self.stdout = stdout
-        self.stderr = stderr
-        self.quiet = False
-        if show_reply is not None:
-            self.show_reply = show_reply
 
-    def __call__(self, *args, **kwargs):
-        try:
-            ret = self.run(*args, **kwargs)
-        except Error, exc:
-            self.error(self.colored.red('Error: %s' % exc))
-            return exc.status
-
-        return ret if ret is not None else EX_OK
-
-    def show_help(self, command):
-        self.run_from_argv(self.prog_name, [command, '--help'])
-        return EX_USAGE
-
-    def error(self, s):
-        self.out(s, fh=self.stderr)
-
-    def out(self, s, fh=None):
-        s = str(s)
-        if not s.endswith('\n'):
-            s += '\n'
-        (fh or self.stdout).write(s)
-
-    def run_from_argv(self, prog_name, argv):
-        self.prog_name = prog_name
-        self.command = argv[0]
-        self.arglist = argv[1:]
-        self.parser = self.create_parser(self.prog_name, self.command)
-        options, args = self.prepare_args(
-            *self.parser.parse_args(self.arglist))
-        self.colored = term.colored(enabled=not options['no_color'])
-        self.quiet = options.get('quiet', False)
-        self.show_body = options.get('show_body', True)
-        return self(*args, **options)
-
-    def usage(self, command):
-        return '%%prog %s [options] %s' % (command, self.args)
-
-    def prettify_list(self, n):
-        c = self.colored
-        if not n:
-            return '- empty -'
-        return '\n'.join(str(c.reset(c.white('*'), ' %s' % (item, )))
-                         for item in n)
-
-    def prettify_dict_ok_error(self, n):
-        c = self.colored
-        try:
-            return (c.green('OK'),
-                    text.indent(self.prettify(n['ok'])[1], 4))
-        except KeyError:
-            pass
-        return (c.red('ERROR'),
-                text.indent(self.prettify(n['error'])[1], 4))
-
-    def say_remote_command_reply(self, replies):
-        c = self.colored
-        node = iter(replies).next()  # <-- take first.
-        reply = replies[node]
-        status, preply = self.prettify(reply)
-        self.say_chat('->', c.cyan(node, ': ') + status,
-                      text.indent(preply, 4) if self.show_reply else '')
-
-    def prettify(self, n):
-        OK = str(self.colored.green('OK'))
-        if isinstance(n, list):
-            return OK, self.prettify_list(n)
-        if isinstance(n, dict):
-            if 'ok' in n or 'error' in n:
-                return self.prettify_dict_ok_error(n)
-        if isinstance(n, basestring):
-            return OK, unicode(n)
-        return OK, pformat(n)
-
-    def say_chat(self, direction, title, body=''):
-        c = self.colored
-        if direction == '<-' and self.quiet:
-            return
-        dirstr = not self.quiet and c.bold(c.white(direction), ' ') or ''
-        self.out(c.reset(dirstr, title))
-        if body and self.show_body:
-            self.out(body)
-
-    @property
-    def description(self):
-        return self.__doc__
+def determine_exit_status(ret):
+    if isinstance(ret, int):
+        return ret
+    return EX_OK if ret else EX_FAILURE
 
 
-class Delegate(Command):
-
-    def __init__(self, *args, **kwargs):
-        super(Delegate, self).__init__(*args, **kwargs)
-
-        self.target = symbol_by_name(self.Command)(app=self.app)
-        self.args = self.target.args
-
-    def get_options(self):
-        return self.option_list + self.target.get_options()
-
-    def create_parser(self, prog_name, command):
-        parser = super(Delegate, self).create_parser(prog_name, command)
-        return self.target.prepare_parser(parser)
-
-    def run(self, *args, **kwargs):
-        self.target.check_args(args)
-        return self.target.run(*args, **kwargs)
+def main(argv=None):
+    # Fix for setuptools generated scripts, so that it will
+    # work with multiprocessing fork emulation.
+    # (see multiprocessing.forking.get_preparation_data())
+    try:
+        if __name__ != '__main__':  # pragma: no cover
+            sys.modules['__main__'] = sys.modules[__name__]
+        cmd = CeleryCommand()
+        cmd.maybe_patch_concurrency()
+        from billiard import freeze_support
+        freeze_support()
+        cmd.execute_from_commandline(argv)
+    except KeyboardInterrupt:
+        pass
 
 
 class multi(Command):
@@ -227,85 +89,11 @@ class multi(Command):
     def get_options(self):
         return ()
 
-    def run_from_argv(self, prog_name, argv):
-        from celery.bin.celeryd_multi import MultiTool
-        return MultiTool().execute_from_commandline(argv, prog_name)
-multi = command(multi)
-
-
-class worker(Delegate):
-    """Start worker instance.
-
-    Examples::
-
-        celery worker --app=proj -l info
-        celery worker -A proj -l info -Q hipri,lopri
-
-        celery worker -A proj --concurrency=4
-        celery worker -A proj --concurrency=1000 -P eventlet
-
-        celery worker --autoscale=10,0
-    """
-    Command = 'celery.bin.celeryd:WorkerCommand'
-worker = command(worker, sortpri=01)
-
-
-class events(Delegate):
-    """Event-stream utilities.
-
-    Commands::
-
-        celery events --app=proj
-            start graphical monitor (requires curses)
-        celery events -d --app=proj
-            dump events to screen.
-        celery events -b amqp://
-        celery events -C <camera> [options]
-            run snapshot camera.
-
-    Examples::
-
-        celery events
-        celery events -d
-        celery events -C mod.attr -F 1.0 --detach --maxrate=100/m -l info
-    """
-    Command = 'celery.bin.celeryev:EvCommand'
-events = command(events, sortpri=10)
-
-
-class beat(Delegate):
-    """Start the celerybeat periodic task scheduler.
-
-    Examples::
-
-        celery beat -l info
-        celery beat -s /var/run/celerybeat/schedule --detach
-        celery beat -S djcelery.schedulers.DatabaseScheduler
-
-    """
-    Command = 'celery.bin.celerybeat:BeatCommand'
-beat = command(beat, sortpri=20)
-
-
-class amqp(Delegate):
-    """AMQP Administration Shell.
-
-    Also works for non-amqp transports.
-
-    Examples::
-
-        celery amqp
-            start shell mode
-        celery amqp help
-            show list of commands
-
-        celery amqp exchange.delete name
-        celery amqp queue.delete queue
-        celery amqp queue.delete queue yes yes
-
-    """
-    Command = 'celery.bin.camqadm:AMQPAdminCommand'
-amqp = command(amqp, sortpri=30)
+    def run_from_argv(self, prog_name, argv, command=None):
+        from celery.bin.multi import MultiTool
+        return MultiTool().execute_from_commandline(
+            [command] + argv, prog_name,
+        )
 
 
 class list_(Command):
@@ -323,10 +111,9 @@ class list_(Command):
         try:
             bindings = management.get_bindings()
         except NotImplementedError:
-            raise Error('Your transport cannot list bindings.')
+            raise self.Error('Your transport cannot list bindings.')
 
-        fmt = lambda q, e, r: self.out('%s %s %s' % (q.ljust(28),
-                                                     e.ljust(28), r))
+        fmt = lambda q, e, r: self.out('{0:<28} {1:<28} {2}'.format(q, e, r))
         fmt('Queue', 'Exchange', 'Routing Key')
         fmt('-' * 16, '-' * 16, '-' * 16)
         for b in bindings:
@@ -336,14 +123,15 @@ class list_(Command):
         topics = {'bindings': self.list_bindings}
         available = ', '.join(topics)
         if not what:
-            raise Error('You must specify what to list (%s)' % available)
+            raise self.UsageError(
+                'You must specify one of {0}'.format(available))
         if what not in topics:
-            raise Error('unknown topic %r (choose one of: %s)' % (
-                what, available))
+            raise self.UsageError(
+                'unknown topic {0!r} (choose one of: {1})'.format(
+                    what, available))
         with self.app.connection() as conn:
             self.app.amqp.TaskConsumer(conn).declare()
             topics[what](conn.manager)
-list_ = command(list_, 'list')
 
 
 class call(Command):
@@ -371,12 +159,12 @@ class call(Command):
     def run(self, name, *_, **kw):
         # Positional args.
         args = kw.get('args') or ()
-        if isinstance(args, basestring):
+        if isinstance(args, string_t):
             args = anyjson.loads(args)
 
         # Keyword args.
         kwargs = kw.get('kwargs') or {}
-        if isinstance(kwargs, basestring):
+        if isinstance(kwargs, string_t):
             kwargs = anyjson.loads(kwargs)
 
         # Expires can be int/float.
@@ -399,7 +187,6 @@ class call(Command):
                                  eta=maybe_iso8601(kw.get('eta')),
                                  expires=expires)
         self.out(res.id)
-call = command(call)
 
 
 class purge(Command):
@@ -408,17 +195,17 @@ class purge(Command):
     WARNING: There is no undo operation for this command.
 
     """
+    fmt_purged = 'Purged {mnum} {messages} from {qnum} known task {queues}.'
+    fmt_empty = 'No messages purged from {qnum} {queues}'
+
     def run(self, *args, **kwargs):
         queues = len(self.app.amqp.queues)
-        messages_removed = self.app.control.purge()
-        if messages_removed:
-            self.out('Purged %s %s from %s known task %s.' % (
-                messages_removed, text.pluralize(messages_removed, 'message'),
-                queues, text.pluralize(queues, 'queue')))
-        else:
-            self.out('No messages purged from %s known %s' % (
-                queues, text.pluralize(queues, 'queue')))
-purge = command(purge)
+        messages = self.app.control.purge()
+        fmt = self.fmt_purged if messages else self.fmt_empty
+        self.out(fmt.format(
+            mnum=messages, qnum=queues,
+            messages=text.pluralize(messages, 'message'),
+            queues=text.pluralize(queues, 'queue')))
 
 
 class result(Command):
@@ -450,8 +237,7 @@ class result(Command):
             value = result.traceback
         else:
             value = result.get()
-        self.out(self.prettify(value)[1])
-result = command(result)
+        self.out(self.pretty(value)[1])
 
 
 class _RemoteControl(Command):
@@ -464,6 +250,11 @@ class _RemoteControl(Command):
         Option('--destination', '-d',
                help='Comma separated list of destination node names.'))
 
+    def __init__(self, *args, **kwargs):
+        self.show_body = kwargs.pop('show_body', True)
+        self.show_reply = kwargs.pop('show_reply', True)
+        super(_RemoteControl, self).__init__(*args, **kwargs)
+
     @classmethod
     def get_command_info(self, command,
                          indent=0, prefix='', color=None, help=False):
@@ -475,8 +266,9 @@ class _RemoteControl(Command):
             # see if it uses args.
             meth = getattr(self, command)
             return text.join([
-                '|' + text.indent('%s%s %s' % (prefix, color(command),
-                                               meth.__doc__), indent), help,
+                '|' + text.indent('{0}{1} {2}'.format(
+                    prefix, color(command), meth.__doc__), indent),
+                help,
             ])
 
         except AttributeError:
@@ -499,7 +291,7 @@ class _RemoteControl(Command):
         ])
 
     def usage(self, command):
-        return '%%prog %s [options] %s <command> [arg1 .. argN]' % (
+        return '%prog {0} [options] {1} <command> [arg1 .. argN]'.format(
             command, self.args)
 
     def call(self, *args, **kwargs):
@@ -507,46 +299,39 @@ class _RemoteControl(Command):
 
     def run(self, *args, **kwargs):
         if not args:
-            raise Error('Missing %s method. See --help' % self.name)
+            raise self.UsageError(
+                'Missing {0.name} method. See --help'.format(self))
         return self.do_call_method(args, **kwargs)
 
     def do_call_method(self, args, **kwargs):
         method = args[0]
         if method == 'help':
-            raise Error("Did you mean '%s --help'?" % self.name)
+            raise self.Error("Did you mean '{0.name} --help'?".format(self))
         if method not in self.choices:
-            raise Error('Unknown %s method %s' % (self.name, method))
+            raise self.UsageError(
+                'Unknown {0.name} method {1}'.format(self, method))
 
-        ensure_broadcast_supported(self.app)
+        if self.app.connection().transport.driver_type == 'sql':
+            raise self.Error('Broadcast not supported by SQL broker transport')
 
         destination = kwargs.get('destination')
         timeout = kwargs.get('timeout') or self.choices[method][0]
-        if destination and isinstance(destination, basestring):
-            destination = [v.strip() for v in destination.split(',')]
+        if destination and isinstance(destination, string_t):
+            destination = [dest.strip() for dest in destination.split(',')]
 
         try:
             handler = getattr(self, method)
         except AttributeError:
             handler = self.call
 
-        # XXX Python 2.5 does not support X(*args, foo=1)
-        kwargs = {"timeout": timeout, "destination": destination,
-                  "callback": self.say_remote_command_reply}
-        replies = handler(method, *args[1:], **kwargs)
+        replies = handler(method, *args[1:], timeout=timeout,
+                          destination=destination,
+                          callback=self.say_remote_command_reply)
         if not replies:
-            raise Error('No nodes replied within time constraint.',
-                        status=EX_UNAVAILABLE)
+            raise self.Error('No nodes replied within time constraint.',
+                             status=EX_UNAVAILABLE)
         return replies
 
-    def say(self, direction, title, body=''):
-        c = self.colored
-        if direction == '<-' and self.quiet:
-            return
-        dirstr = not self.quiet and c.bold(c.white(direction), ' ') or ''
-        self.out(c.reset(dirstr, title))
-        if body and self.show_body:
-            self.out(body)
-
 
 class inspect(_RemoteControl):
     """Inspect the worker at runtime.
@@ -556,8 +341,8 @@ class inspect(_RemoteControl):
     Examples::
 
         celery inspect active --timeout=5
-        celery inspect scheduled -d worker1.example.com
-        celery inspect revoked -d w1.e.com,w2.e.com
+        celery inspect scheduled -d worker1@example.com
+        celery inspect revoked -d w1@e.com,w2@e.com
 
     """
     name = 'inspect'
@@ -570,13 +355,20 @@ class inspect(_RemoteControl):
         'revoked': (1.0, 'dump of revoked task ids'),
         'registered': (1.0, 'dump of registered tasks'),
         'ping': (0.2, 'ping worker(s)'),
-        'report': (1.0, 'get bugreport info')
+        'clock': (1.0, 'get value of logical clock'),
+        'conf': (1.0, 'dump worker configuration'),
+        'report': (1.0, 'get bugreport info'),
+        'memsample': (1.0, 'sample memory (requires psutil)'),
+        'memdump': (1.0, 'dump memory samples (requires psutil)'),
+        'objgraph': (60.0, 'create object graph (requires objgraph)'),
     }
 
     def call(self, method, *args, **options):
         i = self.app.control.inspect(**options)
         return getattr(i, method)(*args)
-inspect = command(inspect)
+
+    def objgraph(self, type_='Request', *args, **kwargs):
+        return self.call('objgraph', type_)
 
 
 class control(_RemoteControl):
@@ -587,7 +379,7 @@ class control(_RemoteControl):
     Examples::
 
         celery control enable_events --timeout=5
-        celery control -d worker1.example.com enable_events
+        celery control -d worker1@example.com enable_events
         celery control -d w1.e.com,w2.e.com enable_events
 
         celery control -d w1.e.com add_consumer queue_name
@@ -612,21 +404,19 @@ class control(_RemoteControl):
     }
 
     def call(self, method, *args, **options):
-        # XXX Python 2.5 doesn't support X(*args, reply=True, **kwargs)
-        return getattr(self.app.control, method)(
-            *args, **dict(options, reply=True))
+        return getattr(self.app.control, method)(*args, reply=True, **options)
 
     def pool_grow(self, method, n=1, **kwargs):
         """[N=1]"""
-        return self.call(method, n, **kwargs)
+        return self.call(method, int(n), **kwargs)
 
     def pool_shrink(self, method, n=1, **kwargs):
         """[N=1]"""
-        return self.call(method, n, **kwargs)
+        return self.call(method, int(n), **kwargs)
 
     def autoscale(self, method, max=None, min=None, **kwargs):
         """[max] [min]"""
-        return self.call(method, max, min, **kwargs)
+        return self.call(method, int(max), int(min), **kwargs)
 
     def rate_limit(self, method, task_name, rate_limit, **kwargs):
         """<task_name> <rate_limit> (e.g. 5/s | 5/m | 5/h)>"""
@@ -634,7 +424,8 @@ class control(_RemoteControl):
 
     def time_limit(self, method, task_name, soft, hard=None, **kwargs):
         """<task_name> <soft_secs> [hard_secs]"""
-        return self.call(method, task_name, soft, hard, reply=True, **kwargs)
+        return self.call(method, task_name,
+                         float(soft), float(hard), reply=True, **kwargs)
 
     def add_consumer(self, method, queue, exchange=None,
                      exchange_type='direct', routing_key=None, **kwargs):
@@ -645,7 +436,6 @@ class control(_RemoteControl):
     def cancel_consumer(self, method, queue, **kwargs):
         """<queue>"""
         return self.call(method, queue, reply=True, **kwargs)
-control = command(control)
 
 
 class status(Command):
@@ -653,20 +443,20 @@ class status(Command):
     option_list = inspect.option_list
 
     def run(self, *args, **kwargs):
-        replies = inspect(
+        I = inspect(
             app=self.app,
             no_color=kwargs.get('no_color', False),
             stdout=self.stdout, stderr=self.stderr,
-            show_reply=False).run(
-                'ping', **dict(kwargs, quiet=True, show_body=False))
+            show_reply=False, show_body=False, quiet=True,
+        )
+        replies = I.run('ping', **kwargs)
         if not replies:
-            raise Error('No nodes replied within time constraint',
-                        status=EX_UNAVAILABLE)
+            raise self.Error('No nodes replied within time constraint',
+                             status=EX_UNAVAILABLE)
         nodecount = len(replies)
         if not kwargs.get('quiet', False):
-            self.out('\n%s %s online.' % (nodecount,
-                                          text.pluralize(nodecount, 'node')))
-status = command(status)
+            self.out('\n{0} {1} online.'.format(
+                nodecount, text.pluralize(nodecount, 'node')))
 
 
 class migrate(Command):
@@ -695,22 +485,19 @@ class migrate(Command):
         Option('--forever', '-F', action='store_true',
                help='Continually migrate tasks until killed.'),
     )
+    progress_fmt = MIGRATE_PROGRESS_FMT
 
     def on_migrate_task(self, state, body, message):
-        self.out('Migrating task %s/%s: %s[%s]' % (
-            state.count, state.strtotal, body['task'], body['id']))
+        self.out(self.progress_fmt.format(state=state, body=body))
 
-    def run(self, *args, **kwargs):
-        if len(args) != 2:
-            return self.show_help('migrate')
+    def run(self, source, destination, **kwargs):
         from kombu import Connection
         from celery.contrib.migrate import migrate_tasks
 
-        migrate_tasks(Connection(args[0]),
-                      Connection(args[1]),
+        migrate_tasks(Connection(source),
+                      Connection(destination),
                       callback=self.on_migrate_task,
                       **kwargs)
-migrate = command(migrate)
 
 
 class shell(Command):  # pragma: no cover
@@ -776,7 +563,7 @@ class shell(Command):  # pragma: no cover
 
         if not without_tasks:
             self.locals.update(dict(
-                (task.__name__, task) for task in self.app.tasks.itervalues()
+                (task.__name__, task) for task in values(self.app.tasks)
                 if not task.name.startswith('celery.')),
             )
 
@@ -826,22 +613,19 @@ class shell(Command):  # pragma: no cover
         import bpython
         bpython.embed(self.locals)
 
-shell = command(shell)
-
 
 class help(Command):
     """Show help screen and exit."""
 
     def usage(self, command):
-        return '%%prog <command> [options] %s' % (self.args, )
+        return '%prog <command> [options] {0.args}'.format(self)
 
     def run(self, *args, **kwargs):
         self.parser.print_help()
-        self.out(HELP % {'prog_name': self.prog_name,
-                         'commands': CeleryCommand.list_commands()})
+        self.out(HELP.format(prog_name=self.prog_name,
+                             commands=CeleryCommand.list_commands()))
 
         return EX_USAGE
-help = command(help)
 
 
 class report(Command):
@@ -850,14 +634,39 @@ class report(Command):
     def run(self, *args, **kwargs):
         self.out(self.app.bugreport())
         return EX_OK
-report = command(report)
 
 
-class CeleryCommand(BaseCommand):
-    commands = commands
+class CeleryCommand(Command):
+    namespace = 'celery'
+    ext_fmt = '{self.namespace}.commands'
+    commands = {
+        'amqp': amqp,
+        'beat': beat,
+        'call': call,
+        'control': control,
+        'events': events,
+        'graph': graph,
+        'help': help,
+        'inspect': inspect,
+        'list': list_,
+        'migrate': migrate,
+        'multi': multi,
+        'purge': purge,
+        'report': report,
+        'result': result,
+        'shell': shell,
+        'status': status,
+        'worker': worker,
+
+    }
     enable_config_from_cmdline = True
     prog_name = 'celery'
 
+    @classmethod
+    def register_command(cls, fun, name=None):
+        cls.commands[name or fun.__name__] = fun
+        return fun
+
     def execute(self, command, argv=None):
         try:
             cls = self.commands[command]
@@ -865,9 +674,26 @@ class CeleryCommand(BaseCommand):
             cls, argv = self.commands['help'], ['help']
         cls = self.commands.get(command) or self.commands['help']
         try:
-            return cls(app=self.app).run_from_argv(self.prog_name, argv)
-        except Error:
-            return self.execute('help', argv)
+            return cls(
+                app=self.app, on_error=self.on_error,
+                on_usage_error=partial(self.on_usage_error, command=command),
+            ).run_from_argv(self.prog_name, argv[1:], command=argv[0])
+        except self.UsageError as exc:
+            self.on_usage_error(exc)
+            return exc.status
+        except self.Error as exc:
+            self.on_error(exc)
+            return exc.status
+
+    def on_usage_error(self, exc, command=None):
+        if command:
+            helps = '{self.prog_name} {command} --help'
+        else:
+            helps = '{self.prog_name} --help'
+        self.error(self.colored.magenta("Error: {0}".format(exc)))
+        self.error("""Please try '{0}'""".format(helps.format(
+            self=self, command=command,
+        )))
 
     def remove_options_at_beginning(self, argv, index=0):
         if argv:
@@ -882,8 +708,13 @@ class CeleryCommand(BaseCommand):
                 index += 1
         return []
 
+    def prepare_prog_name(self, name):
+        if name == '__main__.py':
+            return sys.modules['__main__'].__file__
+        return name
+
     def handle_argv(self, prog_name, argv):
-        self.prog_name = prog_name
+        self.prog_name = self.prepare_prog_name(prog_name)
         argv = self.remove_options_at_beginning(argv)
         _, argv = self.prepare_args(None, argv)
         try:
@@ -906,12 +737,13 @@ class CeleryCommand(BaseCommand):
     def get_command_info(self, command, indent=0, color=None):
         colored = term.colored().names[color] if color else lambda x: x
         obj = self.commands[command]
+        cmd = 'celery {0}'.format(colored(command))
         if obj.leaf:
-            return '|' + text.indent('celery %s' % colored(command), indent)
+            return '|' + text.indent(cmd, indent)
         return text.join([
             ' ',
-            '|' + text.indent('celery %s --help' % colored(command), indent),
-            obj.list_commands(indent, 'celery %s' % command, colored),
+            '|' + text.indent('{0} --help'.format(cmd), indent),
+            obj.list_commands(indent, 'celery {0}'.format(command), colored),
         ])
 
     @classmethod
@@ -920,7 +752,7 @@ class CeleryCommand(BaseCommand):
         ret = []
         for cls, commands, color in command_classes:
             ret.extend([
-                text.indent('+ %s: ' % white(cls), indent),
+                text.indent('+ {0}: '.format(white(cls)), indent),
                 '\n'.join(self.get_command_info(command, indent + 4, color)
                           for command in commands),
                 ''
@@ -928,35 +760,26 @@ class CeleryCommand(BaseCommand):
         return '\n'.join(ret).strip()
 
     def with_pool_option(self, argv):
-        if len(argv) > 1 and argv[1] == 'worker':
+        if len(argv) > 1 and 'worker' in argv[0:3]:
             # this command supports custom pools
             # that may have to be loaded as early as possible.
             return (['-P'], ['--pool'])
 
     def on_concurrency_setup(self):
-        load_extension_commands()
+        self.load_extension_commands()
+
+    def load_extension_commands(self):
+        names = Extensions(self.ext_fmt.format(self=self),
+                           self.register_command).load()
+        if names:
+            command_classes.append(('Extensions', names, 'magenta'))
 
 
-def determine_exit_status(ret):
-    if isinstance(ret, int):
-        return ret
-    return EX_OK if ret else EX_FAILURE
-
-
-def main(argv=None):
-    # Fix for setuptools generated scripts, so that it will
-    # work with multiprocessing fork emulation.
-    # (see multiprocessing.forking.get_preparation_data())
-    try:
-        if __name__ != '__main__':  # pragma: no cover
-            sys.modules['__main__'] = sys.modules[__name__]
-        cmd = CeleryCommand()
-        cmd.maybe_patch_concurrency()
-        from billiard import freeze_support
-        freeze_support()
-        cmd.execute_from_commandline(argv)
-    except KeyboardInterrupt:
-        pass
+def command(*args, **kwargs):
+    """Deprecated: Use classmethod :meth:`CeleryCommand.register_command`
+    instead."""
+    _register = CeleryCommand.register_command
+    return _register(args[0]) if args else _register
 
 
 if __name__ == '__main__':          # pragma: no cover
