@@ -5,28 +5,30 @@
 
     Composing task workflows.
 
-    Documentation for these functions are in :mod:`celery`.
-    You should not import from this module directly.
+    Documentation for some of these types are in :mod:`celery`.
+    You should import these from :mod:`celery` and not this module.
+
 
 """
 from __future__ import absolute_import
 
 from copy import deepcopy
-from functools import partial as _partial
+from functools import partial as _partial, reduce
 from operator import itemgetter
 from itertools import chain as _chain
 
 from kombu.utils import cached_property, fxrange, kwdict, reprcall, uuid
 
 from celery._state import current_app
-from celery.utils.compat import chain_from_iterable
-from celery.result import AsyncResult, GroupResult
 from celery.utils.functional import (
     maybe_list, is_list, regen,
     chunks as _chunks,
 )
 from celery.utils.text import truncate
 
+__all__ = ['Signature', 'chain', 'xmap', 'xstarmap', 'chunks',
+           'group', 'chord', 'signature', 'maybe_signature']
+
 
 class _getitem_property(object):
     """Attribute -> dict key descriptor.
@@ -54,7 +56,7 @@ class _getitem_property(object):
         10
 
         >>> me.deep_thing = 42
-        >>> me.deep_thinge
+        >>> me.deep_thing
         42
         >>> me.deep:
         defaultdict(<type 'dict'>, {'thing': 42})
@@ -82,8 +84,9 @@ class Signature(dict):
     """Class that wraps the arguments and execution options
     for a single task invocation.
 
-    Used as the parts in a :class:`group` or to safely
-    pass tasks around as callbacks.
+    Used as the parts in a :class:`group` and other constructs,
+    or to pass tasks around as callbacks while being compatible
+    with serializers with a strict type subset.
 
     :param task: Either a task class/instance, or the name of a task.
     :keyword args: Positional arguments to apply.
@@ -94,13 +97,13 @@ class Signature(dict):
     arguments will be ignored and the values in the dict will be used
     instead.
 
-        >>> s = subtask('tasks.add', args=(2, 2))
-        >>> subtask(s)
+        >>> s = signature('tasks.add', args=(2, 2))
+        >>> signature(s)
         {'task': 'tasks.add', args=(2, 2), kwargs={}, options={}}
 
     """
     TYPES = {}
-    _type = None
+    _app = _type = None
 
     @classmethod
     def register_type(cls, subclass, name=None):
@@ -108,14 +111,16 @@ class Signature(dict):
         return subclass
 
     @classmethod
-    def from_dict(self, d):
+    def from_dict(self, d, app=None):
         typ = d.get('subtask_type')
         if typ:
-            return self.TYPES[typ].from_dict(kwdict(d))
-        return Signature(d)
+            return self.TYPES[typ].from_dict(kwdict(d), app=app)
+        return Signature(d, app=app)
 
     def __init__(self, task=None, args=None, kwargs=None, options=None,
-                 type=None, subtask_type=None, immutable=False, **ex):
+                 type=None, subtask_type=None, immutable=False,
+                 app=None, **ex):
+        self._app = app
         init = dict.__init__
 
         if isinstance(task, dict):
@@ -137,8 +142,11 @@ class Signature(dict):
              immutable=immutable)
 
     def __call__(self, *partial_args, **partial_kwargs):
+        args, kwargs, _ = self._merge(partial_args, partial_kwargs, None)
+        return self.type(*args, **kwargs)
+
+    def delay(self, *partial_args, **partial_kwargs):
         return self.apply_async(partial_args, partial_kwargs)
-    delay = __call__
 
     def apply(self, args=(), kwargs={}, **options):
         """Apply this task locally."""
@@ -148,29 +156,36 @@ class Signature(dict):
 
     def _merge(self, args=(), kwargs={}, options={}):
         if self.immutable:
-            return self.args, self.kwargs, dict(self.options, **options)
+            return (self.args, self.kwargs,
+                    dict(self.options, **options) if options else self.options)
         return (tuple(args) + tuple(self.args) if args else self.args,
                 dict(self.kwargs, **kwargs) if kwargs else self.kwargs,
                 dict(self.options, **options) if options else self.options)
 
     def clone(self, args=(), kwargs={}, **opts):
         # need to deepcopy options so origins links etc. is not modified.
-        args, kwargs, opts = self._merge(args, kwargs, opts)
+        if args or kwargs or opts:
+            args, kwargs, opts = self._merge(args, kwargs, opts)
+        else:
+            args, kwargs, opts = self.args, self.kwargs, self.options
         s = Signature.from_dict({'task': self.task, 'args': tuple(args),
                                  'kwargs': kwargs, 'options': deepcopy(opts),
                                  'subtask_type': self.subtask_type,
-                                 'immutable': self.immutable})
+                                 'immutable': self.immutable}, app=self._app)
         s._type = self._type
         return s
     partial = clone
 
-    def _freeze(self, _id=None):
+    def freeze(self, _id=None):
         opts = self.options
         try:
             tid = opts['task_id']
         except KeyError:
             tid = opts['task_id'] = _id or uuid()
+        if 'reply_to' not in opts:
+            opts['reply_to'] = self.app.oid
         return self.AsyncResult(tid)
+    _freeze = freeze
 
     def replace(self, args=None, kwargs=None, options=None):
         s = self.clone()
@@ -184,13 +199,19 @@ class Signature(dict):
 
     def set(self, immutable=None, **options):
         if immutable is not None:
-            self.immutable = immutable
+            self.set_immutable(immutable)
         self.options.update(options)
         return self
 
+    def set_immutable(self, immutable):
+        self.immutable = immutable
+
     def apply_async(self, args=(), kwargs={}, **options):
         # For callbacks: extra args are prepended to the stored args.
-        args, kwargs, options = self._merge(args, kwargs, options)
+        if args or kwargs or options:
+            args, kwargs, options = self._merge(args, kwargs, options)
+        else:
+            args, kwargs, options = self.args, self.kwargs, self.options
         return self._apply_async(args, kwargs, **options)
 
     def append_to_list_option(self, key, value):
@@ -206,7 +227,7 @@ class Signature(dict):
         return self.append_to_list_option('link_error', errback)
 
     def flatten_links(self):
-        return list(chain_from_iterable(_chain(
+        return list(_chain.from_iterable(_chain(
             [[self]],
             (link.flatten_links()
                 for link in maybe_list(self.options.get('link')) or [])
@@ -214,15 +235,19 @@ class Signature(dict):
 
     def __or__(self, other):
         if not isinstance(self, chain) and isinstance(other, chain):
-            return chain((self,) + other.tasks)
+            return chain((self, ) + other.tasks, app=self._app)
         elif isinstance(other, chain):
-            return chain(*self.tasks + other.tasks)
+            return chain(*self.tasks + other.tasks, app=self._app)
         elif isinstance(other, Signature):
             if isinstance(self, chain):
-                return chain(*self.tasks + (other, ))
-            return chain(self, other)
+                return chain(*self.tasks + (other, ), app=self._app)
+            return chain(self, other, app=self._app)
         return NotImplemented
 
+    def __deepcopy__(self, memo):
+        memo[id(self)] = self
+        return dict(self)
+
     def __invert__(self):
         return self.apply_async().get()
 
@@ -235,26 +260,41 @@ class Signature(dict):
         args, kwargs, _ = self._merge(args, kwargs, {})
         return reprcall(self['task'], args, kwargs)
 
+    def election(self):
+        type = self.type
+        app = type.app
+        tid = self.options.get('task_id') or uuid()
+
+        with app.producer_or_acquire(None) as P:
+            props = type.backend.on_task_call(P, tid)
+            app.control.election(tid, 'task', self.clone(task_id=tid, **props),
+                                 connection=P.connection)
+            return type.AsyncResult(tid)
+
     def __repr__(self):
         return self.reprcall()
 
     @cached_property
     def type(self):
-        return self._type or current_app.tasks[self['task']]
+        return self._type or self.app.tasks[self['task']]
+
+    @cached_property
+    def app(self):
+        return self._app or current_app
 
     @cached_property
     def AsyncResult(self):
         try:
             return self.type.AsyncResult
         except KeyError:  # task not registered
-            return AsyncResult
+            return self.app.AsyncResult
 
     @cached_property
     def _apply_async(self):
         try:
             return self.type.apply_async
         except KeyError:
-            return _partial(current_app.send_task, self['task'])
+            return _partial(self.app.send_task, self['task'])
     id = _getitem_property('options.task_id')
     task = _getitem_property('task')
     args = _getitem_property('args')
@@ -264,6 +304,7 @@ class Signature(dict):
     immutable = _getitem_property('immutable')
 
 
+@Signature.register_type
 class chain(Signature):
 
     def __init__(self, *tasks, **options):
@@ -280,20 +321,22 @@ class chain(Signature):
             return self.apply_async(args, kwargs)
 
     @classmethod
-    def from_dict(self, d):
+    def from_dict(self, d, app=None):
         tasks = d['kwargs']['tasks']
         if d['args'] and tasks:
             # partial args passed on to first task in chain (Issue #1057).
             tasks[0]['args'] = tasks[0]._merge(d['args'])[0]
-        return chain(*d['kwargs']['tasks'], **kwdict(d['options']))
+        return chain(*d['kwargs']['tasks'], app=app, **kwdict(d['options']))
 
     @property
     def type(self):
-        return self._type or self.tasks[0].type.app.tasks['celery.chain']
+        try:
+            return self._type or self.tasks[0].type.app.tasks['celery.chain']
+        except KeyError:
+            return self.app.tasks['celery.chain']
 
     def __repr__(self):
         return ' | '.join(repr(t) for t in self.tasks)
-Signature.register_type(chain)
 
 
 class _basemap(Signature):
@@ -314,28 +357,31 @@ class _basemap(Signature):
         )
 
     @classmethod
-    def from_dict(self, d):
-        return chunks(*self._unpack_args(d['kwargs']), **d['options'])
+    def from_dict(cls, d, app=None):
+        return cls(*cls._unpack_args(d['kwargs']), app=app, **d['options'])
 
 
+@Signature.register_type
 class xmap(_basemap):
     _task_name = 'celery.map'
 
     def __repr__(self):
         task, it = self._unpack_args(self.kwargs)
-        return '[%s(x) for x in %s]' % (task.task, truncate(repr(it), 100))
-Signature.register_type(xmap)
+        return '[{0}(x) for x in {1}]'.format(task.task,
+                                              truncate(repr(it), 100))
 
 
+@Signature.register_type
 class xstarmap(_basemap):
     _task_name = 'celery.starmap'
 
     def __repr__(self):
         task, it = self._unpack_args(self.kwargs)
-        return '[%s(*x) for x in %s]' % (task.task, truncate(repr(it), 100))
-Signature.register_type(xstarmap)
+        return '[{0}(*x) for x in {1}]'.format(task.task,
+                                               truncate(repr(it), 100))
 
 
+@Signature.register_type
 class chunks(Signature):
     _unpack_args = itemgetter('task', 'it', 'n')
 
@@ -347,8 +393,8 @@ class chunks(Signature):
         )
 
     @classmethod
-    def from_dict(self, d):
-        return chunks(*self._unpack_args(d['kwargs']), **d['options'])
+    def from_dict(self, d, app=None):
+        return chunks(*self._unpack_args(d['kwargs']), app=app, **d['options'])
 
     def apply_async(self, args=(), kwargs={}, **opts):
         return self.group().apply_async(args, kwargs, **opts)
@@ -359,12 +405,13 @@ class chunks(Signature):
     def group(self):
         # need to evaluate generators
         task, it, n = self._unpack_args(self.kwargs)
-        return group(xstarmap(task, part) for part in _chunks(iter(it), n))
+        return group((xstarmap(task, part, app=self._app)
+                      for part in _chunks(iter(it), n)),
+                     app=self._app)
 
     @classmethod
-    def apply_chunks(cls, task, it, n):
-        return cls(task, it, n)()
-Signature.register_type(chunks)
+    def apply_chunks(cls, task, it, n, app=None):
+        return cls(task, it, n, app=app)()
 
 
 def _maybe_group(tasks):
@@ -377,6 +424,12 @@ def _maybe_group(tasks):
     return tasks
 
 
+def _maybe_clone(tasks, app):
+    return [s.clone() if isinstance(s, Signature) else signature(s, app=app)
+            for s in tasks]
+
+
+@Signature.register_type
 class group(Signature):
 
     def __init__(self, *tasks, **options):
@@ -388,43 +441,66 @@ class group(Signature):
         self.tasks, self.subtask_type = tasks, 'group'
 
     @classmethod
-    def from_dict(self, d):
+    def from_dict(self, d, app=None):
         tasks = d['kwargs']['tasks']
         if d['args'] and tasks:
             # partial args passed on to all tasks in the group (Issue #1057).
             for task in tasks:
                 task['args'] = task._merge(d['args'])[0]
-        return group(tasks, **kwdict(d['options']))
+        return group(tasks, app=app, **kwdict(d['options']))
 
-    def __call__(self, *partial_args, **options):
-        tasks = [task.clone() for task in self.tasks]
+    def apply_async(self, args=(), kwargs=None, **options):
+        tasks = _maybe_clone(self.tasks, app=self._app)
         if not tasks:
-            return
+            return self.freeze()
         # taking the app from the first task in the list,
         # there may be a better solution to this, e.g.
         # consolidate tasks with the same app and apply them in
         # batches.
         type = tasks[0].type.app.tasks[self['task']]
-        return type(*type.prepare(options, tasks, partial_args))
+        return type(*type.prepare(dict(self.options, **options),
+                                  tasks, args))
 
-    def _freeze(self, _id=None):
+    def set_immutable(self, immutable):
+        for task in self.tasks:
+            task.set_immutable(immutable)
+
+    def link(self, sig):
+        # Simply link to first task
+        sig = sig.clone().set(immutable=True)
+        return self.tasks[0].link(sig)
+
+    def link_error(self, sig):
+        sig = sig.clone().set(immutable=True)
+        return self.tasks[0].link_error(sig)
+
+    def apply(self, *args, **kwargs):
+        if not self.tasks:
+            return self.freeze()  # empty group returns GroupResult
+        return Signature.apply(self, *args, **kwargs)
+
+    def __call__(self, *partial_args, **options):
+        return self.apply_async(partial_args, **options)
+
+    def freeze(self, _id=None):
         opts = self.options
         try:
-            gid = opts['group']
+            gid = opts['task_id']
         except KeyError:
-            gid = opts['group'] = uuid()
+            gid = opts['task_id'] = uuid()
         new_tasks, results = [], []
         for task in self.tasks:
-            task = maybe_subtask(task).clone()
+            task = maybe_signature(task, app=self._app).clone()
             results.append(task._freeze())
             new_tasks.append(task)
         self.tasks = self.kwargs['tasks'] = new_tasks
-        return GroupResult(gid, results)
+        return self.app.GroupResult(gid, results)
+    _freeze = freeze
 
     def skew(self, start=1.0, stop=None, step=1.0):
-        _next_skew = fxrange(start, stop, step, repeatlast=True).next
+        it = fxrange(start, stop, step, repeatlast=True)
         for task in self.tasks:
-            task.set(countdown=_next_skew())
+            task.set(countdown=next(it))
         return self
 
     def __iter__(self):
@@ -437,9 +513,8 @@ class group(Signature):
     def type(self):
         return self._type or self.tasks[0].type.app.tasks[self['task']]
 
-Signature.register_type(group)
-
 
+@Signature.register_type
 class chord(Signature):
 
     def __init__(self, header, body=None, task='celery.chord',
@@ -447,14 +522,14 @@ class chord(Signature):
         Signature.__init__(
             self, task, args,
             dict(kwargs, header=_maybe_group(header),
-                 body=maybe_subtask(body)), **options
+                 body=maybe_signature(body, app=self._app)), **options
         )
         self.subtask_type = 'chord'
 
     @classmethod
-    def from_dict(self, d):
+    def from_dict(self, d, app=None):
         args, d['kwargs'] = self._unpack_args(**kwdict(d['kwargs']))
-        return self(*args, **kwdict(d))
+        return self(*args, app=app, **kwdict(d))
 
     @staticmethod
     def _unpack_args(header=None, body=None, **kwargs):
@@ -466,14 +541,22 @@ class chord(Signature):
     def type(self):
         return self._type or self.tasks[0].type.app.tasks['celery.chord']
 
-    def __call__(self, body=None, **kwargs):
-        _chord = self.type
-        body = (body or self.kwargs['body']).clone()
-        kwargs = dict(self.kwargs, body=body, **kwargs)
+    def apply_async(self, args=(), kwargs={}, task_id=None, **options):
+        body = kwargs.get('body') or self.kwargs['body']
+        kwargs = dict(self.kwargs, **kwargs)
+        body = body.clone(**options)
+
+        _chord = self._type or body.type.app.tasks['celery.chord']
+
         if _chord.app.conf.CELERY_ALWAYS_EAGER:
-            return self.apply((), kwargs)
-        callback_id = body.options.setdefault('task_id', uuid())
-        return _chord.AsyncResult(callback_id, parent=_chord(**kwargs))
+            return self.apply((), kwargs, task_id=task_id, **options)
+        res = body.freeze(task_id)
+        parent = _chord(self.tasks, body, args, **options)
+        res.parent = parent
+        return res
+
+    def __call__(self, body=None, **options):
+        return self.apply_async((), {'body': body} if body else {}, **options)
 
     def clone(self, *args, **kwargs):
         s = Signature.clone(self, *args, **kwargs)
@@ -492,25 +575,34 @@ class chord(Signature):
         self.body.link_error(errback)
         return errback
 
+    def set_immutable(self, immutable):
+        # changes mutability of header only, not callback.
+        for task in self.tasks:
+            task.set_immutable(immutable)
+
     def __repr__(self):
         if self.body:
             return self.body.reprcall(self.tasks)
-        return '<chord without body: %r>' % (self.tasks, )
+        return '<chord without body: {0.tasks!r}>'.format(self)
 
     tasks = _getitem_property('kwargs.header')
     body = _getitem_property('kwargs.body')
-Signature.register_type(chord)
 
 
-def subtask(varies, *args, **kwargs):
+def signature(varies, *args, **kwargs):
     if not (args or kwargs) and isinstance(varies, dict):
         if isinstance(varies, Signature):
             return varies.clone()
         return Signature.from_dict(varies)
     return Signature(varies, *args, **kwargs)
+subtask = signature   # XXX compat
 
 
-def maybe_subtask(d):
-    if d is not None and isinstance(d, dict) and not isinstance(d, Signature):
-        return subtask(d)
-    return d
+def maybe_signature(d, app=None):
+    if d is not None and isinstance(d, dict):
+        if not isinstance(d, Signature):
+            return signature(d, app=app)
+        if app is not None:
+            d._app = app
+        return d
+maybe_subtask = maybe_signature  # XXX compat
