@@ -10,21 +10,40 @@ from __future__ import absolute_import
 
 import logging
 import os
-import time
+import sys
 
+from billiard.einfo import ExceptionInfo
+from billiard.exceptions import WorkerLostError
 from kombu.utils.encoding import safe_repr
 
+from celery.five import monotonic, reraise
 from celery.utils import timer2
 from celery.utils.log import get_logger
 
-logger = get_logger('celery.concurrency')
+__all__ = ['BasePool', 'apply_target']
+
+logger = get_logger('celery.pool')
 
 
 def apply_target(target, args=(), kwargs={}, callback=None,
-                 accept_callback=None, pid=None, **_):
+                 accept_callback=None, pid=None, getpid=os.getpid,
+                 propagate=(), monotonic=monotonic, **_):
     if accept_callback:
-        accept_callback(pid or os.getpid(), time.time())
-    callback(target(*args, **kwargs))
+        accept_callback(pid or getpid(), monotonic())
+    try:
+        ret = target(*args, **kwargs)
+    except propagate:
+        raise
+    except Exception:
+        raise
+    except BaseException as exc:
+        try:
+            reraise(WorkerLostError, WorkerLostError(repr(exc)),
+                    sys.exc_info()[2])
+        except WorkerLostError:
+            callback(ExceptionInfo())
+    else:
+        callback(ret)
 
 
 class BasePool(object):
@@ -38,15 +57,6 @@ class BasePool(object):
     #: a signal handler.
     signal_safe = True
 
-    #: set to true if pool supports rate limits.
-    #: (this is here for gevent, which currently does not implement
-    #: the necessary timers).
-    rlimit_safe = True
-
-    #: set to true if pool requires the use of a mediator
-    #: thread (e.g. if applying new items can block the current thread).
-    requires_mediator = False
-
     #: set to true if pool uses greenlets.
     is_green = False
 
@@ -71,9 +81,15 @@ class BasePool(object):
     def did_start_ok(self):
         return True
 
+    def flush(self):
+        pass
+
     def on_stop(self):
         pass
 
+    def register_with_event_loop(self, loop):
+        pass
+
     def on_apply(self, *args, **kwargs):
         pass
 
@@ -86,19 +102,16 @@ class BasePool(object):
     def on_hard_timeout(self, job):
         pass
 
-    def maybe_handle_result(self, *args):
-        pass
-
     def maintain_pool(self, *args, **kwargs):
         pass
 
     def terminate_job(self, pid):
         raise NotImplementedError(
-            '%s does not implement kill_job' % (self.__class__, ))
+            '{0} does not implement kill_job'.format(type(self)))
 
     def restart(self):
         raise NotImplementedError(
-            '%s does not implement restart' % (self.__class__, ))
+            '{0} does not implement restart'.format(type(self)))
 
     def stop(self):
         self.on_stop()
@@ -119,9 +132,6 @@ class BasePool(object):
     def on_close(self):
         pass
 
-    def init_callbacks(self, **kwargs):
-        pass
-
     def apply_async(self, target, args=[], kwargs={}, **options):
         """Equivalent of the :func:`apply` built-in function.
 
@@ -152,15 +162,3 @@ class BasePool(object):
     @property
     def num_processes(self):
         return self.limit
-
-    @property
-    def readers(self):
-        return {}
-
-    @property
-    def writers(self):
-        return {}
-
-    @property
-    def timers(self):
-        return {}
