@@ -7,41 +7,30 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
-import os
-
-PATCHED = [0]
-if not os.environ.get('GEVENT_NOPATCH') and not PATCHED[0]:
-    PATCHED[0] += 1
-    from gevent import monkey, version_info
-    monkey.patch_all()
-    if version_info[0] == 0:
-        # Signals are not working along gevent in version prior 1.0
-        # and they are not monkey patch by monkey.patch_all()
-        from gevent import signal as _gevent_signal
-        _signal = __import__('signal')
-        _signal.signal = _gevent_signal
+from time import time
 
 try:
     from gevent import Timeout
-except ImportError:
+except ImportError:  # pragma: no cover
     Timeout = None  # noqa
 
-from time import time
-
 from celery.utils import timer2
 
 from .base import apply_target, BasePool
 
+__all__ = ['TaskPool']
+
 
 def apply_timeout(target, args=(), kwargs={}, callback=None,
                   accept_callback=None, pid=None, timeout=None,
-                  timeout_callback=None, **rest):
+                  timeout_callback=None, Timeout=Timeout,
+                  apply_target=apply_target, **rest):
     try:
         with Timeout(timeout):
             return apply_target(target, args, kwargs, callback,
-                                accept_callback, pid, **rest)
+                                accept_callback, pid,
+                                propagate=(Timeout, ), **rest)
     except Timeout:
         return timeout_callback(False, timeout)
 
@@ -52,9 +41,7 @@ class Schedule(timer2.Schedule):
         from gevent.greenlet import Greenlet, GreenletExit
 
         class _Greenlet(Greenlet):
-
-            def cancel(self):
-                self.kill()
+            cancel = Greenlet.kill
 
         self._Greenlet = _Greenlet
         self._GreenletExit = GreenletExit
@@ -88,7 +75,7 @@ class Schedule(timer2.Schedule):
 
     @property
     def queue(self):
-        return [(g.eta, g.priority, g.entry) for g in self._queue]
+        return self._queue
 
 
 class Timer(timer2.Timer):
@@ -108,7 +95,6 @@ class TaskPool(BasePool):
     Timer = Timer
 
     signal_safe = False
-    rlimit_safe = False
     is_green = True
 
     def __init__(self, *args, **kwargs):
