@@ -6,8 +6,7 @@
     Migration tools.
 
 """
-from __future__ import absolute_import
-from __future__ import with_statement
+from __future__ import absolute_import, print_function, unicode_literals
 
 import socket
 
@@ -16,12 +15,22 @@ from itertools import cycle, islice
 
 from kombu import eventloop, Queue
 from kombu.common import maybe_declare
-from kombu.exceptions import StdChannelError
 from kombu.utils.encoding import ensure_bytes
 
 from celery.app import app_or_default
+from celery.five import string, string_t
 from celery.utils import worker_direct
 
+__all__ = ['StopFiltering', 'State', 'republish', 'migrate_task',
+           'migrate_tasks', 'move', 'task_id_eq', 'task_id_in',
+           'start_filter', 'move_task_by_id', 'move_by_idmap',
+           'move_by_taskmap', 'move_direct', 'move_direct_by_id']
+
+MOVING_PROGRESS_FMT = """\
+Moving task {state.filtered}/{state.strtotal}: \
+{body[task]}[{body[id]}]\
+"""
+
 
 class StopFiltering(Exception):
     pass
@@ -35,13 +44,13 @@ class State(object):
     @property
     def strtotal(self):
         if not self.total_apx:
-            return u'?'
-        return unicode(self.total_apx)
+            return '?'
+        return string(self.total_apx)
 
     def __repr__(self):
         if self.filtered:
-            return '^%s' % self.filtered
-        return '%s/%s' % (self.count, self.strtotal)
+            return '^{0.filtered}'.format(self)
+        return '{0.count}/{0.strtotal}'.format(self)
 
 
 def republish(producer, message, exchange=None, routing_key=None,
@@ -79,7 +88,7 @@ def migrate_task(producer, body_, message, queues=None):
 def filter_callback(callback, tasks):
 
     def filtered(body, message):
-        if tasks and message.payload['task'] not in tasks:
+        if tasks and body['task'] not in tasks:
             return
 
         return callback(body, message)
@@ -108,7 +117,7 @@ def migrate_tasks(source, dest, migrate=migrate_task, app=None,
 
 
 def _maybe_queue(app, q):
-    if isinstance(q, basestring):
+    if isinstance(q, string_t):
         return app.amqp.queues[q]
     return q
 
@@ -161,7 +170,7 @@ def move(predicate, connection=None, exchange=None, routing_key=None,
     .. code-block:: python
 
         def transform(value):
-            if isinstance(value, basestring):
+            if isinstance(value, string_t):
                 return Queue(value, Exchange(value), value)
             return value
 
@@ -220,7 +229,7 @@ def task_id_in(ids, body, message):
 
 
 def prepare_queues(queues):
-    if isinstance(queues, basestring):
+    if isinstance(queues, string_t):
         queues = queues.split(',')
     if isinstance(queues, list):
         queues = dict(tuple(islice(cycle(q.split(':')), None, 2))
@@ -233,12 +242,12 @@ def prepare_queues(queues):
 def start_filter(app, conn, filter, limit=None, timeout=1.0,
                  ack_messages=False, tasks=None, queues=None,
                  callback=None, forever=False, on_declare_queue=None,
-                 consume_from=None, state=None, **kwargs):
+                 consume_from=None, state=None, accept=None, **kwargs):
     state = state or State()
     queues = prepare_queues(queues)
     consume_from = [_maybe_queue(app, q)
-                    for q in consume_from or queues.keys()]
-    if isinstance(tasks, basestring):
+                    for q in consume_from or list(queues)]
+    if isinstance(tasks, string_t):
         tasks = set(tasks.split(','))
     if tasks is None:
         tasks = set([])
@@ -251,7 +260,7 @@ def start_filter(app, conn, filter, limit=None, timeout=1.0,
     def ack_message(body, message):
         message.ack()
 
-    consumer = app.amqp.TaskConsumer(conn, queues=consume_from)
+    consumer = app.amqp.TaskConsumer(conn, queues=consume_from, accept=accept)
 
     if tasks:
         filter = filter_callback(filter, tasks)
@@ -278,7 +287,7 @@ def start_filter(app, conn, filter, limit=None, timeout=1.0,
             _, mcount, _ = queue(consumer.channel).queue_declare(passive=True)
             if mcount:
                 state.total_apx += mcount
-        except conn.channel_errors + (StdChannelError, ):
+        except conn.channel_errors:
             pass
 
     # start migrating messages.
@@ -346,12 +355,11 @@ def move_by_taskmap(map, **kwargs):
     return move(task_name_in_map, **kwargs)
 
 
+def filter_status(state, body, message, **kwargs):
+    print(MOVING_PROGRESS_FMT.format(state=state, body=body, **kwargs))
+
+
 move_direct = partial(move, transform=worker_direct)
 move_direct_by_id = partial(move_task_by_id, transform=worker_direct)
 move_direct_by_idmap = partial(move_by_idmap, transform=worker_direct)
 move_direct_by_taskmap = partial(move_by_taskmap, transform=worker_direct)
-
-
-def filter_status(state, body, message):
-    print('Moving task %s/%s: %s[%s]' % (
-        state.filtered, state.strtotal, body['task'], body['id']))
