@@ -6,26 +6,123 @@
     Custom types and data structures.
 
 """
-from __future__ import absolute_import
-from __future__ import with_statement
+from __future__ import absolute_import, print_function, unicode_literals
 
 import sys
 import time
 
-from collections import defaultdict
+from collections import defaultdict, Mapping, MutableMapping, MutableSet
 from heapq import heapify, heappush, heappop
+from functools import partial
 from itertools import chain
 
-try:
-    from collections import Mapping, MutableMapping
-except ImportError:        # pragma: no cover
-    MutableMapping = None  # noqa
-    Mapping = dict         # noqa
-
 from billiard.einfo import ExceptionInfo  # noqa
+from kombu.utils.encoding import safe_str
 from kombu.utils.limits import TokenBucket  # noqa
 
-from .utils.functional import LRUCache, first, uniq  # noqa
+from celery.five import items
+from celery.utils.functional import LRUCache, first, uniq  # noqa
+
+DOT_HEAD = """
+{IN}{type} {id} {{
+{INp}graph [{attrs}]
+"""
+DOT_ATTR = '{name}={value}'
+DOT_NODE = '{INp}"{0}" [{attrs}]'
+DOT_EDGE = '{INp}"{0}" {dir} "{1}" [{attrs}]'
+DOT_ATTRSEP = ', '
+DOT_DIRS = {'graph': '--', 'digraph': '->'}
+DOT_TAIL = '{IN}}}'
+
+__all__ = ['GraphFormatter', 'CycleError', 'DependencyGraph',
+           'AttributeDictMixin', 'AttributeDict', 'DictAttribute',
+           'ConfigurationView', 'LimitedSet']
+
+
+class GraphFormatter(object):
+    _attr = DOT_ATTR.strip()
+    _node = DOT_NODE.strip()
+    _edge = DOT_EDGE.strip()
+    _head = DOT_HEAD.strip()
+    _tail = DOT_TAIL.strip()
+    _attrsep = DOT_ATTRSEP
+    _dirs = dict(DOT_DIRS)
+
+    scheme = {
+        'shape': 'box',
+        'arrowhead': 'vee',
+        'style': 'filled',
+        'fontname': 'HelveticaNeue',
+    }
+    edge_scheme = {
+        'color': 'darkseagreen4',
+        'arrowcolor': 'black',
+        'arrowsize': 0.7,
+    }
+    node_scheme = {'fillcolor': 'palegreen3', 'color': 'palegreen4'}
+    term_scheme = {'fillcolor': 'palegreen1', 'color': 'palegreen2'}
+    graph_scheme = {'bgcolor': 'mintcream'}
+
+    def __init__(self, root=None, type=None, id=None,
+                 indent=0, inw=' ' * 4, **scheme):
+        self.id = id or 'dependencies'
+        self.root = root
+        self.type = type or 'digraph'
+        self.direction = self._dirs[self.type]
+        self.IN = inw * (indent or 0)
+        self.INp = self.IN + inw
+        self.scheme = dict(self.scheme, **scheme)
+        self.graph_scheme = dict(self.graph_scheme, root=self.label(self.root))
+
+    def attr(self, name, value):
+        value = '"{0}"'.format(value)
+        return self.FMT(self._attr, name=name, value=value)
+
+    def attrs(self, d, scheme=None):
+        d = dict(self.scheme, **dict(scheme, **d or {}) if scheme else d)
+        return self._attrsep.join(
+            safe_str(self.attr(k, v)) for k, v in items(d)
+        )
+
+    def head(self, **attrs):
+        return self.FMT(
+            self._head, id=self.id, type=self.type,
+            attrs=self.attrs(attrs, self.graph_scheme),
+        )
+
+    def tail(self):
+        return self.FMT(self._tail)
+
+    def label(self, obj):
+        return obj
+
+    def node(self, obj, **attrs):
+        return self.draw_node(obj, self.node_scheme, attrs)
+
+    def terminal_node(self, obj, **attrs):
+        return self.draw_node(obj, self.term_scheme, attrs)
+
+    def edge(self, a, b, **attrs):
+        return self.draw_edge(a, b, **attrs)
+
+    def _enc(self, s):
+        return s.encode('utf-8', 'ignore')
+
+    def FMT(self, fmt, *args, **kwargs):
+        return self._enc(fmt.format(
+            *args, **dict(kwargs, IN=self.IN, INp=self.INp)
+        ))
+
+    def draw_edge(self, a, b, scheme=None, attrs=None):
+        return self.FMT(
+            self._edge, self.label(a), self.label(b),
+            dir=self.direction, attrs=self.attrs(attrs, self.edge_scheme),
+        )
+
+    def draw_node(self, obj, scheme=None, attrs=None):
+        return self.FMT(
+            self._node, self.label(obj), attrs=self.attrs(attrs, scheme),
+        )
 
 
 class CycleError(Exception):
@@ -47,7 +144,8 @@ class DependencyGraph(object):
 
     """
 
-    def __init__(self, it=None):
+    def __init__(self, it=None, formatter=None):
+        self.formatter = formatter or GraphFormatter()
         self.adjacent = {}
         if it is not None:
             self.update(it)
@@ -61,6 +159,10 @@ class DependencyGraph(object):
         (``A`` depends on ``B``)."""
         self[A].append(B)
 
+    def connect(self, graph):
+        """Add nodes from another graph."""
+        self.adjacent.update(graph.adjacent)
+
     def topsort(self):
         """Sort the graph topologically.
 
@@ -85,7 +187,7 @@ class DependencyGraph(object):
         return [t[0] for t in graph._khan62()]
 
     def valency_of(self, obj):
-        """Returns the velency (degree) of a vertex in the graph."""
+        """Return the valency (degree) of a vertex in the graph."""
         try:
             l = [len(self[obj])]
         except KeyError:
@@ -105,8 +207,8 @@ class DependencyGraph(object):
                 self.add_edge(obj, dep)
 
     def edges(self):
-        """Returns generator that yields for all edges in the graph."""
-        return (obj for obj, adj in self.iteritems() if adj)
+        """Return generator that yields for all edges in the graph."""
+        return (obj for obj, adj in items(self) if adj)
 
     def _khan62(self):
         """Khans simple topological sort algorithm from '62
@@ -165,19 +267,32 @@ class DependencyGraph(object):
 
         return result
 
-    def to_dot(self, fh, ws=' ' * 4):
+    def to_dot(self, fh, formatter=None):
         """Convert the graph to DOT format.
 
         :param fh: A file, or a file-like object to write the graph to.
 
         """
-        fh.write('digraph dependencies {\n')
-        for obj, adjacent in self.iteritems():
+        seen = set()
+        draw = formatter or self.formatter
+        P = partial(print, file=fh)
+
+        def if_not_seen(fun, obj):
+            if draw.label(obj) not in seen:
+                P(fun(obj))
+                seen.add(draw.label(obj))
+
+        P(draw.head())
+        for obj, adjacent in items(self):
             if not adjacent:
-                fh.write(ws + '"%s"\n' % (obj, ))
+                if_not_seen(draw.terminal_node, obj)
             for req in adjacent:
-                fh.write(ws + '"%s" -> "%s"\n' % (obj, req))
-        fh.write('}\n')
+                if_not_seen(draw.node, obj)
+                P(draw.edge(obj, req))
+        P(draw.tail())
+
+    def format(self, obj):
+        return self.formatter(obj) if self.formatter else obj
 
     def __iter__(self):
         return iter(self.adjacent)
@@ -192,26 +307,26 @@ class DependencyGraph(object):
         return obj in self.adjacent
 
     def _iterate_items(self):
-        return self.adjacent.iteritems()
+        return items(self.adjacent)
     items = iteritems = _iterate_items
 
     def __repr__(self):
         return '\n'.join(self.repr_node(N) for N in self)
 
-    def repr_node(self, obj, level=1):
-        output = ['%s(%s)' % (obj, self.valency_of(obj))]
+    def repr_node(self, obj, level=1, fmt='{0}({1})'):
+        output = [fmt.format(obj, self.valency_of(obj))]
         if obj in self:
             for other in self[obj]:
-                d = '%s(%s)' % (other, self.valency_of(other))
+                d = fmt.format(other, self.valency_of(other))
                 output.append('     ' * level + d)
                 output.extend(self.repr_node(other, level + 1).split('\n')[1:])
         return '\n'.join(output)
 
 
 class AttributeDictMixin(object):
-    """Adds attribute access to mappings.
+    """Augment classes with a Mapping interface by adding attribute access.
 
-    `d.key -> d[key]`
+    I.e. `d.key -> d[key]`.
 
     """
 
@@ -221,7 +336,8 @@ class AttributeDictMixin(object):
             return self[k]
         except KeyError:
             raise AttributeError(
-                "'%s' object has no attribute '%s'" % (type(self).__name__, k))
+                '{0!r} object has no attribute {1!r}'.format(
+                    type(self).__name__, k))
 
     def __setattr__(self, key, value):
         """`d[key] = value -> d.key = value`"""
@@ -237,6 +353,7 @@ class DictAttribute(object):
     """Dict interface to attributes.
 
     `obj[k] -> obj.k`
+    `obj[k] = val -> obj.k = val`
 
     """
     obj = None
@@ -287,9 +404,15 @@ class DictAttribute(object):
             yield key, getattr(self.obj, key)
     iteritems = _iterate_items
 
+    def _iterate_values(self):
+        for key in self._iterate_keys():
+            yield getattr(self.obj, key)
+    itervalues = _iterate_values
+
     if sys.version_info[0] == 3:  # pragma: no cover
         items = _iterate_items
         keys = _iterate_keys
+        values = _iterate_values
     else:
 
         def keys(self):
@@ -298,10 +421,16 @@ class DictAttribute(object):
         def items(self):
             return list(self._iterate_items())
 
+        def values(self):
+            return list(self._iterate_values())
+MutableMapping.register(DictAttribute)
+
 
 class ConfigurationView(AttributeDictMixin):
     """A view over an applications configuration dicts.
 
+    Custom (but older) version of :class:`collections.ChainMap`.
+
     If the key does not exist in ``changes``, the ``defaults`` dicts
     are consulted.
 
@@ -343,6 +472,10 @@ class ConfigurationView(AttributeDictMixin):
         except KeyError:
             return default
 
+    def clear(self):
+        """Remove all changes, but keep defaults."""
+        self.changes.clear()
+
     def setdefault(self, key, default):
         try:
             return self[key]
@@ -354,13 +487,14 @@ class ConfigurationView(AttributeDictMixin):
         return self.changes.update(*args, **kwargs)
 
     def __contains__(self, key):
-        for d in self._order:
-            if key in d:
-                return True
-        return False
+        return any(key in m for m in self._order)
+
+    def __bool__(self):
+        return any(self._order)
+    __nonzero__ = __bool__  # Py2
 
     def __repr__(self):
-        return repr(dict(self.iteritems()))
+        return repr(dict(items(self)))
 
     def __iter__(self):
         return self._iterate_keys()
@@ -368,7 +502,7 @@ class ConfigurationView(AttributeDictMixin):
     def __len__(self):
         # The logic for iterating keys includes uniq(),
         # so to be safe we count by explicitly iterating
-        return len(self.keys())
+        return len(set().union(*self._order))
 
     def _iter(self, op):
         # defaults must be first in the stream, so values in
@@ -387,57 +521,62 @@ class ConfigurationView(AttributeDictMixin):
         return (self[key] for key in self)
     itervalues = _iterate_values
 
-    def keys(self):
-        return list(self._iterate_keys())
+    if sys.version_info[0] == 3:  # pragma: no cover
+        keys = _iterate_keys
+        items = _iterate_items
+        values = _iterate_values
 
-    def items(self):
-        return list(self._iterate_items())
+    else:  # noqa
+        def keys(self):
+            return list(self._iterate_keys())
 
-    def values(self):
-        return list(self._iterate_values())
-if MutableMapping:
-    MutableMapping.register(ConfigurationView)
+        def items(self):
+            return list(self._iterate_items())
+
+        def values(self):
+            return list(self._iterate_values())
+
+MutableMapping.register(ConfigurationView)
 
 
 class LimitedSet(object):
     """Kind-of Set with limitations.
 
     Good for when you need to test for membership (`a in set`),
-    but the list might become to big, so you want to limit it so it doesn't
-    consume too much resources.
+    but the list might become to big.
 
     :keyword maxlen: Maximum number of members before we start
                      evicting expired members.
     :keyword expires: Time in seconds, before a membership expires.
 
     """
-    __slots__ = ('maxlen', 'expires', '_data', '__len__', '_heap')
 
     def __init__(self, maxlen=None, expires=None, data=None, heap=None):
         self.maxlen = maxlen
         self.expires = expires
-        self._data = data or {}
-        self._heap = heap or []
-        self.__len__ = self._data.__len__
+        self._data = {} if data is None else data
+        self._heap = [] if heap is None else heap
+        # make shortcuts
+        self.__len__ = self._heap.__len__
+        self.__iter__ = self._heap.__iter__
+        self.__contains__ = self._data.__contains__
 
-    def add(self, value):
+    def add(self, value, now=time.time):
         """Add a new member."""
-        self.purge(1)
-        now = time.time()
-        self._data[value] = now
-        heappush(self._heap, (now, value))
-
-    def __reduce__(self):
-        return self.__class__, (
-            self.maxlen, self.expires, self._data, self._heap,
-        )
+        # offset is there to modify the length of the list,
+        # this way we can expire an item before inserting the value,
+        # and it will end up in correct order.
+        self.purge(1, offset=1)
+        inserted = now()
+        self._data[value] = inserted
+        heappush(self._heap, (inserted, value))
 
     def clear(self):
         """Remove all members"""
         self._data.clear()
         self._heap[:] = []
 
-    def pop_value(self, value):
+    def discard(self, value):
         """Remove membership by finding value."""
         try:
             itime = self._data[value]
@@ -448,28 +587,29 @@ class LimitedSet(object):
         except ValueError:
             pass
         self._data.pop(value, None)
+    pop_value = discard  # XXX compat
 
-    def _expire_item(self):
-        """Hunt down and remove an expired item."""
-        self.purge(1)
-
-    def __contains__(self, value):
-        return value in self._data
-
-    def purge(self, limit=None):
+    def purge(self, limit=None, offset=0, now=time.time):
+        """Purge expired items."""
         H, maxlen = self._heap, self.maxlen
         if not maxlen:
             return
+
+        # If the data/heap gets corrupted and limit is None
+        # this will go into an infinite loop, so limit must
+        # have a value to guard the loop.
+        limit = len(self) + offset if limit is None else limit
+
         i = 0
-        while len(self) >= maxlen:
-            if limit and i > limit:
+        while len(self) + offset > maxlen:
+            if i >= limit:
                 break
             try:
                 item = heappop(H)
             except IndexError:
                 break
             if self.expires:
-                if time.time() < item[0] + self.expires:
+                if now() < item[0] + self.expires:
                     heappush(H, item)
                     break
             try:
@@ -479,7 +619,7 @@ class LimitedSet(object):
             i += 1
 
     def update(self, other, heappush=heappush):
-        if isinstance(other, self.__class__):
+        if isinstance(other, LimitedSet):
             self._data.update(other._data)
             self._heap.extend(other._heap)
             heapify(self._heap)
@@ -490,17 +630,26 @@ class LimitedSet(object):
     def as_dict(self):
         return self._data
 
-    def __iter__(self):
-        return iter(self._data)
+    def __eq__(self, other):
+        return self._heap == other._heap
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
     def __repr__(self):
-        return 'LimitedSet(%s)' % (repr(list(self._data))[:100], )
+        return 'LimitedSet({0})'.format(len(self))
 
-    @property
-    def chronologically(self):
-        return [value for _, value in self._heap]
+    def __iter__(self):
+        return (item[1] for item in self._heap)
 
-    @property
-    def first(self):
-        """Get the oldest member."""
-        return self._heap[0][1]
+    def __len__(self):
+        return len(self._heap)
+
+    def __contains__(self, key):
+        return key in self._data
+
+    def __reduce__(self):
+        return self.__class__, (
+            self.maxlen, self.expires, self._data, self._heap,
+        )
+MutableSet.register(LimitedSet)
