@@ -9,6 +9,7 @@
 from __future__ import absolute_import
 
 import anyjson
+import imp as _imp
 import importlib
 import os
 import re
@@ -20,25 +21,22 @@ from kombu.utils import cached_property
 from kombu.utils.encoding import safe_str
 
 from celery.datastructures import DictAttribute
-from celery.exceptions import ImproperlyConfigured
+from celery.five import reraise, string_t
+from celery.utils.functional import maybe_list
 from celery.utils.imports import (
     import_from_cwd, symbol_by_name, NotAPackage, find_module,
 )
-from celery.utils.functional import maybe_list
 
-ERROR_ENVVAR_NOT_SET = """\
-The environment variable %r is not set,
-and as such the configuration could not be loaded.
-Please set this variable and make it point to
-a configuration module."""
+__all__ = ['BaseLoader']
 
-CONFIG_INVALID_NAME = """
-Error: Module '%(module)s' doesn't exist, or it's not a valid \
+_RACE_PROTECTION = False
+CONFIG_INVALID_NAME = """\
+Error: Module '{module}' doesn't exist, or it's not a valid \
 Python module name.
 """
 
-CONFIG_WITH_SUFFIX = CONFIG_INVALID_NAME + """
-Did you mean '%(suggest)s'?
+CONFIG_WITH_SUFFIX = CONFIG_INVALID_NAME + """\
+Did you mean '{suggest}'?
 """
 
 
@@ -63,15 +61,13 @@ class BaseLoader(object):
     """
     builtin_modules = frozenset()
     configured = False
-    error_envvar_not_set = ERROR_ENVVAR_NOT_SET
     override_backends = {}
     worker_initialized = False
 
     _conf = None
 
-    def __init__(self, app=None, **kwargs):
-        from celery.app import app_or_default
-        self.app = app_or_default(app)
+    def __init__(self, app, **kwargs):
+        self.app = app
         self.task_modules = set()
 
     def now(self, utc=True):
@@ -136,21 +132,10 @@ class BaseLoader(object):
     def init_worker_process(self):
         self.on_worker_process_init()
 
-    def config_from_envvar(self, variable_name, silent=False):
-        module_name = os.environ.get(variable_name)
-        if not module_name:
-            if silent:
-                return False
-            raise ImproperlyConfigured(self.error_envvar_not_set % module_name)
-        return self.config_from_object(module_name, silent=silent)
-
     def config_from_object(self, obj, silent=False):
-        if isinstance(obj, basestring):
+        if isinstance(obj, string_t):
             try:
-                if '.' in obj:
-                    obj = symbol_by_name(obj, imp=self.import_from_cwd)
-                else:
-                    obj = self.import_from_cwd(obj)
+                obj = self._smart_import(obj, imp=self.import_from_cwd)
             except (ImportError, AttributeError):
                 if silent:
                     return False
@@ -160,15 +145,30 @@ class BaseLoader(object):
         self._conf = obj
         return True
 
+    def _smart_import(self, path, imp=None):
+        imp = self.import_module if imp is None else imp
+        if ':' in path:
+            # Path includes attribute so can just jump here.
+            # e.g. ``os.path:abspath``.
+            return symbol_by_name(path, imp=imp)
+
+        # Not sure if path is just a module name or if it includes an
+        # attribute name (e.g. ``os.path``, vs, ``os.path.abspath``
+        try:
+            return imp(path)
+        except ImportError:
+            # Not a module name, so try module + attribute.
+            return symbol_by_name(path, imp=imp)
+
     def _import_config_module(self, name):
         try:
             self.find_module(name)
         except NotAPackage:
             if name.endswith('.py'):
-                raise NotAPackage, NotAPackage(CONFIG_WITH_SUFFIX % {
-                    'module': name, 'suggest': name[:-3]}), sys.exc_info()[2]
-            raise NotAPackage, NotAPackage(
-                CONFIG_INVALID_NAME % {'module': name}), sys.exc_info()[2]
+                reraise(NotAPackage, NotAPackage(CONFIG_WITH_SUFFIX.format(
+                    module=name, suggest=name[:-3])), sys.exc_info()[2])
+            reraise(NotAPackage, NotAPackage(CONFIG_INVALID_NAME.format(
+                module=name)), sys.exc_info()[2])
         else:
             return self.import_from_cwd(name)
 
@@ -188,7 +188,7 @@ class BaseLoader(object):
 
         def getarg(arg):
             """Parse a single configuration definition from
-            the command line."""
+            the command-line."""
 
             ## find key/value
             # ns.key=value|ns_key=value (case insensitive)
@@ -215,11 +215,11 @@ class BaseLoader(object):
             else:
                 try:
                     value = NAMESPACES[ns][key].to_python(value)
-                except ValueError, exc:
+                except ValueError as exc:
                     # display key name in error message.
-                    raise ValueError('%r: %s' % (ns_key, exc))
+                    raise ValueError('{0!r}: {1}'.format(ns_key, exc))
             return ns_key, value
-        return dict(getarg(v) for v in args)
+        return dict(getarg(arg) for arg in args)
 
     def mail_admins(self, subject, body, fail_silently=False,
                     sender=None, to=None, host=None, port=None,
@@ -234,9 +234,9 @@ class BaseLoader(object):
                                   use_tls=use_tls)
         mailer.send(message, fail_silently=fail_silently)
 
-    def read_configuration(self):
+    def read_configuration(self, env='CELERY_CONFIG_MODULE'):
         try:
-            custom_config = os.environ['CELERY_CONFIG_MODULE']
+            custom_config = os.environ[env]
         except KeyError:
             pass
         else:
@@ -245,6 +245,11 @@ class BaseLoader(object):
                 return DictAttribute(usercfg)
         return {}
 
+    def autodiscover_tasks(self, packages, related_name='tasks'):
+        self.task_modules.update(
+            mod.__name__ for mod in autodiscover_tasks(packages,
+                                                       related_name) if mod)
+
     @property
     def conf(self):
         """Loader configuration."""
@@ -255,3 +260,32 @@ class BaseLoader(object):
     @cached_property
     def mail(self):
         return self.import_module('celery.utils.mail')
+
+
+def autodiscover_tasks(packages, related_name='tasks'):
+    global _RACE_PROTECTION
+
+    if _RACE_PROTECTION:
+        return
+    _RACE_PROTECTION = True
+    try:
+        return [find_related_module(pkg, related_name) for pkg in packages]
+    finally:
+        _RACE_PROTECTION = False
+
+
+def find_related_module(package, related_name):
+    """Given a package name and a module name, tries to find that
+    module."""
+
+    try:
+        pkg_path = importlib.import_module(package).__path__
+    except AttributeError:
+        return
+
+    try:
+        _imp.find_module(related_name, pkg_path)
+    except ImportError:
+        return
+
+    return importlib.import_module('{0}.{1}'.format(package, related_name))
