@@ -7,28 +7,39 @@
     users, groups, and so on.
 
 """
-from __future__ import absolute_import
-from __future__ import with_statement
+from __future__ import absolute_import, print_function
 
 import atexit
 import errno
+import math
 import os
 import platform as _platform
-import shlex
 import signal as _signal
 import sys
 
 from billiard import current_process
+# fileno used to be in this module
+from kombu.utils import maybe_fileno
+from kombu.utils.compat import get_errno
 from kombu.utils.encoding import safe_str
 from contextlib import contextmanager
 
 from .local import try_import
+from .five import items, range, reraise, string_t
 
 _setproctitle = try_import('setproctitle')
 resource = try_import('resource')
 pwd = try_import('pwd')
 grp = try_import('grp')
 
+__all__ = ['EX_OK', 'EX_FAILURE', 'EX_UNAVAILABLE', 'EX_USAGE', 'SYSTEM',
+           'IS_OSX', 'IS_WINDOWS', 'pyimplementation', 'LockFailed',
+           'get_fdmax', 'Pidfile', 'create_pidlock',
+           'close_open_fds', 'DaemonContext', 'detached', 'parse_uid',
+           'parse_gid', 'setgroups', 'initgroups', 'setgid', 'setuid',
+           'maybe_drop_privileges', 'signals', 'set_process_title',
+           'set_mp_process_title', 'get_errno_name', 'ignore_errno']
+
 # exitcodes
 EX_OK = getattr(os, 'EX_OK', 0)
 EX_FAILURE = 1
@@ -39,23 +50,18 @@ SYSTEM = _platform.system()
 IS_OSX = SYSTEM == 'Darwin'
 IS_WINDOWS = SYSTEM == 'Windows'
 
+DAEMON_UMASK = 0
 DAEMON_WORKDIR = '/'
 
 PIDFILE_FLAGS = os.O_CREAT | os.O_EXCL | os.O_WRONLY
 PIDFILE_MODE = ((os.R_OK | os.W_OK) << 6) | ((os.R_OK) << 3) | ((os.R_OK))
 
-PIDLOCKED = """ERROR: Pidfile (%s) already exists.
-Seems we're already running? (pid: %s)"""
-
-try:
-    from io import UnsupportedOperation
-    FILENO_ERRORS = (AttributeError, ValueError, UnsupportedOperation)
-except ImportError:  # Py2
-    FILENO_ERRORS = (AttributeError, ValueError)  # noqa
+PIDLOCKED = """ERROR: Pidfile ({0}) already exists.
+Seems we're already running? (pid: {1})"""
 
 
 def pyimplementation():
-    """Returns string identifying the current Python implementation."""
+    """Return string identifying the current Python implementation."""
     if hasattr(_platform, 'python_implementation'):
         return _platform.python_implementation()
     elif sys.platform.startswith('java'):
@@ -69,51 +75,20 @@ def pyimplementation():
         return 'CPython'
 
 
-def _find_option_with_arg(argv, short_opts=None, long_opts=None):
-    """Search argv for option specifying its short and longopt
-    alternatives.
-
-    Returns the value of the option if found.
-
-    """
-    for i, arg in enumerate(argv):
-        if arg.startswith('-'):
-            if long_opts and arg.startswith('--'):
-                name, _, val = arg.partition('=')
-                if name in long_opts:
-                    return val
-            if short_opts and arg in short_opts:
-                return argv[i + 1]
-    raise KeyError('|'.join(short_opts or [] + long_opts or []))
-
-
-def maybe_patch_concurrency(argv, short_opts=None, long_opts=None):
-    """With short and long opt alternatives that specify the command line
-    option to set the pool, this makes sure that anything that needs
-    to be patched is completed as early as possible.
-    (e.g. eventlet/gevent monkey patches)."""
-    try:
-        pool = _find_option_with_arg(argv, short_opts, long_opts)
-    except KeyError:
-        pass
-    else:
-        # set up eventlet/gevent environments ASAP.
-        from celery import concurrency
-        concurrency.get_implementation(pool)
-
-
 class LockFailed(Exception):
     """Raised if a pidlock can't be acquired."""
 
 
 def get_fdmax(default=None):
-    """Returns the maximum number of open file descriptors
+    """Return the maximum number of open file descriptors
     on this system.
 
     :keyword default: Value returned if there's no file
                       descriptor limit.
 
     """
+    if resource is None:  # Windows
+        return default
     fdmax = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
     if fdmax == resource.RLIM_INFINITY:
         return default
@@ -141,13 +116,13 @@ class Pidfile(object):
         """Acquire lock."""
         try:
             self.write_pid()
-        except OSError, exc:
-            raise LockFailed, LockFailed(str(exc)), sys.exc_info()[2]
+        except OSError as exc:
+            reraise(LockFailed, LockFailed(str(exc)), sys.exc_info()[2])
         return self
     __enter__ = acquire
 
     def is_locked(self):
-        """Returns true if the pid lock exists."""
+        """Return true if the pid lock exists."""
         return os.path.exists(self.path)
 
     def release(self, *args):
@@ -156,32 +131,32 @@ class Pidfile(object):
     __exit__ = release
 
     def read_pid(self):
-        """Reads and returns the current pid."""
+        """Read and return the current pid."""
         with ignore_errno('ENOENT'):
             with open(self.path, 'r') as fh:
                 line = fh.readline()
                 if line.strip() == line:  # must contain '\n'
                     raise ValueError(
-                        'Partially written or invalid pidfile %r' % self.path)
+                        'Partial or invalid pidfile {0.path}'.format(self))
 
                 try:
                     return int(line.strip())
                 except ValueError:
                     raise ValueError(
-                        'pidfile %r contents invalid.' % self.path)
+                        'pidfile {0.path} contents invalid.'.format(self))
 
     def remove(self):
-        """Removes the lock."""
+        """Remove the lock."""
         with ignore_errno(errno.ENOENT, errno.EACCES):
             os.unlink(self.path)
 
     def remove_if_stale(self):
-        """Removes the lock if the process is not running.
+        """Remove the lock if the process is not running.
         (does not respond to signals)."""
         try:
             pid = self.read_pid()
-        except ValueError, exc:
-            sys.stderr.write('Broken pidfile found. Removing it.\n')
+        except ValueError as exc:
+            print('Broken pidfile found. Removing it.', file=sys.stderr)
             self.remove()
             return True
         if not pid:
@@ -190,16 +165,16 @@ class Pidfile(object):
 
         try:
             os.kill(pid, 0)
-        except os.error, exc:
+        except os.error as exc:
             if exc.errno == errno.ESRCH:
-                sys.stderr.write('Stale pidfile exists. Removing it.\n')
+                print('Stale pidfile exists. Removing it.', file=sys.stderr)
                 self.remove()
                 return True
         return False
 
     def write_pid(self):
         pid = os.getpid()
-        content = '%d\n' % (pid, )
+        content = '{0}\n'.format(pid)
 
         pidfile_fd = os.open(self.path, PIDFILE_FLAGS, PIDFILE_MODE)
         pidfile = os.fdopen(pidfile_fd, 'w')
@@ -252,25 +227,11 @@ def create_pidlock(pidfile):
 def _create_pidlock(pidfile):
     pidlock = Pidfile(pidfile)
     if pidlock.is_locked() and not pidlock.remove_if_stale():
-        raise SystemExit(PIDLOCKED % (pidfile, pidlock.read_pid()))
+        raise SystemExit(PIDLOCKED.format(pidfile, pidlock.read_pid()))
     pidlock.acquire()
     return pidlock
 
 
-def fileno(f):
-    if isinstance(f, (int, long)):
-        return f
-    return f.fileno()
-
-
-def maybe_fileno(f):
-    """Get object fileno, or :const:`None` if not defined."""
-    try:
-        return fileno(f)
-    except FILENO_ERRORS:
-        pass
-
-
 def close_open_fds(keep=None):
     keep = [maybe_fileno(f) for f in keep if maybe_fileno(f)] if keep else []
     for fd in reversed(range(get_fdmax(default=2048))):
@@ -284,10 +245,8 @@ class DaemonContext(object):
 
     def __init__(self, pidfile=None, workdir=None, umask=None,
                  fake=False, after_chdir=None, **kwargs):
-        if isinstance(umask, basestring):
-            umask = int(umask, 8)  # convert str -> octal
         self.workdir = workdir or DAEMON_WORKDIR
-        self.umask = umask
+        self.umask = DAEMON_UMASK if umask is None else umask
         self.fake = fake
         self.after_chdir = after_chdir
         self.stdfds = (sys.stdin, sys.stdout, sys.stderr)
@@ -303,8 +262,7 @@ class DaemonContext(object):
                 self._detach()
 
             os.chdir(self.workdir)
-            if self.umask is not None:
-                os.umask(self.umask)
+            os.umask(self.umask)
 
             if self.after_chdir:
                 self.after_chdir()
@@ -393,7 +351,7 @@ def parse_uid(uid):
     """Parse user id.
 
     uid can be an integer (uid) or a string (user name), if a user name
-    the uid is taken from the password file.
+    the uid is taken from the system user registry.
 
     """
     try:
@@ -402,14 +360,14 @@ def parse_uid(uid):
         try:
             return pwd.getpwnam(uid).pw_uid
         except (AttributeError, KeyError):
-            raise KeyError('User does not exist: %r' % (uid, ))
+            raise KeyError('User does not exist: {0}'.format(uid))
 
 
 def parse_gid(gid):
     """Parse group id.
 
     gid can be an integer (gid) or a string (group name), if a group name
-    the gid is taken from the password file.
+    the gid is taken from the system group registry.
 
     """
     try:
@@ -418,7 +376,7 @@ def parse_gid(gid):
         try:
             return grp.getgrnam(gid).gr_gid
         except (AttributeError, KeyError):
-            raise KeyError('Group does not exist: %r' % (gid, ))
+            raise KeyError('Group does not exist: {0}'.format(gid))
 
 
 def _setgroups_hack(groups):
@@ -434,7 +392,7 @@ def _setgroups_hack(groups):
             if len(groups) <= 1:
                 raise
             groups[:] = groups[:-1]
-        except OSError, exc:  # error from the OS.
+        except OSError as exc:  # error from the OS.
             if exc.errno != errno.EINVAL or len(groups) <= 1:
                 raise
             groups[:] = groups[:-1]
@@ -449,7 +407,7 @@ def setgroups(groups):
         pass
     try:
         return _setgroups_hack(groups[:max_groups])
-    except OSError, exc:
+    except OSError as exc:
         if exc.errno != errno.EPERM:
             raise
         if any(group not in groups for group in os.getgroups()):
@@ -500,11 +458,26 @@ def maybe_drop_privileges(uid=None, gid=None):
             gid = pwd.getpwuid(uid).pw_gid
         # Must set the GID before initgroups(), as setgid()
         # is known to zap the group list on some platforms.
+
+        # setgid must happen before setuid (otherwise the setgid operation
+        # may fail because of insufficient privileges and possibly stay
+        # in a privileged group).
         setgid(gid)
         initgroups(uid, gid)
 
         # at last:
         setuid(uid)
+        # ... and make sure privileges cannot be restored:
+        try:
+            setuid(0)
+        except OSError as exc:
+            if get_errno(exc) != errno.EPERM:
+                raise
+            pass  # Good: cannot restore privileges.
+        else:
+            raise RuntimeError(
+                'non-root user able to restore privileges after setuid.')
+
     else:
         gid and setgid(gid)
 
@@ -549,8 +522,27 @@ class Signals(object):
     ignored = _signal.SIG_IGN
     default = _signal.SIG_DFL
 
+    if hasattr(_signal, 'setitimer'):
+
+        def arm_alarm(self, seconds):
+            _signal.setitimer(_signal.ITIMER_REAL, seconds)
+    else:  # pragma: no cover
+        try:
+            from itimer import alarm as _itimer_alarm  # noqa
+        except ImportError:
+
+            def arm_alarm(self, seconds):  # noqa
+                _signal.alarm(math.ceil(seconds))
+        else:  # pragma: no cover
+
+            def arm_alarm(self, seconds):      # noqa
+                return _itimer_alarm(seconds)  # noqa
+
+    def reset_alarm(self):
+        return _signal.alarm(0)
+
     def supported(self, signal_name):
-        """Returns true value if ``signal_name`` exists on this platform."""
+        """Return true value if ``signal_name`` exists on this platform."""
         try:
             return self.signum(signal_name)
         except AttributeError:
@@ -560,7 +552,7 @@ class Signals(object):
         """Get signal number from signal name."""
         if isinstance(signal_name, int):
             return signal_name
-        if not isinstance(signal_name, basestring) \
+        if not isinstance(signal_name, string_t) \
                 or not signal_name.isupper():
             raise TypeError('signal name must be uppercase string.')
         if not signal_name.startswith('SIG'):
@@ -602,10 +594,9 @@ class Signals(object):
 
     def update(self, _d_=None, **sigmap):
         """Set signal handlers from a mapping."""
-        for signal_name, handler in dict(_d_ or {}, **sigmap).iteritems():
+        for signal_name, handler in items(dict(_d_ or {}, **sigmap)):
             self[signal_name] = handler
 
-
 signals = Signals()
 get_signal = signals.signum                   # compat
 install_signal_handler = signals.__setitem__  # compat
@@ -626,8 +617,8 @@ def set_process_title(progname, info=None):
     Only works if :mod:`setproctitle` is installed.
 
     """
-    proctitle = '[%s]' % progname
-    proctitle = '%s %s' % (proctitle, info) if info else proctitle
+    proctitle = '[{0}]'.format(progname)
+    proctitle = '{0} {1}'.format(proctitle, info) if info else proctitle
     if _setproctitle:
         _setproctitle.setproctitle(safe_str(proctitle))
     return proctitle
@@ -646,27 +637,14 @@ else:
 
         """
         if hostname:
-            progname = '%s@%s' % (progname, hostname.split('.')[0])
+            progname = '{0}: {1}'.format(progname, hostname)
         return set_process_title(
-            '%s:%s' % (progname, current_process().name), info=info)
+            '{0}:{1}'.format(progname, current_process().name), info=info)
 
 
-def shellsplit(s):
-    """Compat. version of :func:`shlex.split` that supports
-    the ``posix`` option which was first added in Python 2.6.
-
-    Posix behavior will be disabled if running under Windows.
-
-    """
-    lexer = shlex.shlex(s, posix=not IS_WINDOWS)
-    lexer.whitespace_split = True
-    lexer.commenters = ''
-    return list(lexer)
-
-
-def get_errno(n):
+def get_errno_name(n):
     """Get errno for string, e.g. ``ENOENT``."""
-    if isinstance(n, basestring):
+    if isinstance(n, string_t):
         return getattr(errno, n)
     return n
 
@@ -689,10 +667,10 @@ def ignore_errno(*errnos, **kwargs):
                     defaults to :exc:`Exception`.
     """
     types = kwargs.get('types') or (Exception, )
-    errnos = [get_errno(errno) for errno in errnos]
+    errnos = [get_errno_name(errno) for errno in errnos]
     try:
         yield
-    except types, exc:
+    except types as exc:
         if not hasattr(exc, 'errno'):
             raise
         if exc.errno not in errnos:
