@@ -7,7 +7,6 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import time
 
@@ -20,13 +19,20 @@ from kombu.utils.compat import OrderedDict
 from . import current_app
 from . import states
 from .app import app_or_default
-from .datastructures import DependencyGraph
+from .datastructures import DependencyGraph, GraphFormatter
 from .exceptions import IncompleteStream, TimeoutError
+from .five import items, range, string_t, monotonic
+
+__all__ = ['ResultBase', 'AsyncResult', 'ResultSet', 'GroupResult',
+           'EagerResult', 'result_from_tuple']
 
 
 class ResultBase(object):
     """Base class for all results"""
 
+    #: Parent result (if part of a chain)
+    parent = None
+
 
 class AsyncResult(ResultBase):
     """Query task state.
@@ -46,9 +52,6 @@ class AsyncResult(ResultBase):
     #: The task result backend to use.
     backend = None
 
-    #: Parent result (if part of a chain)
-    parent = None
-
     def __init__(self, id, backend=None, task_name=None,
                  app=None, parent=None):
         self.app = app_or_default(app or self.app)
@@ -57,14 +60,17 @@ class AsyncResult(ResultBase):
         self.task_name = task_name
         self.parent = parent
 
-    def serializable(self):
-        return self.id, None
+    def as_tuple(self):
+        parent = self.parent
+        return (self.id, parent and parent.as_tuple()), None
+    serializable = as_tuple   # XXX compat
 
     def forget(self):
         """Forget about (and possibly remove the result of) this task."""
         self.backend.forget(self.id)
 
-    def revoke(self, connection=None, terminate=False, signal=None):
+    def revoke(self, connection=None, terminate=False, signal=None,
+               wait=False, timeout=None):
         """Send revoke signal to all workers.
 
         Any worker receiving the task, or having reserved the
@@ -74,10 +80,15 @@ class AsyncResult(ResultBase):
             on the task (if any).
         :keyword signal: Name of signal to send to process if terminate.
             Default is TERM.
+        :keyword wait: Wait for replies from workers.  Will wait for 1 second
+           by default or you can specify a custom ``timeout``.
+        :keyword timeout: Time in seconds to wait for replies if ``wait``
+                          enabled.
 
         """
         self.app.control.revoke(self.id, connection=connection,
-                                terminate=terminate, signal=signal)
+                                terminate=terminate, signal=signal,
+                                reply=wait, timeout=timeout)
 
     def get(self, timeout=None, propagate=True, interval=0.5):
         """Wait until task is ready, and return its result.
@@ -103,11 +114,21 @@ class AsyncResult(ResultBase):
         be re-raised.
 
         """
+        if propagate and self.parent:
+            for node in reversed(list(self._parents())):
+                node.get(propagate=True, timeout=timeout, interval=interval)
+
         return self.backend.wait_for(self.id, timeout=timeout,
                                      propagate=propagate,
                                      interval=interval)
     wait = get  # deprecated alias to :meth:`get`.
 
+    def _parents(self):
+        node = self.parent
+        while node:
+            yield node
+            node = node.parent
+
     def collect(self, intermediate=False, **kwargs):
         """Iterator, like :meth:`get` will wait for the task to complete,
         but will also follow :class:`AsyncResult` and :class:`ResultSet`
@@ -119,7 +140,7 @@ class AsyncResult(ResultBase):
 
             @task()
             def A(how_many):
-                return group(B.s(i) for i in xrange(how_many))
+                return group(B.s(i) for i in range(how_many))
 
             @task()
             def B(i):
@@ -176,11 +197,13 @@ class AsyncResult(ResultBase):
         """Returns :const:`True` if the task failed."""
         return self.state == states.FAILURE
 
-    def build_graph(self, intermediate=False):
-        graph = DependencyGraph()
+    def build_graph(self, intermediate=False, formatter=None):
+        graph = DependencyGraph(
+            formatter=formatter or GraphFormatter(root=self.id, shape='oval'),
+        )
         for parent, node in self.iterdeps(intermediate=intermediate):
+            graph.add_arc(node)
             if parent:
-                graph.add_arc(parent)
                 graph.add_edge(parent, node)
         return graph
 
@@ -193,15 +216,18 @@ class AsyncResult(ResultBase):
         return hash(self.id)
 
     def __repr__(self):
-        return '<%s: %s>' % (self.__class__.__name__, self.id)
+        return '<{0}: {1}>'.format(type(self).__name__, self.id)
 
     def __eq__(self, other):
         if isinstance(other, AsyncResult):
             return other.id == self.id
-        elif isinstance(other, basestring):
+        elif isinstance(other, string_t):
             return other == self.id
         return NotImplemented
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __copy__(self):
         return self.__class__(
             self.id, self.backend, self.task_name, self.app, self.parent,
@@ -225,7 +251,7 @@ class AsyncResult(ResultBase):
     def children(self):
         children = self.backend.get_children(self.id)
         if children:
-            return [from_serializable(r, self.app) for r in children]
+            return [result_from_tuple(child, self.app) for child in children]
 
     @property
     def result(self):
@@ -273,12 +299,14 @@ class AsyncResult(ResultBase):
         return self.backend.get_status(self.id)
     status = state
 
-    def _get_task_id(self):
+    @property
+    def task_id(self):
+        """compat alias to :attr:`id`"""
         return self.id
 
-    def _set_task_id(self, id):
+    @task_id.setter  # noqa
+    def task_id(self, id):
         self.id = id
-    task_id = property(_get_task_id, _set_task_id)
 BaseAsyncResult = AsyncResult  # for backwards compatibility.
 
 
@@ -307,12 +335,12 @@ class ResultSet(ResultBase):
             self.results.append(result)
 
     def remove(self, result):
-        """Removes result from the set; it must be a member.
+        """Remove result from the set; it must be a member.
 
         :raises KeyError: if the result is not a member.
 
         """
-        if isinstance(result, basestring):
+        if isinstance(result, string_t):
             result = self.app.AsyncResult(result)
         try:
             self.results.remove(result)
@@ -388,23 +416,26 @@ class ResultSet(ResultBase):
         for result in self.results:
             result.forget()
 
-    def revoke(self, connection=None, terminate=False, signal=None):
+    def revoke(self, connection=None, terminate=False, signal=None,
+               wait=False, timeout=None):
         """Send revoke signal to all workers for all tasks in the set.
 
         :keyword terminate: Also terminate the process currently working
             on the task (if any).
         :keyword signal: Name of signal to send to process if terminate.
             Default is TERM.
+        :keyword wait: Wait for replies from worker.  Will wait for 1 second
+           by default or you can specify a custom ``timeout``.
+        :keyword timeout: Time in seconds to wait for replies if ``wait``
+                          enabled.
 
         """
-        with self.app.connection_or_acquire(connection) as conn:
-            for result in self.results:
-                result.revoke(
-                    connection=conn, terminate=terminate, signal=signal,
-                )
+        self.app.control.revoke([r.id for r in self.results],
+                                connection=connection, timeout=timeout,
+                                terminate=terminate, signal=signal, reply=wait)
 
     def __iter__(self):
-        return self.iterate()
+        return iter(self.results)
 
     def __getitem__(self, index):
         """`res[i] -> res.results[i]`"""
@@ -423,7 +454,7 @@ class ResultSet(ResultBase):
 
         while results:
             removed = set()
-            for task_id, result in results.iteritems():
+            for task_id, result in items(results):
                 if result.ready():
                     yield result.get(timeout=timeout and timeout - elapsed,
                                      propagate=propagate)
@@ -438,7 +469,7 @@ class ResultSet(ResultBase):
             if timeout and elapsed >= timeout:
                 raise TimeoutError('The operation timed out')
 
-    def get(self, timeout=None, propagate=True, interval=0.5):
+    def get(self, timeout=None, propagate=True, interval=0.5, callback=None):
         """See :meth:`join`
 
         This is here for API compatibility with :class:`AsyncResult`,
@@ -447,9 +478,10 @@ class ResultSet(ResultBase):
 
         """
         return (self.join_native if self.supports_native_join else self.join)(
-            timeout=timeout, propagate=propagate, interval=interval)
+            timeout=timeout, propagate=propagate,
+            interval=interval, callback=callback)
 
-    def join(self, timeout=None, propagate=True, interval=0.5):
+    def join(self, timeout=None, propagate=True, interval=0.5, callback=None):
         """Gathers the results of all tasks as a list in order.
 
         .. note::
@@ -476,27 +508,37 @@ class ResultSet(ResultBase):
                            does not have any effect when using the amqp
                            result store backend, as it does not use polling.
 
+        :keyword callback: Optional callback to be called for every result
+                           received.  Must have signature ``(task_id, value)``
+                           No results will be returned by this function if
+                           a callback is specified.  The order of results
+                           is also arbitrary when a callback is used.
+
         :raises celery.exceptions.TimeoutError: if `timeout` is not
             :const:`None` and the operation takes longer than `timeout`
             seconds.
 
         """
-        time_start = time.time()
+        time_start = monotonic()
         remaining = None
 
         results = []
         for result in self.results:
             remaining = None
             if timeout:
-                remaining = timeout - (time.time() - time_start)
+                remaining = timeout - (monotonic() - time_start)
                 if remaining <= 0.0:
                     raise TimeoutError('join operation timed out')
-            results.append(result.get(timeout=remaining,
-                                      propagate=propagate,
-                                      interval=interval))
+            value = result.get(timeout=remaining,
+                               propagate=propagate,
+                               interval=interval)
+            if callback:
+                callback(result.id, value)
+            else:
+                results.append(value)
         return results
 
-    def iter_native(self, timeout=None, interval=None):
+    def iter_native(self, timeout=None, interval=0.5):
         """Backend optimized version of :meth:`iterate`.
 
         .. versionadded:: 2.2
@@ -508,13 +550,15 @@ class ResultSet(ResultBase):
         result backends.
 
         """
-        if not self.results:
+        results = self.results
+        if not results:
             return iter([])
-        backend = self.results[0].backend
-        ids = [result.id for result in self.results]
-        return backend.get_many(ids, timeout=timeout, interval=interval)
+        return results[0].backend.get_many(
+            set(r.id for r in results), timeout=timeout, interval=interval,
+        )
 
-    def join_native(self, timeout=None, propagate=True, interval=0.5):
+    def join_native(self, timeout=None, propagate=True,
+                    interval=0.5, callback=None):
         """Backend optimized version of :meth:`join`.
 
         .. versionadded:: 2.2
@@ -526,13 +570,18 @@ class ResultSet(ResultBase):
         result backends.
 
         """
-        results = self.results
-        acc = [None for _ in xrange(len(self))]
-        for task_id, meta in self.iter_native(timeout=timeout,
-                                              interval=interval):
+        order_index = None if callback else dict(
+            (result.id, i) for i, result in enumerate(self.results)
+        )
+        acc = None if callback else [None for _ in range(len(self))]
+        for task_id, meta in self.iter_native(timeout, interval):
+            value = meta['result']
             if propagate and meta['status'] in states.PROPAGATE_STATES:
-                raise meta['result']
-            acc[results.index(task_id)] = meta['result']
+                raise value
+            if callback:
+                callback(task_id, value)
+            else:
+                acc[order_index[task_id]] = value
         return acc
 
     def _failed_join_report(self):
@@ -548,9 +597,12 @@ class ResultSet(ResultBase):
             return other.results == self.results
         return NotImplemented
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __repr__(self):
-        return '<%s: [%s]>' % (self.__class__.__name__,
-                               ', '.join(r.id for r in self.results))
+        return '<{0}: [{1}]>'.format(type(self).__name__,
+                                     ', '.join(r.id for r in self.results))
 
     @property
     def subtasks(self):
@@ -612,12 +664,16 @@ class GroupResult(ResultSet):
             return other.id == self.id and other.results == self.results
         return NotImplemented
 
-    def __repr__(self):
-        return '<%s: %s [%s]>' % (self.__class__.__name__, self.id,
-                                  ', '.join(r.id for r in self.results))
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
-    def serializable(self):
-        return self.id, [r.serializable() for r in self.results]
+    def __repr__(self):
+        return '<{0}: {1} [{2}]>'.format(type(self).__name__, self.id,
+                                         ', '.join(r.id for r in self.results))
+
+    def as_tuple(self):
+        return self.id, [r.as_tuple() for r in self.results]
+    serializable = as_tuple   # XXX compat
 
     @property
     def children(self):
@@ -650,12 +706,14 @@ class TaskSetResult(GroupResult):
         """Deprecated: Use ``len(r)``."""
         return len(self)
 
-    def _get_taskset_id(self):
+    @property
+    def taskset_id(self):
+        """compat alias to :attr:`self.id`"""
         return self.id
 
-    def _set_taskset_id(self, id):
+    @taskset_id.setter  # noqa
+    def taskset_id(self, id):
         self.id = id
-    taskset_id = property(_get_taskset_id, _set_taskset_id)
 
 
 class EagerResult(AsyncResult):
@@ -697,7 +755,7 @@ class EagerResult(AsyncResult):
         self._state = states.REVOKED
 
     def __repr__(self):
-        return '<EagerResult: %s>' % self.id
+        return '<EagerResult: {0.id}>'.format(self)
 
     @property
     def result(self):
@@ -720,17 +778,21 @@ class EagerResult(AsyncResult):
         return False
 
 
-def from_serializable(r, app=None):
+def result_from_tuple(r, app=None):
     # earlier backends may just pickle, so check if
     # result is already prepared.
     app = app_or_default(app)
     Result = app.AsyncResult
     if not isinstance(r, ResultBase):
-        if isinstance(r, (list, tuple)):
-            id, nodes = r
-            if nodes:
-                return app.GroupResult(id, [Result(sid) for sid, _ in nodes])
-            return Result(id)
-        else:
-            return Result(r)
+        res, nodes = r
+        if nodes:
+            return app.GroupResult(
+                res, [result_from_tuple(child, app) for child in nodes],
+            )
+        # previously did not include parent
+        id, parent = res if isinstance(res, (list, tuple)) else (res, None)
+        if parent:
+            parent = result_from_tuple(parent, app)
+        return Result(id, parent=parent)
     return r
+from_serializable = result_from_tuple  # XXX compat
