@@ -11,19 +11,41 @@ from __future__ import absolute_import
 
 import re
 
+from collections import namedtuple
 from datetime import datetime, timedelta
 
-from dateutil.relativedelta import relativedelta
 from kombu.utils import cached_property
 
 from . import current_app
+from .five import range, string_t
 from .utils import is_iterable
 from .utils.timeutils import (
     timedelta_seconds, weekday, maybe_timedelta, remaining,
-    humanize_seconds, timezone, maybe_make_aware
+    humanize_seconds, timezone, maybe_make_aware, ffwd
 )
 from .datastructures import AttributeDict
 
+__all__ = ['ParseException', 'schedule', 'crontab', 'crontab_parser',
+           'maybe_schedule']
+
+schedstate = namedtuple('schedstate', ('is_due', 'next'))
+
+
+CRON_PATTERN_INVALID = """\
+Invalid crontab pattern. Valid range is {min}-{max}. \
+'{value}' was found.\
+"""
+
+CRON_INVALID_TYPE = """\
+Argument cronspec needs to be of any of the following types: \
+int, str, or an iterable type. {type!r} was given.\
+"""
+
+CRON_REPR = """\
+<crontab: {0._orig_minute} {0._orig_hour} {0._orig_day_of_week} \
+{0._orig_day_of_month} {0._orig_month_of_year} (m/h/d/dM/MY)>\
+"""
+
 
 def cronfield(s):
     return '*' if s is None else s
@@ -34,12 +56,23 @@ class ParseException(Exception):
 
 
 class schedule(object):
+    """Schedule for periodic task.
+
+    :param run_every: Interval in seconds (or a :class:`~datetime.timedelta`).
+    :param relative:  If set to True the run time will be rounded to the
+        resolution of the interval.
+    :param nowfun: Function returning the current date and time
+        (class:`~datetime.datetime`).
+    :param app: Celery app instance.
+
+    """
     relative = False
 
-    def __init__(self, run_every=None, relative=False, nowfun=None):
+    def __init__(self, run_every=None, relative=False, nowfun=None, app=None):
         self.run_every = maybe_timedelta(run_every)
         self.relative = relative
         self.nowfun = nowfun
+        self._app = app
 
     def now(self):
         return (self.nowfun or self.app.now)()
@@ -63,8 +96,8 @@ class schedule(object):
 
         You can override this to decide the interval at runtime,
         but keep in mind the value of :setting:`CELERYBEAT_MAX_LOOP_INTERVAL`,
-        which decides the maximum number of seconds celerybeat can sleep
-        between re-checking the periodic task intervals.  So if you
+        which decides the maximum number of seconds the Beat scheduler can
+        sleep between re-checking the periodic task intervals.  So if you
         dynamically change the next run at value, and the max interval is
         set to 5 minutes, it will take 5 minutes for the change to take
         effect, so you may consider lowering the value of
@@ -80,10 +113,10 @@ class schedule(object):
         """
         last_run_at = self.maybe_make_aware(last_run_at)
         rem_delta = self.remaining_estimate(last_run_at)
-        rem = timedelta_seconds(rem_delta)
-        if rem == 0:
-            return True, self.seconds
-        return False, rem
+        remaining_s = timedelta_seconds(rem_delta)
+        if remaining_s == 0:
+            return schedstate(is_due=True, next=self.seconds)
+        return schedstate(is_due=False, next=remaining_s)
 
     def maybe_make_aware(self, dt):
         if self.utc_enabled:
@@ -91,13 +124,19 @@ class schedule(object):
         return dt
 
     def __repr__(self):
-        return '<freq: %s>' % self.human_seconds
+        return '<freq: {0.human_seconds}>'.format(self)
 
     def __eq__(self, other):
         if isinstance(other, schedule):
             return self.run_every == other.run_every
         return self.run_every == other
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __reduce__(self):
+        return self.__class__, (self.run_every, self.relative, self.nowfun)
+
     @property
     def seconds(self):
         return timedelta_seconds(self.run_every)
@@ -106,13 +145,17 @@ class schedule(object):
     def human_seconds(self):
         return humanize_seconds(self.seconds)
 
-    @cached_property
+    @property
     def app(self):
-        return current_app._get_current_object()
+        return self._app or current_app._get_current_object()
+
+    @app.setter  # noqa
+    def app(self, app):
+        self._app = app
 
     @cached_property
     def tz(self):
-        return timezone.get_timezone(self.app.conf.CELERY_TIMEZONE)
+        return self.app.timezone
 
     @cached_property
     def utc_enabled(self):
@@ -120,7 +163,7 @@ class schedule(object):
 
     def to_local(self, dt):
         if not self.utc_enabled:
-            return timezone.to_local_fallback(dt, self.tz)
+            return timezone.to_local_fallback(dt)
         return dt
 
 
@@ -200,10 +243,9 @@ class crontab_parser(object):
         if len(toks) > 1:
             to = self._expand_number(toks[1])
             if to < fr:  # Wrap around max_ if necessary
-                return range(fr,
-                             self.min_ + self.max_) + range(self.min_,
-                                                            to + 1)
-            return range(fr, to + 1)
+                return (list(range(fr, self.min_ + self.max_)) +
+                        list(range(self.min_, to + 1)))
+            return list(range(fr, to + 1))
         return [fr]
 
     def _range_steps(self, toks):
@@ -217,10 +259,10 @@ class crontab_parser(object):
         return self._expand_star()[::int(toks[0])]
 
     def _expand_star(self, *args):
-        return range(self.min_, self.max_ + self.min_)
+        return list(range(self.min_, self.max_ + self.min_))
 
     def _expand_number(self, s):
-        if isinstance(s, basestring) and s[0] == '-':
+        if isinstance(s, string_t) and s[0] == '-':
             raise self.ParseException('negative numbers not supported')
         try:
             i = int(s)
@@ -228,15 +270,16 @@ class crontab_parser(object):
             try:
                 i = weekday(s)
             except KeyError:
-                raise ValueError("Invalid weekday literal '%s'." % s)
+                raise ValueError('Invalid weekday literal {0!r}.'.format(s))
 
         max_val = self.min_ + self.max_ - 1
         if i > max_val:
             raise ValueError(
-                'Invalid end range: %s > %s.' % (i, max_val))
+                'Invalid end range: {0} > {1}.'.format(i, max_val))
         if i < self.min_:
             raise ValueError(
-                'Invalid beginning range: %s < %s.' % (i, self.min_))
+                'Invalid beginning range: {0} < {1}.'.format(i, self.min_))
+
         return i
 
 
@@ -297,6 +340,15 @@ class crontab(schedule):
           of every quarter) or `month_of_year='2-12/2'` (for every even
           numbered month).
 
+    .. attribute:: nowfun
+
+        Function returning the current date and time
+        (:class:`~datetime.datetime`).
+
+    .. attribute:: app
+
+        The Celery app instance.
+
     It is important to realize that any day on which execution should
     occur must be represented by entries in all three of the day and
     month attributes.  For example, if `day_of_week` is 0 and `day_of_month`
@@ -307,18 +359,33 @@ class crontab(schedule):
 
     """
 
+    def __init__(self, minute='*', hour='*', day_of_week='*',
+                 day_of_month='*', month_of_year='*', nowfun=None, app=None):
+        self._orig_minute = cronfield(minute)
+        self._orig_hour = cronfield(hour)
+        self._orig_day_of_week = cronfield(day_of_week)
+        self._orig_day_of_month = cronfield(day_of_month)
+        self._orig_month_of_year = cronfield(month_of_year)
+        self.hour = self._expand_cronspec(hour, 24)
+        self.minute = self._expand_cronspec(minute, 60)
+        self.day_of_week = self._expand_cronspec(day_of_week, 7)
+        self.day_of_month = self._expand_cronspec(day_of_month, 31, 1)
+        self.month_of_year = self._expand_cronspec(month_of_year, 12, 1)
+        self.nowfun = nowfun
+        self._app = app
+
     @staticmethod
     def _expand_cronspec(cronspec, max_, min_=0):
         """Takes the given cronspec argument in one of the forms::
 
             int         (like 7)
-            basestring  (like '3-5,*/15', '*', or 'monday')
+            str         (like '3-5,*/15', '*', or 'monday')
             set         (like set([0,15,30,45]))
             list        (like [8-17])
 
         And convert it to an (expanded) set representing all time unit
         values on which the crontab triggers.  Only in case of the base
-        type being 'basestring', parsing occurs.  (It is fast and
+        type being 'str', parsing occurs.  (It is fast and
         happens only once for each crontab instance, so there is no
         significant performance overhead involved.)
 
@@ -334,26 +401,20 @@ class crontab(schedule):
         """
         if isinstance(cronspec, int):
             result = set([cronspec])
-        elif isinstance(cronspec, basestring):
+        elif isinstance(cronspec, string_t):
             result = crontab_parser(max_, min_).parse(cronspec)
         elif isinstance(cronspec, set):
             result = cronspec
         elif is_iterable(cronspec):
             result = set(cronspec)
         else:
-            raise TypeError(
-                'Argument cronspec needs to be of any of the '
-                'following types: int, basestring, or an iterable type. '
-                "'%s' was given." % type(cronspec))
+            raise TypeError(CRON_INVALID_TYPE.format(type=type(cronspec)))
 
         # assure the result does not preceed the min or exceed the max
         for number in result:
             if number >= max_ + min_ or number < min_:
-                raise ValueError(
-                    'Invalid crontab pattern. Valid '
-                    "range is %d-%d. '%d' was found." % (
-                        min_, max_ - 1 + min_, number))
-
+                raise ValueError(CRON_PATTERN_INVALID.format(
+                    min=min_, max=max_ - 1 + min_, value=number))
         return result
 
     def _delta_to_next(self, last_run_at, next_hour, next_minute):
@@ -414,39 +475,19 @@ class crontab(schedule):
             datedata.dom += 1
             roll_over()
 
-        return relativedelta(year=datedata.year,
-                             month=months_of_year[datedata.moy],
-                             day=days_of_month[datedata.dom],
-                             hour=next_hour,
-                             minute=next_minute,
-                             second=0,
-                             microsecond=0)
-
-    def __init__(self, minute='*', hour='*', day_of_week='*',
-                 day_of_month='*', month_of_year='*', nowfun=None):
-        self._orig_minute = cronfield(minute)
-        self._orig_hour = cronfield(hour)
-        self._orig_day_of_week = cronfield(day_of_week)
-        self._orig_day_of_month = cronfield(day_of_month)
-        self._orig_month_of_year = cronfield(month_of_year)
-        self.hour = self._expand_cronspec(hour, 24)
-        self.minute = self._expand_cronspec(minute, 60)
-        self.day_of_week = self._expand_cronspec(day_of_week, 7)
-        self.day_of_month = self._expand_cronspec(day_of_month, 31, 1)
-        self.month_of_year = self._expand_cronspec(month_of_year, 12, 1)
-        self.nowfun = nowfun
+        return ffwd(year=datedata.year,
+                    month=months_of_year[datedata.moy],
+                    day=days_of_month[datedata.dom],
+                    hour=next_hour,
+                    minute=next_minute,
+                    second=0,
+                    microsecond=0)
 
     def now(self):
         return (self.nowfun or self.app.now)()
 
     def __repr__(self):
-        return '<crontab: %s %s %s %s %s (m/h/d/dM/MY)>' % (
-            self._orig_minute,
-            self._orig_hour,
-            self._orig_day_of_week,
-            self._orig_day_of_month,
-            self._orig_month_of_year,
-        )
+        return CRON_REPR.format(self)
 
     def __reduce__(self):
         return (self.__class__, (self._orig_minute,
@@ -455,8 +496,7 @@ class crontab(schedule):
                                  self._orig_day_of_month,
                                  self._orig_month_of_year), None)
 
-    def remaining_delta(self, last_run_at, tz=None):
-        """Returns when the periodic task should run next as a timedelta."""
+    def remaining_delta(self, last_run_at, tz=None, ffwd=ffwd):
         tz = tz or self.tz
         last_run_at = self.maybe_make_aware(last_run_at)
         now = self.maybe_make_aware(self.now())
@@ -476,9 +516,7 @@ class crontab(schedule):
         if execute_this_hour:
             next_minute = min(minute for minute in self.minute
                               if minute > last_run_at.minute)
-            delta = relativedelta(minute=next_minute,
-                                  second=0,
-                                  microsecond=0)
+            delta = ffwd(minute=next_minute, second=0, microsecond=0)
         else:
             next_minute = min(self.minute)
             execute_today = (execute_this_date and
@@ -487,10 +525,8 @@ class crontab(schedule):
             if execute_today:
                 next_hour = min(hour for hour in self.hour
                                 if hour > last_run_at.hour)
-                delta = relativedelta(hour=next_hour,
-                                      minute=next_minute,
-                                      second=0,
-                                      microsecond=0)
+                delta = ffwd(hour=next_hour, minute=next_minute,
+                             second=0, microsecond=0)
             else:
                 next_hour = min(self.hour)
                 all_dom_moy = (self._orig_day_of_month == '*' and
@@ -500,20 +536,20 @@ class crontab(schedule):
                                     if day > dow_num] or self.day_of_week)
                     add_week = next_day == dow_num
 
-                    delta = relativedelta(weeks=add_week and 1 or 0,
-                                          weekday=(next_day - 1) % 7,
-                                          hour=next_hour,
-                                          minute=next_minute,
-                                          second=0,
-                                          microsecond=0)
+                    delta = ffwd(weeks=add_week and 1 or 0,
+                                 weekday=(next_day - 1) % 7,
+                                 hour=next_hour,
+                                 minute=next_minute,
+                                 second=0,
+                                 microsecond=0)
                 else:
                     delta = self._delta_to_next(last_run_at,
                                                 next_hour, next_minute)
         return self.to_local(last_run_at), delta, self.to_local(now)
 
-    def remaining_estimate(self, last_run_at):
+    def remaining_estimate(self, last_run_at, ffwd=ffwd):
         """Returns when the periodic task should run next as a timedelta."""
-        return remaining(*self.remaining_delta(last_run_at))
+        return remaining(*self.remaining_delta(last_run_at, ffwd=ffwd))
 
     def is_due(self, last_run_at):
         """Returns tuple of two items `(is_due, next_time_to_run)`,
@@ -528,7 +564,7 @@ class crontab(schedule):
         if due:
             rem_delta = self.remaining_estimate(self.now())
             rem = timedelta_seconds(rem_delta)
-        return due, rem
+        return schedstate(due, rem)
 
     def __eq__(self, other):
         if isinstance(other, crontab):
@@ -537,12 +573,18 @@ class crontab(schedule):
                     other.day_of_week == self.day_of_week and
                     other.hour == self.hour and
                     other.minute == self.minute)
-        return other is self
+        return NotImplemented
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
 
-def maybe_schedule(s, relative=False):
-    if isinstance(s, int):
-        s = timedelta(seconds=s)
-    if isinstance(s, timedelta):
-        return schedule(s, relative)
+def maybe_schedule(s, relative=False, app=None):
+    if s is not None:
+        if isinstance(s, int):
+            s = timedelta(seconds=s)
+        if isinstance(s, timedelta):
+            return schedule(s, relative, app=app)
+        else:
+            s.app = app
     return s
