@@ -8,12 +8,19 @@
 
 """
 from __future__ import absolute_import
-from __future__ import with_statement
 
 from celery._state import get_current_worker_task
 from celery.app import app_or_default
-from celery.canvas import subtask, maybe_subtask  # noqa
-from celery.utils import uuid
+from celery.canvas import maybe_signature  # noqa
+from celery.utils import uuid, warn_deprecated
+
+from celery.canvas import subtask   # noqa
+
+warn_deprecated(
+    'celery.task.sets and TaskSet', removal='4.0',
+    alternative="""\
+Please use "group" instead (see the Canvas section in the userguide)\
+""")
 
 
 class TaskSet(list):
@@ -33,20 +40,21 @@ class TaskSet(list):
     app = None
 
     def __init__(self, tasks=None, app=None, Publisher=None):
-        super(TaskSet, self).__init__(maybe_subtask(t) for t in tasks or [])
         self.app = app_or_default(app or self.app)
+        super(TaskSet, self).__init__(
+            maybe_signature(t, app=self.app) for t in tasks or []
+        )
         self.Publisher = Publisher or self.app.amqp.TaskProducer
         self.total = len(self)  # XXX compat
 
-    def apply_async(self, connection=None, connect_timeout=None,
-                    publisher=None, taskset_id=None):
+    def apply_async(self, connection=None, publisher=None, taskset_id=None):
         """Apply TaskSet."""
         app = self.app
 
         if app.conf.CELERY_ALWAYS_EAGER:
             return self.apply(taskset_id=taskset_id)
 
-        with app.connection_or_acquire(connection, connect_timeout) as conn:
+        with app.connection_or_acquire(connection) as conn:
             setid = taskset_id or uuid()
             pub = publisher or self.Publisher(conn)
             results = self._async_results(setid, pub)
@@ -54,7 +62,7 @@ class TaskSet(list):
             result = app.TaskSetResult(setid, results)
             parent = get_current_worker_task()
             if parent:
-                parent.request.children.append(result)
+                parent.add_trail(result)
             return result
 
     def _async_results(self, taskset_id, publisher):
@@ -69,9 +77,10 @@ class TaskSet(list):
     def _sync_results(self, taskset_id):
         return [task.apply(taskset_id=taskset_id) for task in self]
 
-    def _get_tasks(self):
+    @property
+    def tasks(self):
         return self
 
-    def _set_tasks(self, tasks):
+    @tasks.setter  # noqa
+    def tasks(self, tasks):
         self[:] = tasks
-    tasks = property(_get_tasks, _set_tasks)
