@@ -1,180 +1,257 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import os
+import itertools
 
+from copy import deepcopy
 from mock import Mock, patch
 from pickle import loads, dumps
 
 from kombu import Exchange
 
-from celery import Celery
+from celery import shared_task, current_app
 from celery import app as _app
 from celery import _state
+from celery.app import base as _appbase
 from celery.app import defaults
+from celery.exceptions import ImproperlyConfigured
+from celery.five import items
 from celery.loaders.base import BaseLoader
 from celery.platforms import pyimplementation
 from celery.utils.serialization import pickle
 
-from celery.tests import config
-from celery.tests.utils import (Case, mask_modules, platform_pyimp,
-                                sys_platform, pypy_version)
+from celery.tests.case import (
+    CELERY_TEST_CONFIG,
+    AppCase,
+    depends_on_current_app,
+    mask_modules,
+    platform_pyimp,
+    sys_platform,
+    pypy_version,
+    with_environ,
+)
 from celery.utils import uuid
 from celery.utils.mail import ErrorMail
 
 THIS_IS_A_KEY = 'this is a value'
 
 
+class ObjectConfig(object):
+    FOO = 1
+    BAR = 2
+
+object_config = ObjectConfig()
+dict_config = dict(FOO=10, BAR=20)
+
+
 class Object(object):
 
     def __init__(self, **kwargs):
-        for key, value in kwargs.items():
+        for key, value in items(kwargs):
             setattr(self, key, value)
 
 
 def _get_test_config():
-    return dict((key, getattr(config, key))
-                for key in dir(config)
-                if key.isupper() and not key.startswith('_'))
-
+    return deepcopy(CELERY_TEST_CONFIG)
 test_config = _get_test_config()
 
 
-class test_module(Case):
+class test_module(AppCase):
 
     def test_default_app(self):
         self.assertEqual(_app.default_app, _state.default_app)
 
     def test_bugreport(self):
-        self.assertTrue(_app.bugreport())
+        self.assertTrue(_app.bugreport(app=self.app))
 
 
-class test_App(Case):
+class test_App(AppCase):
 
-    def setUp(self):
-        self.app = Celery(set_as_current=False)
-        self.app.conf.update(test_config)
+    def setup(self):
+        self.app.add_defaults(test_config)
 
     def test_task(self):
-        app = Celery('foozibari', set_as_current=False)
+        with self.Celery('foozibari') as app:
 
-        def fun():
-            pass
+            def fun():
+                pass
 
-        fun.__module__ = '__main__'
-        task = app.task(fun)
-        self.assertEqual(task.name, app.main + '.fun')
+            fun.__module__ = '__main__'
+            task = app.task(fun)
+            self.assertEqual(task.name, app.main + '.fun')
 
-    def test_with_broker(self):
-        prev = os.environ.get('CELERY_BROKER_URL')
-        os.environ.pop('CELERY_BROKER_URL', None)
+    def test_with_config_source(self):
+        with self.Celery(config_source=ObjectConfig) as app:
+            self.assertEqual(app.conf.FOO, 1)
+            self.assertEqual(app.conf.BAR, 2)
+
+    @depends_on_current_app
+    def test_task_windows_execv(self):
+        prev, _appbase._EXECV = _appbase._EXECV, True
         try:
-            app = Celery(set_as_current=False, broker='foo://baribaz')
-            self.assertEqual(app.conf.BROKER_HOST, 'foo://baribaz')
+
+            @self.app.task(shared=False)
+            def foo():
+                pass
+
+            self.assertTrue(foo._get_current_object())  # is proxy
+
         finally:
-            os.environ['CELERY_BROKER_URL'] = prev
+            _appbase._EXECV = prev
+        assert not _appbase._EXECV
+
+    def test_task_takes_no_args(self):
+        with self.assertRaises(TypeError):
+            @self.app.task(1)
+            def foo():
+                pass
+
+    def test_add_defaults(self):
+        self.assertFalse(self.app.configured)
+        _conf = {'FOO': 300}
+        conf = lambda: _conf
+        self.app.add_defaults(conf)
+        self.assertIn(conf, self.app._pending_defaults)
+        self.assertFalse(self.app.configured)
+        self.assertEqual(self.app.conf.FOO, 300)
+        self.assertTrue(self.app.configured)
+        self.assertFalse(self.app._pending_defaults)
+
+        # defaults not pickled
+        appr = loads(dumps(self.app))
+        with self.assertRaises(AttributeError):
+            appr.conf.FOO
+
+        # add more defaults after configured
+        conf2 = {'FOO': 'BAR'}
+        self.app.add_defaults(conf2)
+        self.assertEqual(self.app.conf.FOO, 'BAR')
+
+        self.assertIn(_conf, self.app.conf.defaults)
+        self.assertIn(conf2, self.app.conf.defaults)
+
+    def test_connection_or_acquire(self):
+        with self.app.connection_or_acquire(block=True):
+            self.assertTrue(self.app.pool._dirty)
+
+        with self.app.connection_or_acquire(pool=False):
+            self.assertFalse(self.app.pool._dirty)
+
+    def test_maybe_close_pool(self):
+        cpool = self.app._pool = Mock()
+        ppool = self.app.amqp._producer_pool = Mock()
+        self.app._maybe_close_pool()
+        cpool.force_close_all.assert_called_with()
+        ppool.force_close_all.assert_called_with()
+        self.assertIsNone(self.app._pool)
+        self.assertIsNone(self.app.amqp._producer_pool)
+
+        self.app._pool = Mock()
+        self.app._maybe_close_pool()
+        self.app._maybe_close_pool()
+
+    def test_using_v1_reduce(self):
+        self.app._using_v1_reduce = True
+        self.assertTrue(loads(dumps(self.app)))
+
+    def test_autodiscover_tasks(self):
+        self.app.conf.CELERY_FORCE_BILLIARD_LOGGING = True
+        with patch('celery.app.base.ensure_process_aware_logger') as ep:
+            self.app.loader.autodiscover_tasks = Mock()
+            self.app.autodiscover_tasks(['proj.A', 'proj.B'])
+            ep.assert_called_with()
+            self.app.loader.autodiscover_tasks.assert_called_with(
+                ['proj.A', 'proj.B'], 'tasks',
+            )
+        with patch('celery.app.base.ensure_process_aware_logger') as ep:
+            self.app.conf.CELERY_FORCE_BILLIARD_LOGGING = False
+            self.app.autodiscover_tasks(['proj.A', 'proj.B'])
+            self.assertFalse(ep.called)
+
+    @with_environ('CELERY_BROKER_URL', '')
+    def test_with_broker(self):
+        with self.Celery(broker='foo://baribaz') as app:
+            self.assertEqual(app.conf.BROKER_URL, 'foo://baribaz')
 
     def test_repr(self):
         self.assertTrue(repr(self.app))
 
     def test_custom_task_registry(self):
-        app1 = Celery(set_as_current=False)
-        app2 = Celery(set_as_current=False, tasks=app1.tasks)
-        self.assertIs(app2.tasks, app1.tasks)
+        with self.Celery(tasks=self.app.tasks) as app2:
+            self.assertIs(app2.tasks, self.app.tasks)
 
     def test_include_argument(self):
-        app = Celery(set_as_current=False, include=('foo', 'bar.foo'))
-        self.assertEqual(app.conf.CELERY_IMPORTS, ('foo', 'bar.foo'))
+        with self.Celery(include=('foo', 'bar.foo')) as app:
+            self.assertEqual(app.conf.CELERY_IMPORTS, ('foo', 'bar.foo'))
 
     def test_set_as_current(self):
         current = _state._tls.current_app
         try:
-            app = Celery(set_as_current=True)
+            app = self.Celery(set_as_current=True)
             self.assertIs(_state._tls.current_app, app)
         finally:
             _state._tls.current_app = current
 
     def test_current_task(self):
-        app = Celery(set_as_current=False)
-
-        @app.task
-        def foo():
+        @self.app.task
+        def foo(shared=False):
             pass
 
         _state._task_stack.push(foo)
         try:
-            self.assertEqual(app.current_task.name, foo.name)
+            self.assertEqual(self.app.current_task.name, foo.name)
         finally:
             _state._task_stack.pop()
 
     def test_task_not_shared(self):
-        with patch('celery.app.base.shared_task') as shared_task:
-            app = Celery(set_as_current=False)
-
-            @app.task(shared=False)
+        with patch('celery.app.base.shared_task') as sh:
+            @self.app.task(shared=False)
             def foo():
                 pass
-            self.assertFalse(shared_task.called)
+            self.assertFalse(sh.called)
 
     def test_task_compat_with_filter(self):
-        app = Celery(set_as_current=False, accept_magic_kwargs=True)
-        check = Mock()
+        with self.Celery(accept_magic_kwargs=True) as app:
+            check = Mock()
 
-        def filter(task):
-            check(task)
-            return task
+            def filter(task):
+                check(task)
+                return task
 
-        @app.task(filter=filter)
-        def foo():
-            pass
-        check.assert_called_with(foo)
+            @app.task(filter=filter, shared=False)
+            def foo():
+                pass
+            check.assert_called_with(foo)
 
     def test_task_with_filter(self):
-        app = Celery(set_as_current=False, accept_magic_kwargs=False)
-        check = Mock()
+        with self.Celery(accept_magic_kwargs=False) as app:
+            check = Mock()
 
-        def filter(task):
-            check(task)
-            return task
+            def filter(task):
+                check(task)
+                return task
 
-        @app.task(filter=filter)
-        def foo():
-            pass
-        check.assert_called_with(foo)
+            assert not _appbase._EXECV
+
+            @app.task(filter=filter, shared=False)
+            def foo():
+                pass
+            check.assert_called_with(foo)
 
     def test_task_sets_main_name_MP_MAIN_FILE(self):
         from celery import utils as _utils
         _utils.MP_MAIN_FILE = __file__
         try:
-            app = Celery('xuzzy', set_as_current=False)
+            with self.Celery('xuzzy') as app:
 
-            @app.task
-            def foo():
-                pass
+                @app.task
+                def foo():
+                    pass
 
-            self.assertEqual(foo.name, 'xuzzy.foo')
+                self.assertEqual(foo.name, 'xuzzy.foo')
         finally:
             _utils.MP_MAIN_FILE = None
 
-    def test_base_task_inherits_magic_kwargs_from_app(self):
-        from celery.task import Task as OldTask
-
-        class timkX(OldTask):
-            abstract = True
-
-        app = Celery(set_as_current=False, accept_magic_kwargs=True)
-        timkX.bind(app)
-        # see #918
-        self.assertFalse(timkX.accept_magic_kwargs)
-
-        from celery import Task as NewTask
-
-        class timkY(NewTask):
-            abstract = True
-
-        timkY.bind(app)
-        self.assertFalse(timkY.accept_magic_kwargs)
-
     def test_annotate_decorator(self):
         from celery.app.task import Task
 
@@ -193,12 +270,11 @@ class test_App(Case):
                 return fun(*args, **kwargs)
             return _inner
 
-        app = Celery(set_as_current=False)
-        app.conf.CELERY_ANNOTATIONS = {
+        self.app.conf.CELERY_ANNOTATIONS = {
             adX.name: {'@__call__': deco}
         }
-        adX.bind(app)
-        self.assertIs(adX.app, app)
+        adX.bind(self.app)
+        self.assertIs(adX.app, self.app)
 
         i = adX()
         i(2, 4, x=3)
@@ -208,9 +284,7 @@ class test_App(Case):
         i.annotate()
 
     def test_apply_async_has__self__(self):
-        app = Celery(set_as_current=False)
-
-        @app.task(__self__='hello')
+        @self.app.task(__self__='hello', shared=False)
         def aawsX():
             pass
 
@@ -219,26 +293,14 @@ class test_App(Case):
             args = dt.call_args[0][1]
             self.assertEqual(args, ('hello', 4, 5))
 
-    def test_apply_async__connection_arg(self):
-        app = Celery(set_as_current=False)
-
-        @app.task()
-        def aacaX():
-            pass
-
-        connection = app.connection('asd://')
-        with self.assertRaises(KeyError):
-            aacaX.apply_async(connection=connection)
-
     def test_apply_async_adds_children(self):
         from celery._state import _task_stack
-        app = Celery(set_as_current=False)
 
-        @app.task()
+        @self.app.task(shared=False)
         def a3cX1(self):
             pass
 
-        @app.task()
+        @self.app.task(shared=False)
         def a3cX2(self):
             pass
 
@@ -253,11 +315,6 @@ class test_App(Case):
         finally:
             _task_stack.pop()
 
-    def test_TaskSet(self):
-        ts = self.app.TaskSet()
-        self.assertListEqual(ts.tasks, [])
-        self.assertIs(ts.app, self.app)
-
     def test_pickle_app(self):
         changes = dict(THE_FOO_BAR='bars',
                        THE_MII_MAR='jars')
@@ -268,19 +325,19 @@ class test_App(Case):
         self.assertDictContainsSubset(changes, restored.conf)
 
     def test_worker_main(self):
-        from celery.bin import celeryd
+        from celery.bin import worker as worker_bin
 
-        class WorkerCommand(celeryd.WorkerCommand):
+        class worker(worker_bin.worker):
 
             def execute_from_commandline(self, argv):
                 return argv
 
-        prev, celeryd.WorkerCommand = celeryd.WorkerCommand, WorkerCommand
+        prev, worker_bin.worker = worker_bin.worker, worker
         try:
             ret = self.app.worker_main(argv=['--version'])
             self.assertListEqual(ret, ['--version'])
         finally:
-            celeryd.WorkerCommand = prev
+            worker_bin.worker = prev
 
     def test_config_from_envvar(self):
         os.environ['CELERYTEST_CONFIG_OBJECT'] = 'celery.tests.app.test_app'
@@ -307,7 +364,6 @@ class test_App(Case):
     def test_config_from_cmdline(self):
         cmdline = ['.always_eager=no',
                    '.result_backend=/dev/null',
-                   '.task_error_whitelist=(list)["a", "b", "c"]',
                    'celeryd.prefetch_multiplier=368',
                    '.foobarstring=(string)300',
                    '.foobarint=(int)300',
@@ -316,8 +372,6 @@ class test_App(Case):
         self.assertFalse(self.app.conf.CELERY_ALWAYS_EAGER)
         self.assertEqual(self.app.conf.CELERY_RESULT_BACKEND, '/dev/null')
         self.assertEqual(self.app.conf.CELERYD_PREFETCH_MULTIPLIER, 368)
-        self.assertListEqual(self.app.conf.CELERY_TASK_ERROR_WHITELIST,
-                             ['a', 'b', 'c'])
         self.assertEqual(self.app.conf.CELERY_FOOBARSTRING, '300')
         self.assertEqual(self.app.conf.CELERY_FOOBARINT, 300)
         self.assertDictEqual(self.app.conf.CELERY_RESULT_ENGINE_OPTIONS,
@@ -354,12 +408,46 @@ class test_App(Case):
         x = self.app.Worker
         self.assertIs(x.app, self.app)
 
+    @depends_on_current_app
     def test_AsyncResult(self):
         x = self.app.AsyncResult('1')
         self.assertIs(x.app, self.app)
         r = loads(dumps(x))
         # not set as current, so ends up as default app after reduce
-        self.assertIs(r.app, _state.default_app)
+        self.assertIs(r.app, current_app._get_current_object())
+
+    def test_get_active_apps(self):
+        self.assertTrue(list(_state._get_active_apps()))
+
+        app1 = self.Celery()
+        appid = id(app1)
+        self.assertIn(app1, _state._get_active_apps())
+        app1.close()
+        del(app1)
+
+        # weakref removed from list when app goes out of scope.
+        with self.assertRaises(StopIteration):
+            next(app for app in _state._get_active_apps() if id(app) == appid)
+
+    def test_config_from_envvar_more(self, key='CELERY_HARNESS_CFG1'):
+        self.assertFalse(self.app.config_from_envvar('HDSAJIHWIQHEWQU',
+                                                     silent=True))
+        with self.assertRaises(ImproperlyConfigured):
+            self.app.config_from_envvar('HDSAJIHWIQHEWQU', silent=False)
+        os.environ[key] = __name__ + '.object_config'
+        self.assertTrue(self.app.config_from_envvar(key))
+        self.assertEqual(self.app.conf['FOO'], 1)
+        self.assertEqual(self.app.conf['BAR'], 2)
+
+        os.environ[key] = 'unknown_asdwqe.asdwqewqe'
+        with self.assertRaises(ImportError):
+            self.app.config_from_envvar(key, silent=False)
+        self.assertFalse(self.app.config_from_envvar(key, silent=True))
+
+        os.environ[key] = __name__ + '.dict_config'
+        self.assertTrue(self.app.config_from_envvar(key))
+        self.assertEqual(self.app.conf['FOO'], 10)
+        self.assertEqual(self.app.conf['BAR'], 20)
 
     @patch('celery.bin.celery.CeleryCommand.execute_from_commandline')
     def test_start(self, execute):
@@ -373,7 +461,7 @@ class test_App(Case):
             def mail_admins(*args, **kwargs):
                 return args, kwargs
 
-        self.app.loader = Loader()
+        self.app.loader = Loader(app=self.app)
         self.app.conf.ADMINS = None
         self.assertFalse(self.app.mail_admins('Subject', 'Body'))
         self.app.conf.ADMINS = [('George Costanza', 'george@vandelay.com')]
@@ -385,32 +473,48 @@ class test_App(Case):
              'userid': 'guest',
              'password': 'guest',
              'virtual_host': '/'},
-            self.app.connection('amqp://').info(),
+            self.app.connection('pyamqp://').info(),
         )
         self.app.conf.BROKER_PORT = 1978
         self.app.conf.BROKER_VHOST = 'foo'
         self.assertDictContainsSubset(
             {'port': 1978, 'virtual_host': 'foo'},
-            self.app.connection('amqp://:1978/foo').info(),
+            self.app.connection('pyamqp://:1978/foo').info(),
         )
-        conn = self.app.connection('amqp:////value')
+        conn = self.app.connection('pyamqp:////value')
         self.assertDictContainsSubset({'virtual_host': '/value'},
                                       conn.info())
 
+    def test_amqp_failover_strategy_selection(self):
+        # Test passing in a string and make sure the string
+        # gets there untouched
+        self.app.conf.BROKER_FAILOVER_STRATEGY = 'foo-bar'
+        self.assertEquals(
+            self.app.connection('amqp:////value').failover_strategy,
+            'foo-bar',
+        )
+
+        # Try passing in None
+        self.app.conf.BROKER_FAILOVER_STRATEGY = None
+        self.assertEquals(
+            self.app.connection('amqp:////value').failover_strategy,
+            itertools.cycle,
+        )
+
+        # Test passing in a method
+        def my_failover_strategy(it):
+            yield True
+
+        self.app.conf.BROKER_FAILOVER_STRATEGY = my_failover_strategy
+        self.assertEquals(
+            self.app.connection('amqp:////value').failover_strategy,
+            my_failover_strategy,
+        )
+
     def test_BROKER_BACKEND_alias(self):
         self.assertEqual(self.app.conf.BROKER_BACKEND,
                          self.app.conf.BROKER_TRANSPORT)
 
-    def test_with_default_connection(self):
-
-        @self.app.with_default_connection
-        def handler(connection=None, foo=None):
-            return connection, foo
-
-        connection, foo = handler(foo=42)
-        self.assertEqual(foo, 42)
-        self.assertTrue(connection)
-
     def test_after_fork(self):
         p = self.app._pool = Mock()
         self.app._after_fork(self.app)
@@ -469,8 +573,16 @@ class test_App(Case):
                                  'hostname': 'lana'}
         self.assertTrue(x)
 
+    def test_error_mail_disabled(self):
+        task = Mock()
+        x = ErrorMail(task)
+        x.should_send = Mock()
+        x.should_send.return_value = False
+        x.send(Mock(), Mock())
+        self.assertFalse(task.app.mail_admins.called)
 
-class test_defaults(Case):
+
+class test_defaults(AppCase):
 
     def test_str_to_bool(self):
         for s in ('false', 'no', '0'):
@@ -481,7 +593,7 @@ class test_defaults(Case):
             defaults.strtobool('unsure')
 
 
-class test_debugging_utils(Case):
+class test_debugging_utils(AppCase):
 
     def test_enable_disable_trace(self):
         try:
@@ -493,7 +605,7 @@ class test_debugging_utils(Case):
             _app.disable_trace()
 
 
-class test_pyimplementation(Case):
+class test_pyimplementation(AppCase):
 
     def test_platform_python_implementation(self):
         with platform_pyimp(lambda: 'Xython'):
@@ -517,3 +629,32 @@ class test_pyimplementation(Case):
             with sys_platform('darwin'):
                 with pypy_version():
                     self.assertEqual('CPython', pyimplementation())
+
+
+class test_shared_task(AppCase):
+
+    def test_registers_to_all_apps(self):
+        with self.Celery('xproj', set_as_current=True) as xproj:
+            xproj.finalize()
+
+            @shared_task
+            def foo():
+                return 42
+
+            @shared_task()
+            def bar():
+                return 84
+
+            self.assertIs(foo.app, xproj)
+            self.assertIs(bar.app, xproj)
+            self.assertTrue(foo._get_current_object())
+
+            with self.Celery('yproj', set_as_current=True) as yproj:
+                self.assertIs(foo.app, yproj)
+                self.assertIs(bar.app, yproj)
+
+                @shared_task()
+                def baz():
+                    return 168
+
+                self.assertIs(baz.app, yproj)
