@@ -1,20 +1,17 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import errno
 
 from datetime import datetime, timedelta
 from mock import Mock, call, patch
 from nose import SkipTest
+from pickle import dumps, loads
 
-from celery import current_app
 from celery import beat
-from celery import task
-from celery.result import AsyncResult
+from celery.five import keys, string_t
 from celery.schedules import schedule
-from celery.task.base import Task
 from celery.utils import uuid
-from celery.tests.utils import Case, patch_settings
+from celery.tests.case import AppCase
 
 
 class Object(object):
@@ -46,14 +43,17 @@ class MockService(object):
         self.stopped = True
 
 
-class test_ScheduleEntry(Case):
+class test_ScheduleEntry(AppCase):
     Entry = beat.ScheduleEntry
 
     def create_entry(self, **kwargs):
-        entry = dict(name='celery.unittest.add',
-                     schedule=schedule(timedelta(seconds=10)),
-                     args=(2, 2),
-                     options={'routing_key': 'cpu'})
+        entry = dict(
+            name='celery.unittest.add',
+            schedule=timedelta(seconds=10),
+            args=(2, 2),
+            options={'routing_key': 'cpu'},
+            app=self.app,
+        )
         return self.Entry(**dict(entry, **kwargs))
 
     def test_next(self):
@@ -63,19 +63,21 @@ class test_ScheduleEntry(Case):
         self.assertEqual(entry.total_run_count, 0)
 
         next_run_at = entry.last_run_at + timedelta(seconds=10)
-        next = entry.next(next_run_at)
-        self.assertGreaterEqual(next.last_run_at, next_run_at)
-        self.assertEqual(next.total_run_count, 1)
+        next_entry = entry.next(next_run_at)
+        self.assertGreaterEqual(next_entry.last_run_at, next_run_at)
+        self.assertEqual(next_entry.total_run_count, 1)
 
     def test_is_due(self):
         entry = self.create_entry(schedule=timedelta(seconds=10))
+        self.assertIs(entry.app, self.app)
+        self.assertIs(entry.schedule.app, self.app)
         due1, next_time_to_run1 = entry.is_due()
         self.assertFalse(due1)
         self.assertGreater(next_time_to_run1, 9)
 
         next_run_at = entry.last_run_at - timedelta(seconds=10)
-        next = entry.next(next_run_at)
-        due2, next_time_to_run2 = next.is_due()
+        next_entry = entry.next(next_run_at)
+        due2, next_time_to_run2 = next_entry.is_due()
         self.assertTrue(due2)
         self.assertGreater(next_time_to_run2, 9)
 
@@ -112,7 +114,7 @@ class mScheduler(beat.Scheduler):
                           'args': args,
                           'kwargs': kwargs,
                           'options': options})
-        return AsyncResult(uuid())
+        return self.app.AsyncResult(uuid())
 
 
 class mSchedulerSchedulingError(mScheduler):
@@ -143,73 +145,70 @@ always_due = mocked_schedule(True, 1)
 always_pending = mocked_schedule(False, 1)
 
 
-class test_Scheduler(Case):
+class test_Scheduler(AppCase):
 
     def test_custom_schedule_dict(self):
         custom = {'foo': 'bar'}
-        scheduler = mScheduler(schedule=custom, lazy=True)
+        scheduler = mScheduler(app=self.app, schedule=custom, lazy=True)
         self.assertIs(scheduler.data, custom)
 
     def test_apply_async_uses_registered_task_instances(self):
-        through_task = [False]
 
-        class MockTask(Task):
+        @self.app.task(shared=False)
+        def foo():
+            pass
+        foo.apply_async = Mock(name='foo.apply_async')
+        assert foo.name in foo._get_app().tasks
 
-            @classmethod
-            def apply_async(cls, *args, **kwargs):
-                through_task[0] = True
-
-        assert MockTask.name in MockTask._get_app().tasks
-
-        scheduler = mScheduler()
-        scheduler.apply_async(scheduler.Entry(task=MockTask.name))
-        self.assertTrue(through_task[0])
+        scheduler = mScheduler(app=self.app)
+        scheduler.apply_async(scheduler.Entry(task=foo.name, app=self.app))
+        self.assertTrue(foo.apply_async.called)
 
     def test_apply_async_should_not_sync(self):
 
-        @task()
+        @self.app.task(shared=False)
         def not_sync():
             pass
         not_sync.apply_async = Mock()
 
-        s = mScheduler()
+        s = mScheduler(app=self.app)
         s._do_sync = Mock()
         s.should_sync = Mock()
         s.should_sync.return_value = True
-        s.apply_async(s.Entry(task=not_sync.name))
+        s.apply_async(s.Entry(task=not_sync.name, app=self.app))
         s._do_sync.assert_called_with()
 
         s._do_sync = Mock()
         s.should_sync.return_value = False
-        s.apply_async(s.Entry(task=not_sync.name))
+        s.apply_async(s.Entry(task=not_sync.name, app=self.app))
         self.assertFalse(s._do_sync.called)
 
     @patch('celery.app.base.Celery.send_task')
     def test_send_task(self, send_task):
-        b = beat.Scheduler()
+        b = beat.Scheduler(app=self.app)
         b.send_task('tasks.add', countdown=10)
         send_task.assert_called_with('tasks.add', countdown=10)
 
     def test_info(self):
-        scheduler = mScheduler()
-        self.assertIsInstance(scheduler.info, basestring)
+        scheduler = mScheduler(app=self.app)
+        self.assertIsInstance(scheduler.info, string_t)
 
     def test_maybe_entry(self):
-        s = mScheduler()
-        entry = s.Entry(name='add every', task='tasks.add')
+        s = mScheduler(app=self.app)
+        entry = s.Entry(name='add every', task='tasks.add', app=self.app)
         self.assertIs(s._maybe_entry(entry.name, entry), entry)
         self.assertTrue(s._maybe_entry('add every', {
             'task': 'tasks.add',
         }))
 
     def test_set_schedule(self):
-        s = mScheduler()
+        s = mScheduler(app=self.app)
         s.schedule = {'foo': 'bar'}
         self.assertEqual(s.data, {'foo': 'bar'})
 
     @patch('kombu.connection.Connection.ensure_connection')
     def test_ensure_connection_error_handler(self, ensure):
-        s = mScheduler()
+        s = mScheduler(app=self.app)
         self.assertTrue(s._ensure_connected())
         self.assertTrue(ensure.called)
         callback = ensure.call_args[0][0]
@@ -217,29 +216,26 @@ class test_Scheduler(Case):
         callback(KeyError(), 5)
 
     def test_install_default_entries(self):
-        with patch_settings(CELERY_TASK_RESULT_EXPIRES=None,
-                            CELERYBEAT_SCHEDULE={}):
-            s = mScheduler()
-            s.install_default_entries({})
-            self.assertNotIn('celery.backend_cleanup', s.data)
-        current_app.backend.supports_autoexpire = False
-        with patch_settings(CELERY_TASK_RESULT_EXPIRES=30,
-                            CELERYBEAT_SCHEDULE={}):
-            s = mScheduler()
-            s.install_default_entries({})
-            self.assertIn('celery.backend_cleanup', s.data)
-        current_app.backend.supports_autoexpire = True
-        try:
-            with patch_settings(CELERY_TASK_RESULT_EXPIRES=31,
-                                CELERYBEAT_SCHEDULE={}):
-                s = mScheduler()
-                s.install_default_entries({})
-                self.assertNotIn('celery.backend_cleanup', s.data)
-        finally:
-            current_app.backend.supports_autoexpire = False
+        self.app.conf.CELERY_TASK_RESULT_EXPIRES = None
+        self.app.conf.CELERYBEAT_SCHEDULE = {}
+        s = mScheduler(app=self.app)
+        s.install_default_entries({})
+        self.assertNotIn('celery.backend_cleanup', s.data)
+        self.app.backend.supports_autoexpire = False
+
+        self.app.conf.CELERY_TASK_RESULT_EXPIRES = 30
+        s = mScheduler(app=self.app)
+        s.install_default_entries({})
+        self.assertIn('celery.backend_cleanup', s.data)
+
+        self.app.backend.supports_autoexpire = True
+        self.app.conf.CELERY_TASK_RESULT_EXPIRES = 31
+        s = mScheduler(app=self.app)
+        s.install_default_entries({})
+        self.assertNotIn('celery.backend_cleanup', s.data)
 
     def test_due_tick(self):
-        scheduler = mScheduler()
+        scheduler = mScheduler(app=self.app)
         scheduler.add(name='test_due_tick',
                       schedule=always_due,
                       args=(1, 2),
@@ -248,33 +244,33 @@ class test_Scheduler(Case):
 
     @patch('celery.beat.error')
     def test_due_tick_SchedulingError(self, error):
-        scheduler = mSchedulerSchedulingError()
+        scheduler = mSchedulerSchedulingError(app=self.app)
         scheduler.add(name='test_due_tick_SchedulingError',
                       schedule=always_due)
         self.assertEqual(scheduler.tick(), 1)
         self.assertTrue(error.called)
 
     def test_due_tick_RuntimeError(self):
-        scheduler = mSchedulerRuntimeError()
+        scheduler = mSchedulerRuntimeError(app=self.app)
         scheduler.add(name='test_due_tick_RuntimeError',
                       schedule=always_due)
         self.assertEqual(scheduler.tick(), scheduler.max_interval)
 
     def test_pending_tick(self):
-        scheduler = mScheduler()
+        scheduler = mScheduler(app=self.app)
         scheduler.add(name='test_pending_tick',
                       schedule=always_pending)
         self.assertEqual(scheduler.tick(), 1)
 
     def test_honors_max_interval(self):
-        scheduler = mScheduler()
+        scheduler = mScheduler(app=self.app)
         maxi = scheduler.max_interval
         scheduler.add(name='test_honors_max_interval',
                       schedule=mocked_schedule(False, maxi * 4))
         self.assertEqual(scheduler.tick(), maxi)
 
     def test_ticks(self):
-        scheduler = mScheduler()
+        scheduler = mScheduler(app=self.app)
         nums = [600, 300, 650, 120, 250, 36]
         s = dict(('test_ticks%s' % i,
                  {'schedule': mocked_schedule(False, j)})
@@ -283,20 +279,20 @@ class test_Scheduler(Case):
         self.assertEqual(scheduler.tick(), min(nums))
 
     def test_schedule_no_remain(self):
-        scheduler = mScheduler()
+        scheduler = mScheduler(app=self.app)
         scheduler.add(name='test_schedule_no_remain',
                       schedule=mocked_schedule(False, None))
         self.assertEqual(scheduler.tick(), scheduler.max_interval)
 
     def test_interface(self):
-        scheduler = mScheduler()
+        scheduler = mScheduler(app=self.app)
         scheduler.sync()
         scheduler.setup_schedule()
         scheduler.close()
 
     def test_merge_inplace(self):
-        a = mScheduler()
-        b = mScheduler()
+        a = mScheduler(app=self.app)
+        b = mScheduler(app=self.app)
         a.update_from_dict({'foo': {'schedule': mocked_schedule(True, 10)},
                             'bar': {'schedule': mocked_schedule(True, 20)}})
         b.update_from_dict({'bar': {'schedule': mocked_schedule(True, 40)},
@@ -329,11 +325,12 @@ def create_persistent_scheduler(shelv=None):
     return MockPersistentScheduler, shelv
 
 
-class test_PersistentScheduler(Case):
+class test_PersistentScheduler(AppCase):
 
     @patch('os.remove')
     def test_remove_db(self, remove):
-        s = create_persistent_scheduler()[0](schedule_filename='schedule')
+        s = create_persistent_scheduler()[0](app=self.app,
+                                             schedule_filename='schedule')
         s._remove_db()
         remove.assert_has_calls(
             [call('schedule' + suffix) for suffix in s.known_suffixes]
@@ -347,7 +344,8 @@ class test_PersistentScheduler(Case):
             s._remove_db()
 
     def test_setup_schedule(self):
-        s = create_persistent_scheduler()[0](schedule_filename='schedule')
+        s = create_persistent_scheduler()[0](app=self.app,
+                                             schedule_filename='schedule')
         opens = s.persistence.open = Mock()
         s._remove_db = Mock()
 
@@ -362,27 +360,45 @@ class test_PersistentScheduler(Case):
         s._store = {'__version__': 1}
         s.setup_schedule()
 
+        s._store.clear = Mock()
+        op = s.persistence.open = Mock()
+        op.return_value = s._store
+        s._store['tz'] = 'FUNKY'
+        s.setup_schedule()
+        op.assert_called_with(s.schedule_filename, writeback=True)
+        s._store.clear.assert_called_with()
+        s._store['utc_enabled'] = False
+        s._store.clear = Mock()
+        s.setup_schedule()
+        s._store.clear.assert_called_with()
+
     def test_get_schedule(self):
-        s = create_persistent_scheduler()[0](schedule_filename='schedule')
+        s = create_persistent_scheduler()[0](
+            schedule_filename='schedule', app=self.app,
+        )
         s._store = {'entries': {}}
         s.schedule = {'foo': 'bar'}
         self.assertDictEqual(s.schedule, {'foo': 'bar'})
         self.assertDictEqual(s._store['entries'], s.schedule)
 
 
-class test_Service(Case):
+class test_Service(AppCase):
 
     def get_service(self):
         Scheduler, mock_shelve = create_persistent_scheduler()
-        return beat.Service(scheduler_cls=Scheduler), mock_shelve
+        return beat.Service(app=self.app, scheduler_cls=Scheduler), mock_shelve
+
+    def test_pickleable(self):
+        s = beat.Service(app=self.app, scheduler_cls=Mock)
+        self.assertTrue(loads(dumps(s)))
 
     def test_start(self):
         s, sh = self.get_service()
         schedule = s.scheduler.schedule
         self.assertIsInstance(schedule, dict)
         self.assertIsInstance(s.scheduler, beat.Scheduler)
-        scheduled = schedule.keys()
-        for task_name in sh['entries'].keys():
+        scheduled = list(schedule.keys())
+        for task_name in keys(sh['entries']):
             self.assertIn(task_name, scheduled)
 
         s.sync()
@@ -425,7 +441,7 @@ class test_Service(Case):
         self.assertTrue(s._is_shutdown.isSet())
 
 
-class test_EmbeddedService(Case):
+class test_EmbeddedService(AppCase):
 
     def test_start_stop_process(self):
         try:
@@ -435,7 +451,7 @@ class test_EmbeddedService(Case):
 
         from billiard.process import Process
 
-        s = beat.EmbeddedService()
+        s = beat.EmbeddedService(app=self.app)
         self.assertIsInstance(s, Process)
         self.assertIsInstance(s.service, beat.Service)
         s.service = MockService()
@@ -455,7 +471,7 @@ class test_EmbeddedService(Case):
         self.assertTrue(s._popen.terminated)
 
     def test_start_stop_threaded(self):
-        s = beat.EmbeddedService(thread=True)
+        s = beat.EmbeddedService(thread=True, app=self.app)
         from threading import Thread
         self.assertIsInstance(s, Thread)
         self.assertIsInstance(s.service, beat.Service)
@@ -466,3 +482,24 @@ class test_EmbeddedService(Case):
 
         s.stop()
         self.assertTrue(s.service.stopped)
+
+
+class test_schedule(AppCase):
+
+    def test_maybe_make_aware(self):
+        x = schedule(10, app=self.app)
+        x.utc_enabled = True
+        d = x.maybe_make_aware(datetime.utcnow())
+        self.assertTrue(d.tzinfo)
+        x.utc_enabled = False
+        d2 = x.maybe_make_aware(datetime.utcnow())
+        self.assertIsNone(d2.tzinfo)
+
+    def test_to_local(self):
+        x = schedule(10, app=self.app)
+        x.utc_enabled = True
+        d = x.to_local(datetime.utcnow())
+        self.assertIsNone(d.tzinfo)
+        x.utc_enabled = False
+        d = x.to_local(datetime.utcnow())
+        self.assertTrue(d.tzinfo)
