@@ -1,16 +1,14 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import os
 import sys
+import warnings
 
 from mock import Mock, patch
 
 from celery import loaders
-from celery.app import app_or_default
 from celery.exceptions import (
     NotConfigured,
-    ImproperlyConfigured,
     CPendingDeprecationWarning,
 )
 from celery.loaders import base
@@ -19,23 +17,9 @@ from celery.loaders.app import AppLoader
 from celery.utils.imports import NotAPackage
 from celery.utils.mail import SendmailWarning
 
-from celery.tests.utils import AppCase, Case
-from celery.tests.compat import catch_warnings
-
-
-class ObjectConfig(object):
-    FOO = 1
-    BAR = 2
-
-object_config = ObjectConfig()
-dict_config = dict(FOO=10, BAR=20)
-
-
-class Object(object):
-
-    def __init__(self, **kwargs):
-        for k, v in kwargs.items():
-            setattr(self, k, v)
+from celery.tests.case import (
+    AppCase, Case, depends_on_current_app, with_environ,
+)
 
 
 class DummyLoader(base.BaseLoader):
@@ -47,16 +31,17 @@ class DummyLoader(base.BaseLoader):
 class test_loaders(AppCase):
 
     def test_get_loader_cls(self):
-
         self.assertEqual(loaders.get_loader_cls('default'),
                          default.Loader)
 
+    @depends_on_current_app
     def test_current_loader(self):
         with self.assertWarnsRegex(
                 CPendingDeprecationWarning,
                 r'deprecation'):
             self.assertIs(loaders.current_loader(), self.app.loader)
 
+    @depends_on_current_app
     def test_load_settings(self):
         with self.assertWarnsRegex(
                 CPendingDeprecationWarning,
@@ -64,7 +49,7 @@ class test_loaders(AppCase):
             self.assertIs(loaders.load_settings(), self.app.conf)
 
 
-class test_LoaderBase(Case):
+class test_LoaderBase(AppCase):
     message_options = {'subject': 'Subject',
                        'body': 'Body',
                        'sender': 'x@x.com',
@@ -75,14 +60,32 @@ class test_LoaderBase(Case):
                       'password': 'qwerty',
                       'timeout': 3}
 
-    def setUp(self):
-        self.loader = DummyLoader()
-        self.app = app_or_default()
+    def setup(self):
+        self.loader = DummyLoader(app=self.app)
 
     def test_handlers_pass(self):
         self.loader.on_task_init('foo.task', 'feedface-cafebabe')
         self.loader.on_worker_init()
 
+    def test_now(self):
+        self.assertTrue(self.loader.now(utc=True))
+        self.assertTrue(self.loader.now(utc=False))
+
+    def test_read_configuration_no_env(self):
+        self.assertDictEqual(
+            base.BaseLoader(app=self.app).read_configuration(
+                'FOO_X_S_WE_WQ_Q_WE'),
+            {},
+        )
+
+    def test_autodiscovery(self):
+        with patch('celery.loaders.base.autodiscover_tasks') as auto:
+            auto.return_value = [Mock()]
+            auto.return_value[0].__name__ = 'moo'
+            self.loader.autodiscover_tasks(['A', 'B'])
+            self.assertIn('moo', self.loader.task_modules)
+            self.loader.task_modules.discard('moo')
+
     def test_import_task_module(self):
         self.assertEqual(sys, self.loader.import_task_module('sys'))
 
@@ -104,15 +107,11 @@ class test_LoaderBase(Case):
 
     def test_import_default_modules(self):
         modnames = lambda l: [m.__name__ for m in l]
-        prev, self.app.conf.CELERY_IMPORTS = (
-            self.app.conf.CELERY_IMPORTS, ('os', 'sys'))
-        try:
-            self.assertEqual(
-                sorted(modnames(self.loader.import_default_modules())),
-                sorted(modnames([os, sys])),
-            )
-        finally:
-            self.app.conf.CELERY_IMPORTS = prev
+        self.app.conf.CELERY_IMPORTS = ('os', 'sys')
+        self.assertEqual(
+            sorted(modnames(self.loader.import_default_modules())),
+            sorted(modnames([os, sys])),
+        )
 
     def test_import_from_cwd_custom_imp(self):
 
@@ -147,7 +146,7 @@ class test_LoaderBase(Case):
 
     def test_mail_attribute(self):
         from celery.utils import mail
-        loader = base.BaseLoader()
+        loader = base.BaseLoader(app=self.app)
         self.assertIs(loader.mail, mail)
 
     def test_cmdline_config_ValueError(self):
@@ -155,34 +154,32 @@ class test_LoaderBase(Case):
             self.loader.cmdline_config_parser(['broker.port=foobar'])
 
 
-class test_DefaultLoader(Case):
+class test_DefaultLoader(AppCase):
 
     @patch('celery.loaders.base.find_module')
     def test_read_configuration_not_a_package(self, find_module):
         find_module.side_effect = NotAPackage()
-        l = default.Loader()
+        l = default.Loader(app=self.app)
         with self.assertRaises(NotAPackage):
-            l.read_configuration()
+            l.read_configuration(fail_silently=False)
 
     @patch('celery.loaders.base.find_module')
+    @with_environ('CELERY_CONFIG_MODULE', 'celeryconfig.py')
     def test_read_configuration_py_in_name(self, find_module):
-        prev = os.environ['CELERY_CONFIG_MODULE']
-        os.environ['CELERY_CONFIG_MODULE'] = 'celeryconfig.py'
-        try:
-            find_module.side_effect = NotAPackage()
-            l = default.Loader()
-            with self.assertRaises(NotAPackage):
-                l.read_configuration()
-        finally:
-            os.environ['CELERY_CONFIG_MODULE'] = prev
+        find_module.side_effect = NotAPackage()
+        l = default.Loader(app=self.app)
+        with self.assertRaises(NotAPackage):
+            l.read_configuration(fail_silently=False)
 
     @patch('celery.loaders.base.find_module')
     def test_read_configuration_importerror(self, find_module):
         default.C_WNOCONF = True
         find_module.side_effect = ImportError()
-        l = default.Loader()
+        l = default.Loader(app=self.app)
         with self.assertWarnsRegex(NotConfigured, r'make sure it exists'):
-            l.read_configuration()
+            l.read_configuration(fail_silently=True)
+        default.C_WNOCONF = False
+        l.read_configuration(fail_silently=True)
 
     def test_read_configuration(self):
         from types import ModuleType
@@ -190,17 +187,18 @@ class test_DefaultLoader(Case):
         class ConfigModule(ModuleType):
             pass
 
-        celeryconfig = ConfigModule('celeryconfig')
-        celeryconfig.CELERY_IMPORTS = ('os', 'sys')
         configname = os.environ.get('CELERY_CONFIG_MODULE') or 'celeryconfig'
+        celeryconfig = ConfigModule(configname)
+        celeryconfig.CELERY_IMPORTS = ('os', 'sys')
 
         prevconfig = sys.modules.get(configname)
         sys.modules[configname] = celeryconfig
         try:
-            l = default.Loader()
-            settings = l.read_configuration()
+            l = default.Loader(app=self.app)
+            l.find_module = Mock(name='find_module')
+            settings = l.read_configuration(fail_silently=False)
             self.assertTupleEqual(settings.CELERY_IMPORTS, ('os', 'sys'))
-            settings = l.read_configuration()
+            settings = l.read_configuration(fail_silently=False)
             self.assertTupleEqual(settings.CELERY_IMPORTS, ('os', 'sys'))
             l.on_worker_init()
         finally:
@@ -208,7 +206,7 @@ class test_DefaultLoader(Case):
                 sys.modules[configname] = prevconfig
 
     def test_import_from_cwd(self):
-        l = default.Loader()
+        l = default.Loader(app=self.app)
         old_path = list(sys.path)
         try:
             sys.path.remove(os.getcwd())
@@ -232,45 +230,47 @@ class test_DefaultLoader(Case):
             def find_module(self, name):
                 raise ImportError(name)
 
-        with catch_warnings(record=True):
-            l = _Loader()
+        with warnings.catch_warnings(record=True):
+            l = _Loader(app=self.app)
             self.assertFalse(l.configured)
             context_executed[0] = True
         self.assertTrue(context_executed[0])
 
 
-class test_AppLoader(Case):
+class test_AppLoader(AppCase):
 
-    def setUp(self):
-        self.app = app_or_default()
+    def setup(self):
         self.loader = AppLoader(app=self.app)
 
-    def test_config_from_envvar(self, key='CELERY_HARNESS_CFG1'):
-        self.assertFalse(self.loader.config_from_envvar('HDSAJIHWIQHEWQU',
-                                                        silent=True))
-        with self.assertRaises(ImproperlyConfigured):
-            self.loader.config_from_envvar('HDSAJIHWIQHEWQU', silent=False)
-        os.environ[key] = __name__ + '.object_config'
-        self.assertTrue(self.loader.config_from_envvar(key))
-        self.assertEqual(self.loader.conf['FOO'], 1)
-        self.assertEqual(self.loader.conf['BAR'], 2)
-
-        os.environ[key] = 'unknown_asdwqe.asdwqewqe'
-        with self.assertRaises(ImportError):
-            self.loader.config_from_envvar(key, silent=False)
-        self.assertFalse(self.loader.config_from_envvar(key, silent=True))
-
-        os.environ[key] = __name__ + '.dict_config'
-        self.assertTrue(self.loader.config_from_envvar(key))
-        self.assertEqual(self.loader.conf['FOO'], 10)
-        self.assertEqual(self.loader.conf['BAR'], 20)
-
     def test_on_worker_init(self):
-        prev, self.app.conf.CELERY_IMPORTS = (
-            self.app.conf.CELERY_IMPORTS, ('subprocess', ))
+        self.app.conf.CELERY_IMPORTS = ('subprocess', )
+        sys.modules.pop('subprocess', None)
+        self.loader.init_worker()
+        self.assertIn('subprocess', sys.modules)
+
+
+class test_autodiscovery(Case):
+
+    def test_autodiscover_tasks(self):
+        base._RACE_PROTECTION = True
         try:
-            sys.modules.pop('subprocess', None)
-            self.loader.init_worker()
-            self.assertIn('subprocess', sys.modules)
+            base.autodiscover_tasks(['foo'])
         finally:
-            self.app.conf.CELERY_IMPORTS = prev
+            base._RACE_PROTECTION = False
+        with patch('celery.loaders.base.find_related_module') as frm:
+            base.autodiscover_tasks(['foo'])
+            self.assertTrue(frm.called)
+
+    def test_find_related_module(self):
+        with patch('importlib.import_module') as imp:
+            with patch('imp.find_module') as find:
+                imp.return_value = Mock()
+                imp.return_value.__path__ = 'foo'
+                base.find_related_module(base, 'tasks')
+
+                imp.side_effect = AttributeError()
+                base.find_related_module(base, 'tasks')
+                imp.side_effect = None
+
+                find.side_effect = ImportError()
+                base.find_related_module(base, 'tasks')
