@@ -1,7 +1,4 @@
-from __future__ import absolute_import
-from __future__ import with_statement
-
-import sys
+from __future__ import absolute_import, unicode_literals
 
 from datetime import datetime
 
@@ -9,13 +6,12 @@ from nose import SkipTest
 from pickle import loads, dumps
 
 from celery import states
-from celery.app import app_or_default
 from celery.exceptions import ImproperlyConfigured
-from celery.result import AsyncResult
 from celery.utils import uuid
 
-from celery.tests.utils import (
-    Case,
+from celery.tests.case import (
+    AppCase,
+    depends_on_current_app,
     mask_modules,
     skip_if_pypy,
     skip_if_jython,
@@ -24,9 +20,9 @@ from celery.tests.utils import (
 try:
     import sqlalchemy  # noqa
 except ImportError:
-    DatabaseBackend = Task = TaskSet = None  # noqa
+    DatabaseBackend = Task = TaskSet = retry = None  # noqa
 else:
-    from celery.backends.database import DatabaseBackend
+    from celery.backends.database import DatabaseBackend, retry
     from celery.backends.database.models import Task, TaskSet
 
 
@@ -36,13 +32,28 @@ class SomeClass(object):
         self.data = data
 
 
-class test_DatabaseBackend(Case):
+class test_DatabaseBackend(AppCase):
 
     @skip_if_pypy
     @skip_if_jython
-    def setUp(self):
+    def setup(self):
         if DatabaseBackend is None:
             raise SkipTest('sqlalchemy not installed')
+        self.uri = 'sqlite:///test.db'
+
+    def test_retry_helper(self):
+        from celery.backends.database import OperationalError
+
+        calls = [0]
+
+        @retry
+        def raises():
+            calls[0] += 1
+            raise OperationalError(1, 2, 3)
+
+        with self.assertRaises(OperationalError):
+            raises(max_retries=5)
+        self.assertEqual(calls[0], 5)
 
     def test_missing_SQLAlchemy_raises_ImproperlyConfigured(self):
         with mask_modules('sqlalchemy'):
@@ -50,47 +61,26 @@ class test_DatabaseBackend(Case):
             with self.assertRaises(ImproperlyConfigured):
                 _sqlalchemy_installed()
 
-    def test_pickle_hack_for_sqla_05(self):
-        import sqlalchemy as sa
-        from celery.backends.database import session
-        prev_base = session.ResultModelBase
-        prev_ver, sa.__version__ = sa.__version__, '0.5.0'
-        prev_models = sys.modules.pop('celery.backends.database.models', None)
-        try:
-            from sqlalchemy.ext.declarative import declarative_base
-            session.ResultModelBase = declarative_base()
-            from celery.backends.database.dfd042c7 import PickleType as Type1
-            from celery.backends.database.models import PickleType as Type2
-            self.assertIs(Type1, Type2)
-        finally:
-            sys.modules['celery.backends.database.models'] = prev_models
-            sa.__version__ = prev_ver
-            session.ResultModelBase = prev_base
-
     def test_missing_dburi_raises_ImproperlyConfigured(self):
-        conf = app_or_default().conf
-        prev, conf.CELERY_RESULT_DBURI = conf.CELERY_RESULT_DBURI, None
-        try:
-            with self.assertRaises(ImproperlyConfigured):
-                DatabaseBackend()
-        finally:
-            conf.CELERY_RESULT_DBURI = prev
+        self.app.conf.CELERY_RESULT_DBURI = None
+        with self.assertRaises(ImproperlyConfigured):
+            DatabaseBackend(app=self.app)
 
     def test_missing_task_id_is_PENDING(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         self.assertEqual(tb.get_status('xxx-does-not-exist'), states.PENDING)
 
     def test_missing_task_meta_is_dict_with_pending(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         self.assertDictContainsSubset({
             'status': states.PENDING,
             'task_id': 'xxx-does-not-exist-at-all',
             'result': None,
-            'traceback': None,
+            'traceback': None
         }, tb.get_task_meta('xxx-does-not-exist-at-all'))
 
     def test_mark_as_done(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
 
         tid = uuid()
 
@@ -102,7 +92,7 @@ class test_DatabaseBackend(Case):
         self.assertEqual(tb.get_result(tid), 42)
 
     def test_is_pickled(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
 
         tid2 = uuid()
         result = {'foo': 'baz', 'bar': SomeClass(12345)}
@@ -113,23 +103,23 @@ class test_DatabaseBackend(Case):
         self.assertEqual(rindb.get('bar').data, 12345)
 
     def test_mark_as_started(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         tid = uuid()
         tb.mark_as_started(tid)
         self.assertEqual(tb.get_status(tid), states.STARTED)
 
     def test_mark_as_revoked(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         tid = uuid()
         tb.mark_as_revoked(tid)
         self.assertEqual(tb.get_status(tid), states.REVOKED)
 
     def test_mark_as_retry(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         tid = uuid()
         try:
             raise KeyError('foo')
-        except KeyError, exception:
+        except KeyError as exception:
             import traceback
             trace = '\n'.join(traceback.format_stack())
             tb.mark_as_retry(tid, exception, traceback=trace)
@@ -138,12 +128,12 @@ class test_DatabaseBackend(Case):
             self.assertEqual(tb.get_traceback(tid), trace)
 
     def test_mark_as_failure(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
 
         tid3 = uuid()
         try:
             raise KeyError('foo')
-        except KeyError, exception:
+        except KeyError as exception:
             import traceback
             trace = '\n'.join(traceback.format_stack())
             tb.mark_as_failure(tid3, exception, traceback=trace)
@@ -152,27 +142,28 @@ class test_DatabaseBackend(Case):
             self.assertEqual(tb.get_traceback(tid3), trace)
 
     def test_forget(self):
-        tb = DatabaseBackend(backend='memory://')
+        tb = DatabaseBackend(self.uri, backend='memory://', app=self.app)
         tid = uuid()
         tb.mark_as_done(tid, {'foo': 'bar'})
         tb.mark_as_done(tid, {'foo': 'bar'})
-        x = AsyncResult(tid, backend=tb)
+        x = self.app.AsyncResult(tid, backend=tb)
         x.forget()
         self.assertIsNone(x.result)
 
     def test_process_cleanup(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         tb.process_cleanup()
 
+    @depends_on_current_app
     def test_reduce(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         self.assertTrue(loads(dumps(tb)))
 
     def test_save__restore__delete_group(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
 
         tid = uuid()
-        res = {u'something': 'special'}
+        res = {'something': 'special'}
         self.assertEqual(tb.save_group(tid, res), res)
 
         res2 = tb.restore_group(tid)
@@ -184,7 +175,7 @@ class test_DatabaseBackend(Case):
         self.assertIsNone(tb.restore_group('xxx-nonexisting-id'))
 
     def test_cleanup(self):
-        tb = DatabaseBackend()
+        tb = DatabaseBackend(self.uri, app=self.app)
         for i in range(10):
             tb.mark_as_done(uuid(), 42)
             tb.save_group(uuid(), {'foo': 'bar'})
