@@ -1,5 +1,4 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import datetime
 import uuid
@@ -8,12 +7,11 @@ from mock import MagicMock, Mock, patch, sentinel
 from nose import SkipTest
 from pickle import loads, dumps
 
-from celery import Celery
 from celery import states
 from celery.backends import mongodb as module
 from celery.backends.mongodb import MongoBackend, Bunch, pymongo
 from celery.exceptions import ImproperlyConfigured
-from celery.tests.utils import AppCase
+from celery.tests.case import AppCase, depends_on_current_app
 
 COLLECTION = 'taskmeta_celery'
 TASK_ID = str(uuid.uuid1())
@@ -27,7 +25,7 @@ MONGODB_COLLECTION = 'collection1'
 
 class test_MongoBackend(AppCase):
 
-    def setUp(self):
+    def setup(self):
         if pymongo is None:
             raise SkipTest('pymongo is not installed.')
 
@@ -37,9 +35,9 @@ class test_MongoBackend(AppCase):
         R['Binary'], module.Binary = module.Binary, Mock()
         R['datetime'], datetime.datetime = datetime.datetime, Mock()
 
-        self.backend = MongoBackend()
+        self.backend = MongoBackend(app=self.app)
 
-    def tearDown(self):
+    def teardown(self):
         MongoBackend.encode = self._reset['encode']
         MongoBackend.decode = self._reset['decode']
         module.Binary = self._reset['Binary']
@@ -54,74 +52,75 @@ class test_MongoBackend(AppCase):
         prev, module.pymongo = module.pymongo, None
         try:
             with self.assertRaises(ImproperlyConfigured):
-                MongoBackend()
+                MongoBackend(app=self.app)
         finally:
             module.pymongo = prev
 
     def test_init_no_settings(self):
-        celery = Celery(set_as_current=False)
-        celery.conf.CELERY_MONGODB_BACKEND_SETTINGS = []
+        self.app.conf.CELERY_MONGODB_BACKEND_SETTINGS = []
         with self.assertRaises(ImproperlyConfigured):
-            MongoBackend(app=celery)
+            MongoBackend(app=self.app)
 
     def test_init_settings_is_None(self):
-        celery = Celery(set_as_current=False)
-        celery.conf.CELERY_MONGODB_BACKEND_SETTINGS = None
-        MongoBackend(app=celery)
+        self.app.conf.CELERY_MONGODB_BACKEND_SETTINGS = None
+        MongoBackend(app=self.app)
 
     def test_restore_group_no_entry(self):
-        x = MongoBackend()
+        x = MongoBackend(app=self.app)
         x.collection = Mock()
         fo = x.collection.find_one = Mock()
         fo.return_value = None
         self.assertIsNone(x._restore_group('1f3fab'))
 
+    @depends_on_current_app
     def test_reduce(self):
-        x = MongoBackend()
+        x = MongoBackend(app=self.app)
         self.assertTrue(loads(dumps(x)))
 
     def test_get_connection_connection_exists(self):
 
-        with patch('pymongo.connection.Connection') as mock_Connection:
+        with patch('pymongo.MongoClient') as mock_Connection:
             self.backend._connection = sentinel._connection
 
             connection = self.backend._get_connection()
 
-            self.assertEquals(sentinel._connection, connection)
+            self.assertEqual(sentinel._connection, connection)
             self.assertFalse(mock_Connection.called)
 
     def test_get_connection_no_connection_host(self):
 
-        with patch('pymongo.connection.Connection') as mock_Connection:
+        with patch('pymongo.MongoClient') as mock_Connection:
             self.backend._connection = None
-            self.backend.mongodb_host = MONGODB_HOST
-            self.backend.mongodb_port = MONGODB_PORT
+            self.backend.host = MONGODB_HOST
+            self.backend.port = MONGODB_PORT
             mock_Connection.return_value = sentinel.connection
 
             connection = self.backend._get_connection()
             mock_Connection.assert_called_once_with(
-                MONGODB_HOST, MONGODB_PORT, max_pool_size=10)
-            self.assertEquals(sentinel.connection, connection)
+                host='mongodb://localhost:27017', ssl=False, max_pool_size=10,
+                auto_start_request=False)
+            self.assertEqual(sentinel.connection, connection)
 
     def test_get_connection_no_connection_mongodb_uri(self):
 
-        with patch('pymongo.connection.Connection') as mock_Connection:
+        with patch('pymongo.MongoClient') as mock_Connection:
             mongodb_uri = 'mongodb://%s:%d' % (MONGODB_HOST, MONGODB_PORT)
             self.backend._connection = None
-            self.backend.mongodb_host = mongodb_uri
+            self.backend.host = mongodb_uri
 
             mock_Connection.return_value = sentinel.connection
 
             connection = self.backend._get_connection()
             mock_Connection.assert_called_once_with(
-                mongodb_uri, max_pool_size=10)
-            self.assertEquals(sentinel.connection, connection)
+                host=mongodb_uri, ssl=False, max_pool_size=10,
+                auto_start_request=False)
+            self.assertEqual(sentinel.connection, connection)
 
     @patch('celery.backends.mongodb.MongoBackend._get_connection')
     def test_get_database_no_existing(self, mock_get_connection):
         # Should really check for combinations of these two, to be complete.
-        self.backend.mongodb_user = MONGODB_USER
-        self.backend.mongodb_password = MONGODB_PASSWORD
+        self.backend.user = MONGODB_USER
+        self.backend.password = MONGODB_PASSWORD
 
         mock_database = Mock()
         mock_connection = MagicMock(spec=['__getitem__'])
@@ -138,8 +137,8 @@ class test_MongoBackend(AppCase):
     @patch('celery.backends.mongodb.MongoBackend._get_connection')
     def test_get_database_no_existing_no_auth(self, mock_get_connection):
         # Should really check for combinations of these two, to be complete.
-        self.backend.mongodb_user = None
-        self.backend.mongodb_password = None
+        self.backend.user = None
+        self.backend.password = None
 
         mock_database = Mock()
         mock_connection = MagicMock(spec=['__getitem__'])
@@ -155,15 +154,15 @@ class test_MongoBackend(AppCase):
     def test_process_cleanup(self):
         self.backend._connection = None
         self.backend.process_cleanup()
-        self.assertEquals(self.backend._connection, None)
+        self.assertEqual(self.backend._connection, None)
 
         self.backend._connection = 'not none'
         self.backend.process_cleanup()
-        self.assertEquals(self.backend._connection, None)
+        self.assertEqual(self.backend._connection, None)
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_store_result(self, mock_get_database):
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -177,12 +176,12 @@ class test_MongoBackend(AppCase):
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
         mock_collection.save.assert_called_once()
-        self.assertEquals(sentinel.result, ret_val)
+        self.assertEqual(sentinel.result, ret_val)
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_get_task_meta_for(self, mock_get_database):
         datetime.datetime = self._reset['datetime']
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -195,14 +194,14 @@ class test_MongoBackend(AppCase):
 
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
-        self.assertEquals(
+        self.assertEqual(
             ['status', 'task_id', 'date_done', 'traceback', 'result',
              'children'],
-            ret_val.keys())
+            list(ret_val.keys()))
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_get_task_meta_for_no_result(self, mock_get_database):
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -215,11 +214,11 @@ class test_MongoBackend(AppCase):
 
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
-        self.assertEquals({'status': states.PENDING, 'result': None}, ret_val)
+        self.assertEqual({'status': states.PENDING, 'result': None}, ret_val)
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_save_group(self, mock_get_database):
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -233,11 +232,11 @@ class test_MongoBackend(AppCase):
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
         mock_collection.save.assert_called_once()
-        self.assertEquals(sentinel.result, ret_val)
+        self.assertEqual(sentinel.result, ret_val)
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_restore_group(self, mock_get_database):
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -252,11 +251,14 @@ class test_MongoBackend(AppCase):
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
         mock_collection.find_one.assert_called_once_with(
             {'_id': sentinel.taskset_id})
-        self.assertEquals(['date_done', 'result', 'task_id'], ret_val.keys())
+        self.assertEqual(
+            ['date_done', 'result', 'task_id'],
+            list(ret_val.keys()),
+        )
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_delete_group(self, mock_get_database):
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -273,7 +275,7 @@ class test_MongoBackend(AppCase):
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_forget(self, mock_get_database):
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -292,7 +294,7 @@ class test_MongoBackend(AppCase):
     @patch('celery.backends.mongodb.MongoBackend._get_database')
     def test_cleanup(self, mock_get_database):
         datetime.datetime = self._reset['datetime']
-        self.backend.mongodb_taskmeta_collection = MONGODB_COLLECTION
+        self.backend.taskmeta_collection = MONGODB_COLLECTION
 
         mock_database = MagicMock(spec=['__getitem__', '__setitem__'])
         mock_collection = Mock()
@@ -309,13 +311,13 @@ class test_MongoBackend(AppCase):
         mock_collection.assert_called_once()
 
     def test_get_database_authfailure(self):
-        x = MongoBackend()
+        x = MongoBackend(app=self.app)
         x._get_connection = Mock()
         conn = x._get_connection.return_value = {}
-        db = conn[x.mongodb_database] = Mock()
+        db = conn[x.database_name] = Mock()
         db.authenticate.return_value = False
-        x.mongodb_user = 'jerry'
-        x.mongodb_password = 'cere4l'
+        x.user = 'jerry'
+        x.password = 'cere4l'
         with self.assertRaises(ImproperlyConfigured):
             x._get_database()
         db.authenticate.assert_called_with('jerry', 'cere4l')
