@@ -1,20 +1,19 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 from mock import Mock, patch
 
-from celery import current_app
+from celery.platforms import IS_WINDOWS
 from celery.bin.celeryd_detach import (
     detach,
     detached_celeryd,
     main,
 )
 
-from celery.tests.utils import Case, override_stdouts
+from celery.tests.case import AppCase, override_stdouts
 
 
-if not current_app.IS_WINDOWS:
-    class test_detached(Case):
+if not IS_WINDOWS:
+    class test_detached(AppCase):
 
         @patch('celery.bin.celeryd_detach.detached')
         @patch('os.execv')
@@ -27,23 +26,23 @@ if not current_app.IS_WINDOWS:
 
             detach('/bin/boo', ['a', 'b', 'c'], logfile='/var/log',
                    pidfile='/var/pid')
-            detached.assert_called_with('/var/log', '/var/pid', None, None,
-                                        None, None, False)
+            detached.assert_called_with('/var/log', '/var/pid', None, None, 0,
+                                        None, False)
             execv.assert_called_with('/bin/boo', ['/bin/boo', 'a', 'b', 'c'])
 
             execv.side_effect = Exception('foo')
             r = detach('/bin/boo', ['a', 'b', 'c'],
-                       logfile='/var/log', pidfile='/var/pid')
+                       logfile='/var/log', pidfile='/var/pid', app=self.app)
             context.__enter__.assert_called_with()
             self.assertTrue(logger.critical.called)
             setup_logs.assert_called_with('ERROR', '/var/log')
             self.assertEqual(r, 1)
 
 
-class test_PartialOptionParser(Case):
+class test_PartialOptionParser(AppCase):
 
     def test_parser(self):
-        x = detached_celeryd()
+        x = detached_celeryd(self.app)
         p = x.Parser('celeryd_detach')
         options, values = p.parse_args(['--logfile=foo', '--fake', '--enable',
                                         'a', 'b', '-c1', '-d', '2'])
@@ -65,13 +64,13 @@ class test_PartialOptionParser(Case):
         p.get_option('--logfile').nargs = 1
 
 
-class test_Command(Case):
+class test_Command(AppCase):
     argv = ['--autoscale=10,2', '-c', '1',
             '--logfile=/var/log', '-lDEBUG',
             '--', '.disable_rate_limits=1']
 
     def test_parse_options(self):
-        x = detached_celeryd()
+        x = detached_celeryd(app=self.app)
         o, v, l = x.parse_options('cd', self.argv)
         self.assertEqual(o.logfile, '/var/log')
         self.assertEqual(l, ['--autoscale=10,2', '-c', '1',
@@ -82,19 +81,22 @@ class test_Command(Case):
     @patch('sys.exit')
     @patch('celery.bin.celeryd_detach.detach')
     def test_execute_from_commandline(self, detach, exit):
-        x = detached_celeryd()
+        x = detached_celeryd(app=self.app)
         x.execute_from_commandline(self.argv)
         self.assertTrue(exit.called)
         detach.assert_called_with(
             path=x.execv_path, uid=None, gid=None,
-            umask=None, fake=False, logfile='/var/log', pidfile='celeryd.pid',
-            argv=['-m', 'celery.bin.celeryd', '-c', '1', '-lDEBUG',
-                  '--logfile=/var/log', '--pidfile=celeryd.pid',
-                  '--', '.disable_rate_limits=1'],
+            umask=0, fake=False, logfile='/var/log', pidfile='celeryd.pid',
+            argv=x.execv_argv + [
+                '-c', '1', '-lDEBUG',
+                '--logfile=/var/log', '--pidfile=celeryd.pid',
+                '--', '.disable_rate_limits=1'
+            ],
+            app=self.app,
         )
 
     @patch('celery.bin.celeryd_detach.detached_celeryd')
     def test_main(self, command):
         c = command.return_value = Mock()
-        main()
+        main(self.app)
         c.execute_from_commandline.assert_called_with()
