@@ -1,7 +1,5 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
-import os
 import sys
 
 from nose import SkipTest
@@ -15,13 +13,13 @@ from celery.concurrency.eventlet import (
     TaskPool,
 )
 
-from celery.tests.utils import Case, mock_module, patch_many, skip_if_pypy
+from celery.tests.case import AppCase, mock_module, patch_many, skip_if_pypy
 
 
-class EventletCase(Case):
+class EventletCase(AppCase):
 
     @skip_if_pypy
-    def setUp(self):
+    def setup(self):
         if is_pypy:
             raise SkipTest('mock_modules not working on PyPy1.9')
         try:
@@ -31,7 +29,7 @@ class EventletCase(Case):
                 'eventlet not installed, skipping related tests.')
 
     @skip_if_pypy
-    def tearDown(self):
+    def teardown(self):
         for mod in [mod for mod in sys.modules if mod.startswith('eventlet')]:
             try:
                 del(sys.modules[mod])
@@ -42,19 +40,10 @@ class EventletCase(Case):
 class test_aaa_eventlet_patch(EventletCase):
 
     def test_aaa_is_patched(self):
-        raise SkipTest("side effects")
-        monkey_patched = []
-        prev_monkey_patch = self.eventlet.monkey_patch
-        self.eventlet.monkey_patch = lambda: monkey_patched.append(True)
-        prev_eventlet = sys.modules.pop('celery.concurrency.eventlet', None)
-        os.environ.pop('EVENTLET_NOPATCH')
-        try:
-            import celery.concurrency.eventlet  # noqa
-            self.assertTrue(monkey_patched)
-        finally:
-            sys.modules['celery.concurrency.eventlet'] = prev_eventlet
-            os.environ['EVENTLET_NOPATCH'] = 'yes'
-            self.eventlet.monkey_patch = prev_monkey_patch
+        with patch('eventlet.monkey_patch', create=True) as monkey_patch:
+            from celery import maybe_patch_concurrency
+            maybe_patch_concurrency(['x', '-P', 'eventlet'])
+            monkey_patch.assert_called_with()
 
 
 eventlet_modules = (
