@@ -1,11 +1,10 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 from mock import Mock
 
 from celery.concurrency.threads import NullDict, TaskPool, apply_target
 
-from celery.tests.utils import Case, mask_modules, mock_module
+from celery.tests.case import AppCase, Case, mask_modules, mock_module
 
 
 class test_NullDict(Case):
@@ -17,7 +16,7 @@ class test_NullDict(Case):
             x['foo']
 
 
-class test_TaskPool(Case):
+class test_TaskPool(AppCase):
 
     def test_without_threadpool(self):
 
