@@ -1,13 +1,11 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import socket
 
 from mock import Mock
 
-from celery import Celery
-from celery import events
-from celery.tests.utils import AppCase
+from celery.events import Event
+from celery.tests.case import AppCase
 
 
 class MockProducer(object):
@@ -34,7 +32,7 @@ class MockProducer(object):
 class test_Event(AppCase):
 
     def test_constructor(self):
-        event = events.Event('world war II')
+        event = Event('world war II')
         self.assertEqual(event['type'], 'world war II')
         self.assertTrue(event['timestamp'])
 
@@ -42,21 +40,29 @@ class test_Event(AppCase):
 class test_EventDispatcher(AppCase):
 
     def test_redis_uses_fanout_exchange(self):
-        with Celery(set_as_current=False) as app:
-            app.connection = Mock()
-            conn = app.connection.return_value = Mock()
-            conn.transport.driver_type = 'redis'
+        self.app.connection = Mock()
+        conn = self.app.connection.return_value = Mock()
+        conn.transport.driver_type = 'redis'
 
-            dispatcher = app.events.Dispatcher(conn, enabled=False)
-            self.assertEqual(dispatcher.exchange.type, 'fanout')
+        dispatcher = self.app.events.Dispatcher(conn, enabled=False)
+        self.assertEqual(dispatcher.exchange.type, 'fanout')
 
     def test_others_use_topic_exchange(self):
-        with Celery(set_as_current=False) as app:
-            app.connection = Mock()
-            conn = app.connection.return_value = Mock()
-            conn.transport.driver_type = 'amqp'
-            dispatcher = app.events.Dispatcher(conn, enabled=False)
-            self.assertEqual(dispatcher.exchange.type, 'topic')
+        self.app.connection = Mock()
+        conn = self.app.connection.return_value = Mock()
+        conn.transport.driver_type = 'amqp'
+        dispatcher = self.app.events.Dispatcher(conn, enabled=False)
+        self.assertEqual(dispatcher.exchange.type, 'topic')
+
+    def test_takes_channel_connection(self):
+        x = self.app.events.Dispatcher(channel=Mock())
+        self.assertIs(x.connection, x.channel.connection.client)
+
+    def test_sql_transports_disabled(self):
+        conn = Mock()
+        conn.transport.driver_type = 'sql'
+        x = self.app.events.Dispatcher(connection=conn)
+        self.assertFalse(x.enabled)
 
     def test_send(self):
         producer = MockProducer()
@@ -164,9 +170,11 @@ class test_EventReceiver(AppCase):
 
         connection = Mock()
         connection.transport_cls = 'memory'
-        r = events.EventReceiver(connection,
-                                 handlers={'world-war': my_handler},
-                                 node_id='celery.tests')
+        r = self.app.events.Receiver(
+            connection,
+            handlers={'world-war': my_handler},
+            node_id='celery.tests',
+        )
         r._receive(message, object())
         self.assertTrue(got_event[0])
 
@@ -181,31 +189,38 @@ class test_EventReceiver(AppCase):
 
         connection = Mock()
         connection.transport_cls = 'memory'
-        r = events.EventReceiver(connection, node_id='celery.tests')
-        events.EventReceiver.handlers['*'] = my_handler
-        try:
-            r._receive(message, object())
-            self.assertTrue(got_event[0])
-        finally:
-            events.EventReceiver.handlers = {}
+        r = self.app.events.Receiver(connection, node_id='celery.tests')
+        r.handlers['*'] = my_handler
+        r._receive(message, object())
+        self.assertTrue(got_event[0])
 
     def test_itercapture(self):
         connection = self.app.connection()
         try:
             r = self.app.events.Receiver(connection, node_id='celery.tests')
             it = r.itercapture(timeout=0.0001, wakeup=False)
-            consumer = it.next()
-            self.assertTrue(consumer.queues)
-            self.assertEqual(consumer.callbacks[0], r._receive)
 
             with self.assertRaises(socket.timeout):
-                it.next()
+                next(it)
 
             with self.assertRaises(socket.timeout):
                 r.capture(timeout=0.00001)
         finally:
             connection.close()
 
+    def test_event_from_message_localize_disabled(self):
+        r = self.app.events.Receiver(Mock(), node_id='celery.tests')
+        r.adjust_clock = Mock()
+        ts_adjust = Mock()
+
+        r.event_from_message(
+            {'type': 'worker-online', 'clock': 313},
+            localize=False,
+            adjust_timestamp=ts_adjust,
+        )
+        self.assertFalse(ts_adjust.called)
+        r.adjust_clock.assert_called_with(313)
+
     def test_itercapture_limit(self):
         connection = self.app.connection()
         channel = connection.channel()
@@ -215,17 +230,19 @@ class test_EventReceiver(AppCase):
             def handler(event):
                 events_received[0] += 1
 
-            producer = self.app.events.Dispatcher(connection,
-                                                  enabled=True,
-                                                  channel=channel)
-            r = self.app.events.Receiver(connection,
-                                         handlers={'*': handler},
-                                         node_id='celery.tests')
+            producer = self.app.events.Dispatcher(
+                connection, enabled=True, channel=channel,
+            )
+            r = self.app.events.Receiver(
+                connection,
+                handlers={'*': handler},
+                node_id='celery.tests',
+            )
             evs = ['ev1', 'ev2', 'ev3', 'ev4', 'ev5']
             for ev in evs:
                 producer.send(ev)
             it = r.itercapture(limit=4, wakeup=True)
-            it.next()  # skip consumer (see itercapture)
+            next(it)  # skip consumer (see itercapture)
             list(it)
             self.assertEqual(events_received[0], 4)
         finally:
