@@ -2,12 +2,12 @@ from __future__ import absolute_import
 
 from nose import SkipTest
 
-from celery.tests.utils import Case
+from celery.tests.case import AppCase
 
 
-class SecurityCase(Case):
+class SecurityCase(AppCase):
 
-    def setUp(self):
+    def setup(self):
         try:
             from OpenSSL import crypto  # noqa
         except ImportError:
