@@ -1,67 +1,133 @@
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import sys
 import socket
 
+from collections import defaultdict
 from datetime import datetime, timedelta
 
 from kombu import pidbox
-from mock import Mock, patch
+from mock import Mock, patch, call
 
-from celery import current_app
 from celery.datastructures import AttributeDict
-from celery.task import task
+from celery.five import Queue as FastQueue
 from celery.utils import uuid
 from celery.utils.timer2 import Timer
 from celery.worker import WorkController as _WC
 from celery.worker import consumer
 from celery.worker import control
-from celery.worker import state
-from celery.worker.buckets import FastQueue
-from celery.worker.job import TaskRequest
+from celery.worker import state as worker_state
+from celery.worker.job import Request
 from celery.worker.state import revoked
 from celery.worker.control import Panel
-from celery.tests.utils import Case
+from celery.worker.pidbox import Pidbox, gPidbox
+
+from celery.tests.case import AppCase
 
 hostname = socket.gethostname()
 
 
-@task(rate_limit=200)                   # for extra info in dump_tasks
-def mytask():
-    pass
-
-
 class WorkController(object):
     autoscaler = None
 
+    def stats(self):
+        return {'total': worker_state.total_count}
+
 
 class Consumer(consumer.Consumer):
 
-    def __init__(self):
-        self.ready_queue = FastQueue()
+    def __init__(self, app):
+        self.app = app
+        self.buffer = FastQueue()
+        self.handle_task = self.buffer.put
         self.timer = Timer()
-        self.app = current_app
         self.event_dispatcher = Mock()
         self.controller = WorkController()
         self.task_consumer = Mock()
+        self.prefetch_multiplier = 1
+        self.initial_prefetch_count = 1
 
         from celery.concurrency.base import BasePool
         self.pool = BasePool(10)
-
-    @property
-    def info(self):
-        return {'xyz': 'XYZ'}
+        self.task_buckets = defaultdict(lambda: None)
 
 
-class test_ControlPanel(Case):
+class test_Pidbox(AppCase):
 
-    def setUp(self):
-        self.app = current_app
-        self.panel = self.create_panel(consumer=Consumer())
+    def test_shutdown(self):
+        with patch('celery.worker.pidbox.ignore_errors') as eig:
+            parent = Mock()
+            pbox = Pidbox(parent)
+            pbox._close_channel = Mock()
+            self.assertIs(pbox.c, parent)
+            pconsumer = pbox.consumer = Mock()
+            cancel = pconsumer.cancel
+            pbox.shutdown(parent)
+            eig.assert_called_with(parent, cancel)
+            pbox._close_channel.assert_called_with(parent)
+
+
+class test_Pidbox_green(AppCase):
+
+    def test_stop(self):
+        parent = Mock()
+        g = gPidbox(parent)
+        stopped = g._node_stopped = Mock()
+        shutdown = g._node_shutdown = Mock()
+        close_chan = g._close_channel = Mock()
+
+        g.stop(parent)
+        shutdown.set.assert_called_with()
+        stopped.wait.assert_called_with()
+        close_chan.assert_called_with(parent)
+        self.assertIsNone(g._node_stopped)
+        self.assertIsNone(g._node_shutdown)
+
+        close_chan.reset()
+        g.stop(parent)
+        close_chan.assert_called_with(parent)
+
+    def test_resets(self):
+        parent = Mock()
+        g = gPidbox(parent)
+        g._resets = 100
+        g.reset()
+        self.assertEqual(g._resets, 101)
+
+    def test_loop(self):
+        parent = Mock()
+        conn = parent.connect.return_value = self.app.connection()
+        drain = conn.drain_events = Mock()
+        g = gPidbox(parent)
+        parent.connection = Mock()
+        do_reset = g._do_reset = Mock()
+
+        call_count = [0]
+
+        def se(*args, **kwargs):
+            if call_count[0] > 2:
+                g._node_shutdown.set()
+            g.reset()
+            call_count[0] += 1
+        drain.side_effect = se
+        g.loop(parent)
+
+        self.assertEqual(do_reset.call_count, 4)
+
+
+class test_ControlPanel(AppCase):
+
+    def setup(self):
+        self.panel = self.create_panel(consumer=Consumer(self.app))
+
+        @self.app.task(rate_limit=200, shared=False)
+        def mytask():
+            pass
+        self.mytask = mytask
 
     def create_state(self, **kwargs):
         kwargs.setdefault('app', self.app)
+        kwargs.setdefault('hostname', hostname)
         return AttributeDict(kwargs)
 
     def create_panel(self, **kwargs):
@@ -70,31 +136,63 @@ class test_ControlPanel(Case):
                                              handlers=Panel.data)
 
     def test_enable_events(self):
-        consumer = Consumer()
+        consumer = Consumer(self.app)
         panel = self.create_panel(consumer=consumer)
-        consumer.event_dispatcher.enabled = False
+        evd = consumer.event_dispatcher
+        evd.groups = set()
         panel.handle('enable_events')
-        self.assertTrue(consumer.event_dispatcher.enable.call_count)
-        self.assertIn(
-            ('worker-online', ),
-            consumer.event_dispatcher.send.call_args,
-        )
-        consumer.event_dispatcher.enabled = True
+        self.assertIn('task', evd.groups)
+        evd.groups = set(['task'])
         self.assertIn('already enabled', panel.handle('enable_events')['ok'])
 
     def test_disable_events(self):
-        consumer = Consumer()
+        consumer = Consumer(self.app)
         panel = self.create_panel(consumer=consumer)
-        consumer.event_dispatcher.enabled = True
+        evd = consumer.event_dispatcher
+        evd.enabled = True
+        evd.groups = set(['task'])
         panel.handle('disable_events')
-        self.assertTrue(consumer.event_dispatcher.disable.call_count)
-        self.assertIn(('worker-offline', ),
-                      consumer.event_dispatcher.send.call_args)
-        consumer.event_dispatcher.enabled = False
+        self.assertNotIn('task', evd.groups)
         self.assertIn('already disabled', panel.handle('disable_events')['ok'])
 
+    def test_clock(self):
+        consumer = Consumer(self.app)
+        panel = self.create_panel(consumer=consumer)
+        panel.state.app.clock.value = 313
+        x = panel.handle('clock')
+        self.assertEqual(x['clock'], 313)
+
+    def test_hello(self):
+        consumer = Consumer(self.app)
+        panel = self.create_panel(consumer=consumer)
+        panel.state.app.clock.value = 313
+        worker_state.revoked.add('revoked1')
+        try:
+            x = panel.handle('hello', {'from_node': 'george@vandelay.com'})
+            self.assertIn('revoked1', x['revoked'])
+            self.assertEqual(x['clock'], 314)  # incremented
+        finally:
+            worker_state.revoked.discard('revoked1')
+
+    def test_conf(self):
+        return
+        consumer = Consumer(self.app)
+        panel = self.create_panel(consumer=consumer)
+        self.app.conf.SOME_KEY6 = 'hello world'
+        x = panel.handle('dump_conf')
+        self.assertIn('SOME_KEY6', x)
+
+    def test_election(self):
+        consumer = Consumer(self.app)
+        panel = self.create_panel(consumer=consumer)
+        consumer.gossip = Mock()
+        panel.handle(
+            'election', {'id': 'id', 'topic': 'topic', 'action': 'action'},
+        )
+        consumer.gossip.election.assert_called_with('id', 'topic', 'action')
+
     def test_heartbeat(self):
-        consumer = Consumer()
+        consumer = Consumer(self.app)
         panel = self.create_panel(consumer=consumer)
         consumer.event_dispatcher.enabled = True
         panel.handle('heartbeat')
@@ -103,29 +201,29 @@ class test_ControlPanel(Case):
 
     def test_time_limit(self):
         panel = self.create_panel(consumer=Mock())
-        th, ts = mytask.time_limit, mytask.soft_time_limit
-        try:
-            r = panel.handle('time_limit', arguments=dict(
-                task_name=mytask.name, hard=30, soft=10))
-            self.assertEqual((mytask.time_limit, mytask.soft_time_limit),
-                             (30, 10))
-            self.assertIn('ok', r)
-            r = panel.handle('time_limit', arguments=dict(
-                task_name=mytask.name, hard=None, soft=None))
-            self.assertEqual((mytask.time_limit, mytask.soft_time_limit),
-                             (None, None))
-            self.assertIn('ok', r)
+        r = panel.handle('time_limit', arguments=dict(
+            task_name=self.mytask.name, hard=30, soft=10))
+        self.assertEqual(
+            (self.mytask.time_limit, self.mytask.soft_time_limit),
+            (30, 10),
+        )
+        self.assertIn('ok', r)
+        r = panel.handle('time_limit', arguments=dict(
+            task_name=self.mytask.name, hard=None, soft=None))
+        self.assertEqual(
+            (self.mytask.time_limit, self.mytask.soft_time_limit),
+            (None, None),
+        )
+        self.assertIn('ok', r)
 
-            r = panel.handle('time_limit', arguments=dict(
-                task_name='248e8afya9s8dh921eh928', hard=30))
-            self.assertIn('error', r)
-        finally:
-            mytask.time_limit, mytask.soft_time_limit = th, ts
+        r = panel.handle('time_limit', arguments=dict(
+            task_name='248e8afya9s8dh921eh928', hard=30))
+        self.assertIn('error', r)
 
     def test_active_queues(self):
         import kombu
 
-        x = kombu.Consumer(current_app.connection(),
+        x = kombu.Consumer(self.app.connection(),
                            [kombu.Queue('foo', kombu.Exchange('foo'), 'foo'),
                             kombu.Queue('bar', kombu.Exchange('bar'), 'bar')],
                            auto_declare=False)
@@ -142,28 +240,28 @@ class test_ControlPanel(Case):
         self.assertIn('rate_limit=200', info)
 
     def test_stats(self):
-        prev_count, state.total_count = state.total_count, 100
+        prev_count, worker_state.total_count = worker_state.total_count, 100
         try:
-            self.assertDictContainsSubset({'total': 100,
-                                           'consumer': {'xyz': 'XYZ'}},
+            self.assertDictContainsSubset({'total': 100},
                                           self.panel.handle('stats'))
-            self.panel.state.consumer = Mock()
-            self.panel.handle('stats')
-            self.assertTrue(
-                self.panel.state.consumer.controller.autoscaler.info.called)
         finally:
-            state.total_count = prev_count
+            worker_state.total_count = prev_count
 
     def test_report(self):
         self.panel.handle('report')
 
     def test_active(self):
-        r = TaskRequest(mytask.name, 'do re mi', (), {})
-        state.active_requests.add(r)
+        r = Request({
+            'task': self.mytask.name,
+            'id': 'do re mi',
+            'args': (),
+            'kwargs': {},
+        }, app=self.app)
+        worker_state.active_requests.add(r)
         try:
             self.assertTrue(self.panel.handle('dump_active'))
         finally:
-            state.active_requests.discard(r)
+            worker_state.active_requests.discard(r)
 
     def test_pool_grow(self):
 
@@ -178,14 +276,24 @@ class test_ControlPanel(Case):
             def shrink(self, n=1):
                 self.size -= n
 
-        consumer = Consumer()
-        consumer.pool = MockPool()
+            @property
+            def num_processes(self):
+                return self.size
+
+        consumer = Consumer(self.app)
+        consumer.prefetch_multiplier = 8
+        consumer.qos = Mock(name='qos')
+        consumer.pool = MockPool(1)
         panel = self.create_panel(consumer=consumer)
 
         panel.handle('pool_grow')
         self.assertEqual(consumer.pool.size, 2)
+        consumer.qos.increment_eventually.assert_called_with(8)
+        self.assertEqual(consumer.initial_prefetch_count, 16)
         panel.handle('pool_shrink')
         self.assertEqual(consumer.pool.size, 1)
+        consumer.qos.decrement_eventually.assert_called_with(8)
+        self.assertEqual(consumer.initial_prefetch_count, 8)
 
         panel.state.consumer = Mock()
         panel.state.consumer.controller = Mock()
@@ -214,7 +322,7 @@ class test_ControlPanel(Case):
             def consuming_from(self, queue):
                 return queue in self.queues
 
-        consumer = Consumer()
+        consumer = Consumer(self.app)
         consumer.task_consumer = MockConsumer()
         panel = self.create_panel(consumer=consumer)
 
@@ -226,58 +334,56 @@ class test_ControlPanel(Case):
         self.assertIn('MyQueue', consumer.task_consumer.cancelled)
 
     def test_revoked(self):
-        state.revoked.clear()
-        state.revoked.add('a1')
-        state.revoked.add('a2')
+        worker_state.revoked.clear()
+        worker_state.revoked.add('a1')
+        worker_state.revoked.add('a2')
 
         try:
             self.assertEqual(sorted(self.panel.handle('dump_revoked')),
                              ['a1', 'a2'])
         finally:
-            state.revoked.clear()
+            worker_state.revoked.clear()
 
     def test_dump_schedule(self):
-        consumer = Consumer()
+        consumer = Consumer(self.app)
         panel = self.create_panel(consumer=consumer)
         self.assertFalse(panel.handle('dump_schedule'))
-        r = TaskRequest(mytask.name, 'CAFEBABE', (), {})
-        consumer.timer.schedule.enter(
+        r = Request({
+            'task': self.mytask.name,
+            'id': 'CAFEBABE',
+            'args': (),
+            'kwargs': {},
+        }, app=self.app)
+        consumer.timer.schedule.enter_at(
             consumer.timer.Entry(lambda x: x, (r, )),
             datetime.now() + timedelta(seconds=10))
+        consumer.timer.schedule.enter_at(
+            consumer.timer.Entry(lambda x: x, (object(), )),
+            datetime.now() + timedelta(seconds=10))
         self.assertTrue(panel.handle('dump_schedule'))
 
     def test_dump_reserved(self):
-        from celery.worker import state
-        consumer = Consumer()
-        state.reserved_requests.add(
-            TaskRequest(mytask.name, uuid(), args=(2, 2), kwargs={}),
-        )
+        consumer = Consumer(self.app)
+        worker_state.reserved_requests.add(Request({
+            'task': self.mytask.name,
+            'id': uuid(),
+            'args': (2, 2),
+            'kwargs': {},
+        }, app=self.app))
         try:
             panel = self.create_panel(consumer=consumer)
             response = panel.handle('dump_reserved', {'safe': True})
             self.assertDictContainsSubset(
-                {'name': mytask.name,
+                {'name': self.mytask.name,
                  'args': (2, 2),
                  'kwargs': {},
                  'hostname': socket.gethostname()},
                 response[0],
             )
-            state.reserved_requests.clear()
+            worker_state.reserved_requests.clear()
             self.assertFalse(panel.handle('dump_reserved'))
         finally:
-            state.reserved_requests.clear()
-
-    def test_rate_limit_when_disabled(self):
-        app = current_app
-        app.conf.CELERY_DISABLE_RATE_LIMITS = True
-        try:
-            e = self.panel.handle(
-                'rate_limit',
-                arguments={'task_name': mytask.name,
-                           'rate_limit': '100/m'})
-            self.assertIn('rate limits disabled', e.get('error'))
-        finally:
-            app.conf.CELERY_DISABLE_RATE_LIMITS = False
+            worker_state.reserved_requests.clear()
 
     def test_rate_limit_invalid_rate_limit_string(self):
         e = self.panel.handle('rate_limit', arguments=dict(
@@ -286,34 +392,25 @@ class test_ControlPanel(Case):
 
     def test_rate_limit(self):
 
-        class Consumer(object):
+        class xConsumer(object):
+            reset = False
 
-            class ReadyQueue(object):
-                fresh = False
+            def reset_rate_limits(self):
+                self.reset = True
 
-                def refresh(self):
-                    self.fresh = True
+        consumer = xConsumer()
+        panel = self.create_panel(app=self.app, consumer=consumer)
 
-            def __init__(self):
-                self.ready_queue = self.ReadyQueue()
-
-        consumer = Consumer()
-        panel = self.create_panel(app=current_app, consumer=consumer)
-
-        task = current_app.tasks[mytask.name]
-        old_rate_limit = task.rate_limit
-        try:
-            panel.handle('rate_limit', arguments=dict(task_name=task.name,
-                                                      rate_limit='100/m'))
-            self.assertEqual(task.rate_limit, '100/m')
-            self.assertTrue(consumer.ready_queue.fresh)
-            consumer.ready_queue.fresh = False
-            panel.handle('rate_limit', arguments=dict(task_name=task.name,
-                                                      rate_limit=0))
-            self.assertEqual(task.rate_limit, 0)
-            self.assertTrue(consumer.ready_queue.fresh)
-        finally:
-            task.rate_limit = old_rate_limit
+        task = self.app.tasks[self.mytask.name]
+        panel.handle('rate_limit', arguments=dict(task_name=task.name,
+                                                  rate_limit='100/m'))
+        self.assertEqual(task.rate_limit, '100/m')
+        self.assertTrue(consumer.reset)
+        consumer.reset = False
+        panel.handle('rate_limit', arguments=dict(task_name=task.name,
+                                                  rate_limit=0))
+        self.assertEqual(task.rate_limit, 0)
+        self.assertTrue(consumer.reset)
 
     def test_rate_limit_nonexistant_task(self):
         self.panel.handle('rate_limit', arguments={
@@ -329,7 +426,7 @@ class test_ControlPanel(Case):
         m = {'method': 'revoke',
              'destination': hostname,
              'arguments': {'task_id': tid,
-                           'task_name': mytask.name}}
+                           'task_name': self.mytask.name}}
         self.panel.handle_message(m, None)
         self.assertIn(tid, revoked)
 
@@ -359,17 +456,17 @@ class test_ControlPanel(Case):
     def test_revoke_terminate(self):
         request = Mock()
         request.id = tid = uuid()
-        state.reserved_requests.add(request)
+        worker_state.reserved_requests.add(request)
         try:
             r = control.revoke(Mock(), tid, terminate=True)
             self.assertIn(tid, revoked)
             self.assertTrue(request.terminate.call_count)
-            self.assertIn('terminating', r['ok'])
+            self.assertIn('terminate:', r['ok'])
             # unknown task id only revokes
             r = control.revoke(Mock(), uuid(), terminate=True)
-            self.assertIn('not found', r['ok'])
+            self.assertIn('tasks unknown', r['ok'])
         finally:
-            state.reserved_requests.discard(request)
+            worker_state.reserved_requests.discard(request)
 
     def test_autoscale(self):
         self.panel.state.consumer = Mock()
@@ -390,7 +487,7 @@ class test_ControlPanel(Case):
         m = {'method': 'ping',
              'destination': hostname}
         r = self.panel.handle_message(m, None)
-        self.assertEqual(r, 'pong')
+        self.assertEqual(r, {'ok': 'pong'})
 
     def test_shutdown(self):
         m = {'method': 'shutdown',
@@ -408,17 +505,17 @@ class test_ControlPanel(Case):
                 replies.append(data)
 
         panel = _Node(hostname=hostname,
-                      state=self.create_state(consumer=Consumer()),
+                      state=self.create_state(consumer=Consumer(self.app)),
                       handlers=Panel.data,
                       mailbox=self.app.control.mailbox)
         r = panel.dispatch('ping', reply_to={'exchange': 'x',
                                              'routing_key': 'x'})
-        self.assertEqual(r, 'pong')
-        self.assertDictEqual(replies[0], {panel.hostname: 'pong'})
+        self.assertEqual(r, {'ok': 'pong'})
+        self.assertDictEqual(replies[0], {panel.hostname: {'ok': 'pong'}})
 
     def test_pool_restart(self):
-        consumer = Consumer()
-        consumer.controller = _WC(app=current_app)
+        consumer = Consumer(self.app)
+        consumer.controller = _WC(app=self.app)
         consumer.controller.pool.restart = Mock()
         panel = self.create_panel(consumer=consumer)
         panel.app = self.app
@@ -428,67 +525,59 @@ class test_ControlPanel(Case):
         with self.assertRaises(ValueError):
             panel.handle('pool_restart', {'reloader': _reload})
 
-        current_app.conf.CELERYD_POOL_RESTARTS = True
-        try:
-            panel.handle('pool_restart', {'reloader': _reload})
-            self.assertTrue(consumer.controller.pool.restart.called)
-            self.assertFalse(_reload.called)
-            self.assertFalse(_import.called)
-        finally:
-            current_app.conf.CELERYD_POOL_RESTARTS = False
+        self.app.conf.CELERYD_POOL_RESTARTS = True
+        panel.handle('pool_restart', {'reloader': _reload})
+        self.assertTrue(consumer.controller.pool.restart.called)
+        self.assertFalse(_reload.called)
+        self.assertFalse(_import.called)
 
     def test_pool_restart_import_modules(self):
-        consumer = Consumer()
-        consumer.controller = _WC(app=current_app)
+        consumer = Consumer(self.app)
+        consumer.controller = _WC(app=self.app)
         consumer.controller.pool.restart = Mock()
         panel = self.create_panel(consumer=consumer)
         panel.app = self.app
         _import = consumer.controller.app.loader.import_from_cwd = Mock()
         _reload = Mock()
 
-        current_app.conf.CELERYD_POOL_RESTARTS = True
-        try:
-            panel.handle('pool_restart', {'modules': ['foo', 'bar'],
-                                          'reloader': _reload})
-            self.assertTrue(consumer.controller.pool.restart.called)
-            self.assertFalse(_reload.called)
-            self.assertEqual(
-                [(('foo',), {}), (('bar',), {})],
-                _import.call_args_list,
-            )
-        finally:
-            current_app.conf.CELERYD_POOL_RESTARTS = False
+        self.app.conf.CELERYD_POOL_RESTARTS = True
+        panel.handle('pool_restart', {'modules': ['foo', 'bar'],
+                                      'reloader': _reload})
 
-    def test_pool_restart_relaod_modules(self):
-        consumer = Consumer()
-        consumer.controller = _WC(app=current_app)
+        self.assertTrue(consumer.controller.pool.restart.called)
+        self.assertFalse(_reload.called)
+        self.assertItemsEqual(
+            [call('bar'), call('foo')],
+            _import.call_args_list,
+        )
+
+    def test_pool_restart_reload_modules(self):
+        consumer = Consumer(self.app)
+        consumer.controller = _WC(app=self.app)
         consumer.controller.pool.restart = Mock()
         panel = self.create_panel(consumer=consumer)
         panel.app = self.app
         _import = panel.app.loader.import_from_cwd = Mock()
         _reload = Mock()
 
-        current_app.conf.CELERYD_POOL_RESTARTS = True
-        try:
-            with patch.dict(sys.modules, {'foo': None}):
-                panel.handle('pool_restart', {'modules': ['foo'],
-                                              'reload': False,
-                                              'reloader': _reload})
+        self.app.conf.CELERYD_POOL_RESTARTS = True
+        with patch.dict(sys.modules, {'foo': None}):
+            panel.handle('pool_restart', {'modules': ['foo'],
+                                          'reload': False,
+                                          'reloader': _reload})
 
-                self.assertTrue(consumer.controller.pool.restart.called)
-                self.assertFalse(_reload.called)
-                self.assertFalse(_import.called)
+            self.assertTrue(consumer.controller.pool.restart.called)
+            self.assertFalse(_reload.called)
+            self.assertFalse(_import.called)
 
-                _import.reset_mock()
-                _reload.reset_mock()
-                consumer.controller.pool.restart.reset_mock()
+            _import.reset_mock()
+            _reload.reset_mock()
+            consumer.controller.pool.restart.reset_mock()
 
-                panel.handle('pool_restart', {'modules': ['foo'],
-                                              'reload': True,
-                                              'reloader': _reload})
+            panel.handle('pool_restart', {'modules': ['foo'],
+                                          'reload': True,
+                                          'reloader': _reload})
 
-                self.assertTrue(consumer.controller.pool.restart.called)
-                self.assertTrue(_reload.called)
-                self.assertFalse(_import.called)
-        finally:
-            current_app.conf.CELERYD_POOL_RESTARTS = False
+            self.assertTrue(consumer.controller.pool.restart.called)
+            self.assertTrue(_reload.called)
+            self.assertFalse(_import.called)
