@@ -6,36 +6,38 @@
     Utility functions.
 
 """
-from __future__ import absolute_import
-from __future__ import with_statement
+from __future__ import absolute_import, print_function
 
 import os
 import sys
 import traceback
 import warnings
-import types
 import datetime
 
-from functools import wraps
+from functools import partial, wraps
 from inspect import getargspec
 from pprint import pprint
 
 from kombu.entity import Exchange, Queue
 
 from celery.exceptions import CPendingDeprecationWarning, CDeprecationWarning
-from .compat import StringIO
+from celery.five import StringIO, items, reraise, string_t
+
+__all__ = ['worker_direct', 'warn_deprecated', 'deprecated', 'lpmerge',
+           'is_iterable', 'isatty', 'cry', 'maybe_reraise', 'strtobool',
+           'jsonify', 'gen_task_name', 'nodename', 'nodesplit',
+           'cached_property']
 
-from .functional import noop
 
 PENDING_DEPRECATION_FMT = """
-    %(description)s is scheduled for deprecation in \
-    version %(deprecation)s and removal in version v%(removal)s. \
-    %(alternative)s
+    {description} is scheduled for deprecation in \
+    version {deprecation} and removal in version v{removal}. \
+    {alternative}
 """
 
 DEPRECATION_FMT = """
-    %(description)s is deprecated and scheduled for removal in
-    version %(removal)s. %(alternative)s
+    {description} is deprecated and scheduled for removal in
+    version {removal}. {alternative}
 """
 
 #: Billiard sets this when execv is enabled.
@@ -48,16 +50,26 @@ MP_MAIN_FILE = os.environ.get('MP_MAIN_FILE') or None
 WORKER_DIRECT_EXCHANGE = Exchange('C.dq')
 
 #: Format for worker direct queue names.
-WORKER_DIRECT_QUEUE_FORMAT = '%s.dq'
+WORKER_DIRECT_QUEUE_FORMAT = '{hostname}.dq'
+
+#: Separator for worker node name and hostname.
+NODENAME_SEP = '@'
 
 
 def worker_direct(hostname):
+    """Return :class:`kombu.Queue` that is a direct route to
+    a worker by hostname.
+
+    :param hostname: The fully qualified node name of a worker
+                     (e.g. ``w1@example.com``).  If passed a
+                     :class:`kombu.Queue` instance it will simply return
+                     that instead.
+    """
     if isinstance(hostname, Queue):
         return hostname
-    return Queue(WORKER_DIRECT_QUEUE_FORMAT % hostname,
+    return Queue(WORKER_DIRECT_QUEUE_FORMAT.format(hostname=hostname),
                  WORKER_DIRECT_EXCHANGE,
-                 hostname,
-                 auto_delete=True)
+                 hostname, auto_delete=True)
 
 
 def warn_deprecated(description=None, deprecation=None,
@@ -66,15 +78,26 @@ def warn_deprecated(description=None, deprecation=None,
            'deprecation': deprecation, 'removal': removal,
            'alternative': alternative}
     if deprecation is not None:
-        w = CPendingDeprecationWarning(PENDING_DEPRECATION_FMT % ctx)
+        w = CPendingDeprecationWarning(PENDING_DEPRECATION_FMT.format(**ctx))
     else:
-        w = CDeprecationWarning(DEPRECATION_FMT % ctx)
+        w = CDeprecationWarning(DEPRECATION_FMT.format(**ctx))
     warnings.warn(w)
 
 
 def deprecated(description=None, deprecation=None,
                removal=None, alternative=None):
+    """Decorator for deprecated functions.
 
+    A deprecation warning will be emitted when the function is called.
+
+    :keyword description: Description of what is being deprecated.
+    :keyword deprecation: Version that marks first deprecation, if this
+      argument is not set a ``PendingDeprecationWarning`` will be emitted
+      instead.
+    :keyword removed:  Future version when this feature will be removed.
+    :keyword alternative:  Instructions for an alternative solution (if any).
+
+    """
     def _inner(fun):
 
         @wraps(fun)
@@ -94,7 +117,7 @@ def lpmerge(L, R):
 
     Keeps values from `L`, if the value in `R` is :const:`None`."""
     set = L.__setitem__
-    [set(k, v) for k, v in R.iteritems() if v is not None]
+    [set(k, v) for k, v in items(R) if v is not None]
     return L
 
 
@@ -128,20 +151,20 @@ def fun_takes_kwargs(fun, kwlist=[]):
         ['logfile', 'loglevel', 'task_id']
 
     """
-    argspec = getattr(fun, 'argspec', getargspec(fun))
-    args, _varargs, keywords, _defaults = argspec
-    if keywords is not None:
+    S = getattr(fun, 'argspec', getargspec(fun))
+    if S.keywords is not None:
         return kwlist
-    return [kw for kw in kwlist if kw in args]
+    return [kw for kw in kwlist if kw in S.args]
 
 
 def isatty(fh):
-    # Fixes bug with mod_wsgi:
-    #   mod_wsgi.Log object has no attribute isatty.
-    return getattr(fh, 'isatty', None) and fh.isatty()
+    try:
+        return fh.isatty()
+    except AttributeError:
+        pass
 
 
-def cry():  # pragma: no cover
+def cry(out=None, sepchr='=', seplen=49):  # pragma: no cover
     """Return stacktrace of all active threads.
 
     From https://gist.github.com/737056
@@ -149,41 +172,37 @@ def cry():  # pragma: no cover
     """
     import threading
 
-    tmap = {}
-    main_thread = None
+    out = StringIO() if out is None else out
+    P = partial(print, file=out)
+
     # get a map of threads by their ID so we can print their names
     # during the traceback dump
-    for t in threading.enumerate():
-        if getattr(t, 'ident', None):
-            tmap[t.ident] = t
-        else:
-            main_thread = t
+    tmap = dict((t.ident, t) for t in threading.enumerate())
 
-    out = StringIO()
-    sep = '=' * 49 + '\n'
-    for tid, frame in sys._current_frames().iteritems():
-        thread = tmap.get(tid, main_thread)
+    sep = sepchr * seplen
+    for tid, frame in items(sys._current_frames()):
+        thread = tmap.get(tid)
         if not thread:
             # skip old junk (left-overs from a fork)
             continue
-        out.write('%s\n' % (thread.getName(), ))
-        out.write(sep)
+        P('{0.name}'.format(thread))
+        P(sep)
         traceback.print_stack(frame, file=out)
-        out.write(sep)
-        out.write('LOCAL VARIABLES\n')
-        out.write(sep)
+        P(sep)
+        P('LOCAL VARIABLES')
+        P(sep)
         pprint(frame.f_locals, stream=out)
-        out.write('\n\n')
+        P('\n')
     return out.getvalue()
 
 
 def maybe_reraise():
-    """Reraise if an exception is currently being handled, or return
+    """Re-raise if an exception is currently being handled, or return
     otherwise."""
     exc_info = sys.exc_info()
     try:
         if exc_info[2]:
-            raise exc_info[0], exc_info[1], exc_info[2]
+            reraise(exc_info[0], exc_info[1], exc_info[2])
     finally:
         # see http://docs.python.org/library/sys.html#sys.exc_info
         del(exc_info)
@@ -192,24 +211,39 @@ def maybe_reraise():
 def strtobool(term, table={'false': False, 'no': False, '0': False,
                            'true': True, 'yes': True, '1': True,
                            'on': True, 'off': False}):
-    if isinstance(term, basestring):
+    """Convert common terms for true/false to bool
+    (true/false/yes/no/on/off/1/0)."""
+    if isinstance(term, string_t):
         try:
             return table[term.lower()]
         except KeyError:
-            raise TypeError('Cannot coerce %r to type bool' % (term, ))
+            raise TypeError('Cannot coerce {0!r} to type bool'.format(term))
     return term
 
 
-def jsonify(obj):
-    "Transforms object making it suitable for json serialization"
-    if isinstance(obj, (int, float, basestring, types.NoneType)):
+def jsonify(obj,
+            builtin_types=(int, float, string_t), key=None,
+            keyfilter=None,
+            unknown_type_filter=None):
+    """Transforms object making it suitable for json serialization"""
+    from kombu.abstract import Object as KombuDictType
+    _jsonify = partial(jsonify, builtin_types=builtin_types, key=key,
+                       keyfilter=keyfilter,
+                       unknown_type_filter=unknown_type_filter)
+
+    if isinstance(obj, KombuDictType):
+        obj = obj.as_dict(recurse=True)
+
+    if obj is None or isinstance(obj, builtin_types):
         return obj
     elif isinstance(obj, (tuple, list)):
-        return [jsonify(v) for v in obj]
+        return [_jsonify(v) for v in obj]
     elif isinstance(obj, dict):
-        return dict((k, jsonify(v)) for k, v in obj.iteritems())
-    # See "Date Time String Format" in the ECMA-262 specification.
+        return dict((k, _jsonify(v, key=k))
+                    for k, v in items(obj)
+                    if (keyfilter(k) if keyfilter else 1))
     elif isinstance(obj, datetime.datetime):
+        # See "Date Time String Format" in the ECMA-262 specification.
         r = obj.isoformat()
         if obj.microsecond:
             r = r[:23] + r[26:]
@@ -226,10 +260,15 @@ def jsonify(obj):
     elif isinstance(obj, datetime.timedelta):
         return str(obj)
     else:
-        raise ValueError("Unsupported type: %s" % type(obj))
+        if unknown_type_filter is None:
+            raise ValueError(
+                'Unsupported type: {0!r} {1!r} (parent: {2})'.format(
+                    type(obj), obj, key))
+        return unknown_type_filter(obj)
 
 
 def gen_task_name(app, name, module_name):
+    """Generate task name from name/module pair."""
     try:
         module = sys.modules[module_name]
     except KeyError:
@@ -248,6 +287,20 @@ def gen_task_name(app, name, module_name):
         return '.'.join([app.main, name])
     return '.'.join(p for p in (module_name, name) if p)
 
+
+def nodename(name, hostname):
+    """Create node name from name/hostname pair."""
+    return NODENAME_SEP.join((name, hostname))
+
+
+def nodesplit(nodename):
+    """Split node name into tuple of name/hostname."""
+    parts = nodename.split(NODENAME_SEP, 1)
+    if len(parts) == 1:
+        return None, parts[0]
+    return parts
+
+
 # ------------------------------------------------------------------------ #
 # > XXX Compat
 from .log import LOG_LEVELS     # noqa
