@@ -6,7 +6,7 @@
     Logging utilities.
 
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
 import logging
 import os
@@ -14,14 +14,22 @@ import sys
 import threading
 import traceback
 
+from contextlib import contextmanager
 from billiard import current_process, util as mputil
 from kombu.log import get_logger as _get_logger, LOG_LEVELS
+from kombu.utils.encoding import safe_str
+
+from celery.five import string_t, text_t
 
-from .encoding import safe_str, str_t
 from .term import colored
 
+__all__ = ['ColorFormatter', 'LoggingProxy', 'base_logger',
+           'set_in_sighandler', 'in_sighandler', 'get_logger',
+           'get_task_logger', 'mlevel', 'ensure_process_aware_logger',
+           'get_multiprocessing_logger', 'reset_multiprocessing_logger']
+
 _process_aware = False
-is_py3k = sys.version_info[0] == 3
+PY3 = sys.version_info[0] == 3
 
 MP_LOG = os.environ.get('MP_LOG', False)
 
@@ -34,25 +42,51 @@ MP_LOG = os.environ.get('MP_LOG', False)
 base_logger = logger = _get_logger('celery')
 mp_logger = _get_logger('multiprocessing')
 
-in_sighandler = False
+_in_sighandler = False
 
 
 def set_in_sighandler(value):
-    global in_sighandler
-    in_sighandler = value
+    global _in_sighandler
+    _in_sighandler = value
+
+
+@contextmanager
+def in_sighandler():
+    set_in_sighandler(True)
+    try:
+        yield
+    finally:
+        set_in_sighandler(False)
+
+
+def logger_isa(l, p):
+    this, seen = l, set()
+    while this:
+        if this == p:
+            return True
+        else:
+            if this in seen:
+                raise RuntimeError(
+                    'Logger {0!r} parents recursive'.format(l),
+                )
+            seen.add(this)
+            this = this.parent
+    return False
 
 
 def get_logger(name):
     l = _get_logger(name)
     if logging.root not in (l, l.parent) and l is not base_logger:
-        l.parent = base_logger
+        if not logger_isa(l, base_logger):
+            l.parent = base_logger
     return l
 task_logger = get_logger('celery.task')
+worker_logger = get_logger('celery.worker')
 
 
 def get_task_logger(name):
     logger = get_logger(name)
-    if logger.parent is logging.root:
+    if not logger_isa(logger, task_logger):
         logger.parent = task_logger
     return logger
 
@@ -77,37 +111,35 @@ class ColorFormatter(logging.Formatter):
         if ei and not isinstance(ei, tuple):
             ei = sys.exc_info()
         r = logging.Formatter.formatException(self, ei)
-        if isinstance(r, str) and not is_py3k:
+        if isinstance(r, str) and not PY3:
             return safe_str(r)
         return r
 
     def format(self, record):
-        levelname = record.levelname
-        color = self.colors.get(levelname)
+        sformat = logging.Formatter.format
+        color = self.colors.get(record.levelname)
 
-        if self.use_color and color:
+        if color and self.use_color:
+            msg = record.msg
             try:
-                msg = record.msg
                 # safe_str will repr the color object
                 # and color will break on non-string objects
                 # so need to reorder calls based on type.
                 # Issue #427
-                if isinstance(msg, basestring):
-                    record.msg = str_t(color(safe_str(msg)))
-                else:
-                    record.msg = safe_str(color(msg))
-            except Exception, exc:
-                record.msg = '<Unrepresentable %r: %r>' % (
-                    type(record.msg), exc)
+                try:
+                    if isinstance(msg, string_t):
+                        record.msg = text_t(color(safe_str(msg)))
+                    else:
+                        record.msg = safe_str(color(msg))
+                except UnicodeDecodeError:
+                    record.msg = safe_str(msg)  # skip colors
+            except Exception as exc:
+                record.msg = '<Unrepresentable {0!r}: {1!r}>'.format(
+                    type(msg), exc)
                 record.exc_info = True
-
-        if not is_py3k and 'processName' not in record.__dict__:
-            # Very ugly, but have to make sure processName is supported
-            # by foreign logger instances.
-            # (processName is always supported by Python 2.7)
-            process_name = current_process and current_process()._name or ''
-            record.__dict__['processName'] = process_name
-        return safe_str(logging.Formatter.format(self, record))
+            return sformat(self, record)
+        else:
+            return safe_str(sformat(self, record))
 
 
 class LoggingProxy(object):
@@ -151,13 +183,12 @@ class LoggingProxy(object):
                         del(exc_info)
 
             handler.handleError = WithSafeHandleError().handleError
-
         return [wrap_handler(h) for h in self.logger.handlers]
 
     def write(self, data):
         """Write message to logging object."""
-        if in_sighandler:
-            return sys.__stderr__.write(safe_str(data))
+        if _in_sighandler:
+            return print(safe_str(data), file=sys.__stderr__)
         if getattr(self._thread, 'recurse_protection', False):
             # Logger is logging back to this file, so stop recursing.
             return
@@ -192,7 +223,7 @@ class LoggingProxy(object):
         self.closed = True
 
     def isatty(self):
-        """Always returns :const:`False`. Just here for file support."""
+        """Always return :const:`False`. Just here for file support."""
         return False
 
 
@@ -240,8 +271,7 @@ def _patch_logger_class():
                 _signal_safe = True
 
                 def log(self, *args, **kwargs):
-                    if in_sighandler:
-                        sys.__stderr__.write('CANNOT LOG IN SIGHANDLER')
+                    if _in_sighandler:
                         return
                     return OldLoggerClass.log(self, *args, **kwargs)
             logging.setLoggerClass(SigSafeLogger)
