@@ -2,15 +2,15 @@ from __future__ import absolute_import
 
 from celery import Celery
 
-celery = Celery('proj.celery',
-                broker='amqp://',
-                backend='amqp://',
-                include=['proj.tasks'])
+app = Celery('proj.celery',
+             broker='amqp://',
+             backend='amqp://',
+             include=['proj.tasks'])
 
 # Optional configuration, see the application user guide.
-celery.conf.update(
+app.conf.update(
     CELERY_TASK_RESULT_EXPIRES=3600,
 )
 
 if __name__ == '__main__':
-    celery.start()
+    app.start()
